@@ -3,7 +3,7 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2003-01-07
+ *  Updated   : 2003-06-29
  *  Notes     :
  *
  * Copyright (c) 1997-2003 Urs Janssen <urs@tin.org>
@@ -38,10 +38,6 @@
 #ifndef PROTO_H
 #	define PROTO_H 1
 
-#	ifndef RFC2046_H
-#		include <rfc2046.h>
-#	endif /* !RFC2046_H */
-
 /* This fixes ambiguities on platforms that don't distinguish extern case */
 #ifdef CASE_PROBLEM
 #	define Raw tin_raw
@@ -50,6 +46,7 @@
 
 /* active.c */
 extern char group_flag(char ch);
+extern int find_newnews_index(const char *cur_newnews_host);
 extern int get_active_num(void);
 extern t_bool match_group_list(const char *group, const char *group_list);
 extern t_bool parse_active_line(char *line, long *max, long *min, char *moderated);
@@ -61,19 +58,18 @@ extern void load_newnews_info(char *info);
 extern void read_news_active_file(void);
 
 /* art.c */
-extern char *find_nov_file(struct t_group *group, int mode);
 extern int global_get_multipart_info(int aindex, MultiPartInfo *setme);
 extern t_bool index_group(struct t_group *group);
 extern void do_update(t_bool catchup);
 extern void find_base(struct t_group *group);
 extern void make_threads(struct t_group *group, t_bool rethread);
 extern void set_article(struct t_article *art);
-extern void show_art_msg(char *group);
+extern void show_art_msg(const char *group);
 extern void sort_arts(unsigned /* int */ sort_art_type);
 extern void write_nov_file(struct t_group *group);
 
 /* attrib.c */
-extern void read_attributes_file(t_bool global_file);
+extern void read_attributes_files(void);
 extern void write_attributes_file(const char *file);
 
 /* auth.c */
@@ -112,23 +108,24 @@ extern t_bool match_long(char *line, const char *pat, long *dst);
 extern t_bool match_string(char *line, const char *pat, char *dst, size_t dstlen);
 extern t_bool read_config_file(char *file, t_bool global_file);
 extern void quote_dash_to_space(char *str);
+extern void read_server_config(void);
 extern void refresh_config_page(int act_option);
 extern void show_menu_help(const char *help_message);
 extern void write_config_file(char *file);
 
 /* cook.c */
 extern const char *get_filename(t_param *ptr);
-extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int tabs, t_bool uue);
-extern t_bool expand_ctrl_chars(char *to, const char *from, int length, size_t cook_width);
+extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int tabs, int hide_uue);
+extern t_bool expand_ctrl_chars(char **line, int *length, size_t cook_width);
 
 /* curses.c */
 extern OUTC_RETTYPE outchar(OUTC_ARGS);
 extern int InitScreen(void);
 extern int RawState(void);
 extern int ReadCh(void);
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && !defined(USE_CURSES)
 	extern wint_t ReadWch(void);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE && !USE_CURSES */
 extern int get_arrow_key(int prech);
 extern int get_termcaps(void);
 extern void ClearScreen(void);
@@ -144,7 +141,6 @@ extern void InitWin(void);
 #endif /* USE_CURSES */
 extern void Raw(int state);
 extern void StartInverse(void);
-extern void ToggleInverse(void);
 extern void cursoroff(void);
 extern void cursoron(void);
 extern void highlight_string(int row, int col, int size);
@@ -154,6 +150,9 @@ extern void set_xclick_off(void);
 extern void set_xclick_on(void);
 extern void setup_screen(void);
 extern void word_highlight_string(int row, int col, int size, int color);
+#if 0
+	extern void ToggleInverse(void);
+#endif /* 0 */
 
 /* debug.c */
 #ifdef DEBUG
@@ -163,7 +162,6 @@ extern void word_highlight_string(int row, int col, int size, int color);
 	extern void debug_print_arts(void);
 	extern void debug_print_filters(void);
 	extern void debug_print_header(struct t_article *s);
-	extern void debug_save_comp(void);
 	extern void vDbgPrintMalloc(int iIsMalloc, const char *pcFile, int iLine, size_t iSize);
 #endif /* DEBUG */
 #ifdef DEBUG_NEWSRC
@@ -181,7 +179,6 @@ extern void envargs(int *Pargc, char ***Pargv, const char *envstr);
 extern void feed_articles(int function, int level, struct t_group *group, int respnum);
 
 /* filter.c */
-extern int unfilter_articles(void);
 extern t_bool filter_articles(struct t_group *group);
 extern t_bool filter_menu(int type, struct t_group *group, struct t_article *art);
 extern t_bool quick_filter(int type, struct t_group *group, struct t_article *art);
@@ -189,6 +186,7 @@ extern t_bool quick_filter_select_posted_art(struct t_group *group, const char *
 extern t_bool read_filter_file(const char *file);
 extern void free_filter_array(struct t_filters *ptr);
 extern void refresh_filter_menu(void);
+extern void unfilter_articles(void);
 extern void write_filter_file(const char *filename);
 
 /* getline.c */
@@ -225,11 +223,6 @@ extern void hash_reclaim(void);
 extern void show_help_page(const int level, const char *title);
 extern void show_mini_help(int level);
 extern void toggle_mini_help(int level);
-#ifdef USE_CURSES
-	extern void display_help_page(t_bool first);
-#else
-	extern void display_help_page(void);
-#endif /* USE_CURSES */
 
 /* header.c */
 extern const char *get_domain_name(void);
@@ -246,16 +239,11 @@ extern void get_user_info(char *user_name, char *full_name);
 
 /* init.c */
 extern t_bool(*wildcard_func)(const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
-extern t_bool create_mail_save_dirs(void);
 extern void init_selfinfo(void);
 extern void postinit_regexp(void);
-extern void set_up_private_index_cache(void);
 #ifdef HAVE_COLOR
 	extern void postinit_colors(void);
 #endif /* HAVE_COLOR */
-#ifdef USE_INN_NNTPLIB
-	extern char *GetConfigValue(const char *name);
-#endif /* USE_INN_NNTPLIB */
 
 /* joinpath.c */
 extern void joinpath(char *result, const char *dir, const char *file);
@@ -288,14 +276,13 @@ extern t_bool dot_unlock(const char *filename);
 #endif /* 0 */
 
 /* mail.c */
-extern t_bool art_edit(struct t_group *psGrp, struct t_article *psArt);
-extern void find_art_max_min(char *group_path, long *art_max, long *art_min);
-extern void make_base_group_path(char *base_dir, char *group_name, char *group_path);
-extern void print_active_head(char *active_file);
-extern void print_group_line(FILE *fp, char *group_name, long art_max, long art_min, char *base_dir);
+extern t_bool art_edit(struct t_group *group, struct t_article *article);
+extern void find_art_max_min(const char *group_path, long *art_max, long *art_min);
+extern void print_active_head(const char *active_file);
+extern void print_group_line(FILE *fp, const char *group_name, long art_max, long art_min, const char *base_dir);
 extern void read_descriptions(t_bool verb);
-extern void vGrpDelMailArts(struct t_group *psGrp);
-extern void vGrpDelMailArt(struct t_article *psArt);
+extern void grp_del_mail_arts(struct t_group *group);
+extern void grp_del_mail_art(struct t_article *article);
 #ifdef HAVE_MH_MAIL_HANDLING
 	extern void read_mail_active_file(void);
 	extern void write_mail_active_file(void);
@@ -303,6 +290,7 @@ extern void vGrpDelMailArt(struct t_article *psArt);
 
 /* mimetypes.c */
 extern void lookup_mimetype(const char *ext, t_part *part);
+extern t_bool lookup_extension(char *extension, size_t ext_len, const char *major, const char *minor);
 
 /* main.c */
 extern int main(int argc, char *argv[]);
@@ -318,6 +306,7 @@ extern void init_alloc(void);
 extern void free_all_arrays(void);
 extern void free_art_array(void);
 extern void free_attributes_array(void);
+extern void free_if_not_default(char **attrib, char *deflt);
 extern void free_save_array(void);
 extern void *my_malloc1(const char *file, int line, size_t size);
 extern void *my_calloc1(const char *file, int line, size_t nmemb, size_t size);
@@ -342,7 +331,8 @@ extern int gnksa_do_check_from(const char *from, char *address, char *realname);
 extern int my_chdir(char *path);
 extern int my_isprint(int c);
 extern int my_mkdir(char *path, mode_t mode);
-extern int strfmailer(char *the_mailer, char *subject, char *to, const char *filename, char *s, size_t maxsize, char *format);
+extern int parse_from(const char *from, char *address, char *realname);
+extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
 extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group);
 extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
 extern long file_mtime(const char *file);
@@ -354,33 +344,32 @@ extern t_bool invoke_editor(const char *filename, int lineno);
 extern t_bool mail_check(void);
 extern void append_file(char *old_filename, char *new_filename);
 extern void asfail(const char *file, int line, const char *cond);
-extern void base_name(char *fullpath, char *program);
+extern void base_name(const char *fullpath, char *program);
 extern void cleanup_tmp_files(void);
 extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, char *initl, t_bool raw_data);
 extern void create_index_lock_file(char *the_lock_file);
+extern void dir_name(const char *fullpath, char *dir);
 extern void draw_percent_mark(long cur_num, long max_num);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
-extern void make_group_path(char *name, char *path);
+extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path);
+extern void make_group_path(const char *name, char *path);
+extern void process_charsets(char **line, int *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
 extern void read_input_history_file(void);
 extern void rename_file(const char *old_filename, const char *new_filename);
 extern void show_inverse_video_status(void);
-extern void strip_double_ngs(char *ngs_list);
-extern void strip_name(char *the_address, char *stripped_address);
+extern void strip_name(const char *from, char *address);
 extern void tin_done(int ret);
 extern void toggle_inverse_video(void);
-extern int parse_from(const char *from, char *address, char *realname);
-#if defined(LOCAL_CHARSET) || defined(CHARSET_CONVERSION)
-	extern void buffer_to_network(char *line, int mmnwcharset);
-#endif /* LOCAL_CHARSET || CHARSET_CONVERSION */
-
-extern void process_charsets(char **line, int *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
+#ifdef CHARSET_CONVERSION
+	extern t_bool buffer_to_network(char *line, int mmnwcharset);
+#endif /* CHARSET_CONVERSION */
 #ifdef HAVE_COLOR
 	extern t_bool toggle_color(void);
 	extern void show_color_status(void);
 #endif /* HAVE_COLOR */
 #ifdef HAVE_ISPELL
-	extern t_bool invoke_ispell(const char *nam, struct t_group *psGrp);
+	extern t_bool invoke_ispell(const char *nam, struct t_group *group);
 #endif /* HAVE_ISPELL */
 #ifndef M_UNIX
 	extern void make_post_process_cmd(char *cmd, char *dir, char *file);
@@ -391,21 +380,19 @@ extern void process_charsets(char **line, int *max_line_len, const char *network
 #endif /* !NO_SHELL_ESCAPE */
 
 /* newsrc.c */
-extern int pos_group_in_newsrc(struct t_group *group, int pos);
 extern signed long int read_newsrc(char *newsrc_file, t_bool allgroups);
 extern signed long int write_newsrc(void);
-extern void art_mark_read(struct t_group *group, struct t_article *art);
-extern void art_mark_unread(struct t_group *group, struct t_article *art);
-extern void art_mark_will_return(struct t_group *group, struct t_article *art);
+extern t_bool pos_group_in_newsrc(struct t_group *group, int pos);
+extern void art_mark(struct t_group *group, struct t_article *art, int flag);
 extern void backup_newsrc(void);
 extern void catchup_newsrc_file(void);
 extern void delete_group(char *group);
 extern void expand_bitmap(struct t_group *group, long min);
-extern void grp_mark_read(struct t_group *group, struct t_article *psArt);
+extern void grp_mark_read(struct t_group *group, struct t_article *art);
 extern void grp_mark_unread(struct t_group *group);
 extern void parse_unread_arts(struct t_group *group);
 extern void reset_newsrc(void);
-extern void subscribe(struct t_group *group, int sub_state);
+extern void subscribe(struct t_group *group, int sub_state, t_bool get_info);
 extern void thd_mark_read(struct t_group *group, long thread);
 extern void thd_mark_unread(struct t_group *group, long thread);
 extern void set_default_bitmap(struct t_group *group);
@@ -433,20 +420,18 @@ extern int get_newsrcname(char *newsrc_name, const char *nntpserver_name);
 extern void get_nntpserver(char *nntpserver_name, char *nick_name);
 
 /* open.c */
-extern FILE *open_art_fp(const char *group_path, long art);
+extern FILE *open_art_fp(struct t_group *group, long art);
 extern FILE *open_newgroups_fp(int the_index);
 extern FILE *open_news_active_fp(void);
 extern FILE *open_newsgroups_fp(void);
 extern FILE *open_overview_fmt_fp(void);
 extern FILE *open_subscription_fp(void);
-extern FILE *open_xover_fp(struct t_group *psGrp, const char *mode, long lMin, long lMax);
 extern FILE *open_art_header(long art);
 extern int get_respcode(char *, size_t);
 extern int get_only_respcode(char *, size_t);
 extern int nntp_open(void);
 extern int group_get_art_info(char *tin_spooldir, char *groupname, int grouptype, long *art_count, long *art_max, long *art_min);
-extern long setup_hard_base(struct t_group *group, const char *group_path);
-extern t_bool stat_article(long art, const char *group_path);
+extern long setup_hard_base(struct t_group *group);
 extern void nntp_close(void);
 extern void vGet1GrpArtInfo(struct t_group *grp);
 #ifdef HAVE_MH_MAIL_HANDLING
@@ -547,16 +532,11 @@ extern void free_mailcap(t_mailcap *tmailcap);
 
 /* rfc2045.c */
 extern void rfc1521_encode(char *line, FILE *f, int e);
-extern void rfc1557_encode(char *line, FILE *f, int e);
-#if 0
-	extern void rfc1468_encode(char *line, FILE *f, int e);
-	extern void rfc1922_encode(char *line, FILE *f, int e);
-#endif /* 0 */
 
 /* rfc2046.c */
 extern const char *get_param(t_param *list, const char *name);
-extern char *parse_header(char *buf, const char *pat, t_bool decode);
-extern int art_open(t_bool wrap_lines, struct t_article *art, const char *group_path, t_openartinfo *artinfo, t_bool show_progress_meter);
+extern char *parse_header(char *buf, const char *pat, t_bool decode, t_bool structured);
+extern int art_open(t_bool wrap_lines, struct t_article *art, struct t_group *group, t_openartinfo *artinfo, t_bool show_progress_meter);
 extern int content_type(char *type);
 extern int parse_rfc822_headers(struct t_header *hdr, FILE *from, FILE *to);
 extern t_part *new_part(t_part *part);
@@ -573,14 +553,12 @@ extern void rfc15211522_encode(const char *filename, constext *mime_encoding, st
 
 /* save.c */
 extern int check_start_save_any_news(int function, t_bool catchup);
-extern t_bool add_to_save_list(struct t_article *art, const char *path);
 extern t_bool create_path(const char *path);
+extern t_bool expand_save_filename(char *outpath, const char *path);
 extern t_bool post_process_files(int proc_type_ch, t_bool auto_delete);
-extern t_bool save_art_to_file(int indexnum, t_openartinfo *artinfo);
-extern t_bool save_batch(char type, const char *group_path);
+extern t_bool save_and_process_art(t_openartinfo *artinfo, struct t_article *artptr, t_bool is_mailbox, const char *inpath, int max, t_bool post_process);
 extern void decode_save_mime(t_openartinfo *art, t_bool postproc);
 extern void print_art_seperator_line(FILE *fp, t_bool is_mailbox);
-extern void sort_save_list(void);
 
 /* screen.c */
 extern void center_line(int line, t_bool inverse, const char *str);
@@ -592,7 +570,7 @@ extern void info_message(const char *fmt, ...);
 extern void perror_message(const char *fmt, ...);
 extern void ring_bell(void);
 extern void show_progress(const char *txt, long count, long total);
-extern void show_title(char *title);
+extern void show_title(const char *title);
 extern void spin_cursor(void);
 extern void stow_cursor(void);
 extern void wait_message(unsigned int sdelay, const char *fmt, ...);
@@ -603,7 +581,7 @@ extern int search(int key, int current_art, t_bool forward, t_bool repeat);
 extern int search_active(t_bool forward, t_bool repeat);
 extern int search_article(t_bool forward, t_bool repeat, int start_line, int lines, t_lineinfo *line, int reveal_ctrl_l_lines, FILE *fp);
 extern int search_config(t_bool forward, t_bool repeat, int current, int last);
-extern int search_body(int current_art, t_bool repeat);
+extern int search_body(struct t_group *group, int current_art, t_bool repeat);
 
 /* select.c */
 extern int add_my_group(const char *group, t_bool add);
@@ -672,19 +650,22 @@ extern void str_lwr(char *str);
 #	define strrstr(s,p)	my_strrstr(s,p)
 #endif /* !HAVE_STRRSTR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	extern void wcspart(wchar_t *to, const wchar_t *from, int columns, int size_to);
+	extern void wcspart(wchar_t *to, const wchar_t *from, int columns, int size_to, t_bool pad);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* tags.c */
 extern int line_is_tagged(int n);
+extern int mark_tagged_read(struct t_group *group);
 extern int tag_multipart(int base_index);
+extern t_bool arts_selected(void);
+extern t_bool got_tagged_unread_arts(void);
 extern t_bool set_range(int level, int min, int max, int curr);
 extern t_bool tag_article(int art);
 extern t_bool untag_all_articles(void);
 extern void do_auto_select_arts(void);
-extern void remove_tag(long art);
 extern void undo_auto_select_arts(void);
 extern void undo_selections(void);
+extern void untag_article(long art);
 
 /* tmpfile.c */
 #ifndef HAVE_TMPFILE
@@ -711,10 +692,24 @@ extern int thread_page(struct t_group *group, int respnum, int thread_depth, t_p
 extern void draw_line(int i, int magic);
 extern void fixup_thread(int respnum, t_bool redraw);
 
+/* version.c */
+extern int check_upgrade(char *line, const char *skip, const char *version);
+extern void upgrade_prompt_quit(int reason, const char *file);
+
 /* wildmat.c */
 extern t_bool wildmat(const char *text, char *p, t_bool icase);
 extern t_bool wildmatpos(const char *text, char *p, t_bool icase, int *srch_offsets, int srch_offsets_size);
 
+/* xface.c */
+#ifdef XFACE_ABLE
+	extern void slrnface_stop(void);
+	extern void slrnface_start(void);
+	extern void slrnface_display_xface(char *face);
+	extern void slrnface_clear_xface(void);
+	extern void slrnface_suppress_xface(void);
+	extern void slrnface_show_xface(void);
+#endif /* XFACE_ABLE */
+
 /* xref.c */
 extern t_bool overview_xref_support(void);
 extern void NSETRNG0(t_bitmap *bitmap, long low, long high);
