@@ -3,7 +3,7 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-16
+ *  Updated   : 2003-07-28
  *  Notes     : #include files, #defines & struct's
  *
  * Copyright (c) 1997-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -117,6 +117,7 @@
 
 enum context { cMain, cArt, cConfig, cFilter, cGroup, cHelp, cInfopager, cPage, cSelect, cThread };
 enum resizer { cNo, cYes, cRedraw };
+enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 #ifdef VMS
 #	ifdef __DECC
@@ -180,8 +181,10 @@ enum resizer { cNo, cYes, cRedraw };
 #else
 #	ifdef HAVE_SYS_ERRNO_H
 #		include	<sys/errno.h>
-#	else
-#		error "No errno.h or sys/errno.h found"
+/*
+	#	else
+	#		error "No errno.h or sys/errno.h found"
+*/
 #	endif /* HAVE_SYS_ERRNO_H */
 #endif /* HAVE_ERRNO_H */
 #if !defined(errno)
@@ -268,6 +271,17 @@ enum resizer { cNo, cYes, cRedraw };
 #	endif /* HAVE_STRINGS_H */
 #endif /* !__QNX__ */
 
+/*
+ * FIXME: make this autoconf
+ */
+#ifdef SEIUX
+#	include <bsd/sys/time.h>
+#	include <bsd/sys/signal.h>
+#	include <bsd/sys/types.h>
+#	include <posix/unistd.h>
+#	include <bsd/netinet/in.h>
+#endif /* SEIUX */
+
 #ifdef HAVE_FCNTL_H
 #	include <fcntl.h>
 #endif /* HAVE_FCNTL_H */
@@ -316,9 +330,13 @@ enum resizer { cNo, cYes, cRedraw };
 #endif /* HAVE_SYS_WAIT_H */
 
 #ifndef WEXITSTATUS
-#	define WEXITSTATUS(status)		((int) (((status) >> 8) & 0xFF))
+#	define WEXITSTATUS(status)	((int) (((status) >> 8) & 0xFF))
 #endif /* !WEXITSTATUS */
 
+#ifndef WIFEXITED
+#	define WIFEXITED(status)	((int) (((status) & 0xFF) == 0))
+#endif /* !WIFEXITED */
+
 /*
  * Needed for timeout in user abort of indexing a group (BSD & SYSV variaties)
  */
@@ -437,6 +455,13 @@ enum resizer { cNo, cYes, cRedraw };
 #	ifndef NNTP_INEWS
 #		define NNTP_INEWS	1
 #	endif /* !NNTP_INEWS */
+#else
+#	ifdef XHDR_XREF
+#		undef XHDR_XREF
+#	endif /* XHDR_XREF */
+#	ifdef BROKEN_LISTGROUP
+#		undef BROKEN_LISTGROUP
+#	endif /* BROKEN_LISTGROUP */
 #endif /* NNTP_ABLE || NNTP_ONLY */
 
 #define FAKE_NNTP_FP		(FILE *) 9999
@@ -598,7 +623,7 @@ enum resizer { cNo, cYes, cRedraw };
 #endif /* !DEFAULT_MAILBOX */
 
 
-/* FIXME: remove absolut-paths! */
+/* FIXME: remove absolute-paths! */
 /*
  * Miscellaneous program-paths
  */
@@ -606,9 +631,12 @@ enum resizer { cNo, cYes, cRedraw };
 #	define PATH_ISPELL	"ispell"
 #endif /* !PATH_ISPELL */
 
-#ifndef PATH_METAMAIL
+#ifndef PATH_METAMAIL	/* unly unset if !HAVE_METAMAIL */
 #	define PATH_METAMAIL	"metamail"
 #endif /* !PATH_METAMAIL */
+#define METAMAIL_CMD		PATH_METAMAIL" -e -p -m \"tin\""
+
+#define INTERNAL_CMD	"--internal"
 
 /*
  * Fix up the 'sum' path and parameter for './configure'd systems
@@ -625,11 +653,13 @@ enum resizer { cNo, cYes, cRedraw };
 #endif /* PATH_SUM */
 
 #ifdef HAVE_LONG_FILE_NAMES
-#	define LONG_PATH_PART	"part"
-#	define LONG_PATH_PATCH	"patch"
+#	define PATH_PART	"part"
+#	define PATH_PATCH	"patch"
+#	define INDEX_LOCK	"%stin.%s.LCK"
 #else
-#	define LONG_PATH_PART	""
-#	define LONG_PATH_PATCH	"p"
+#	define PATH_PART	""
+#	define PATH_PATCH	"p"
+#	define INDEX_LOCK	"%s%s.LCK"
 #endif /* HAVE_LONG_FILE_NAMES */
 
 /*
@@ -670,9 +700,12 @@ enum resizer { cNo, cYes, cRedraw };
 #define ACTIVE_TIMES_FILE	"active.times"
 #define ATTRIBUTES_FILE	"attributes"
 #define CONFIG_FILE	"tinrc"
+#define SERVERCONFIG_FILE	"serverrc"
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
 #define DEFAULT_URL_HANDLER "url_handler.sh"
+/* Prefixes saved attachments with no set filename */
+#define SAVEFILE_PREFIX		"unknown"
 
 
 /* MMDF-mailbox seperator */
@@ -759,7 +792,7 @@ enum resizer { cNo, cYes, cRedraw };
 
 #define FILTER_FILE	"filter"
 /* editor offset for filter-file; TODO: doesn't match for german filter-file */
-#define FILTER_FILE_OFFSET	32
+#define FILTER_FILE_OFFSET	27
 #define INPUT_HISTORY_FILE	".inputhistory"
 #ifdef HAVE_MH_MAIL_HANDLING
 #	define MAILGROUPS_FILE	"mailgroups"
@@ -769,15 +802,18 @@ enum resizer { cNo, cYes, cRedraw };
 /* ifdef APPEND_PID (default) NEWNEWSRC_FILE will be .newnewsrc<pid> */
 #define NEWNEWSRC_FILE	".newnewsrc"
 #define OLDNEWSRC_FILE	".oldnewsrc"
-#ifndef	OVERVIEW_FILE
+#ifndef OVERVIEW_FILE
 #	define OVERVIEW_FILE	".overview"
 #endif /* !OVERVIEW_FILE */
 #define OVERVIEW_FMT	"overview.fmt"
 #define POSTED_FILE	"posted"
 #define POSTPONED_FILE	"postponed.articles"
 #define SUBSCRIPTIONS_FILE	"subscriptions"
+#define NEWSGROUPS_FILE	"newsgroups"
+
 
 #ifdef USE_INN_NNTPLIB
+#	include <libinn.h> /* TODO: add configure chek for it */
 #	define _CONF_FROMHOST	"fromhost"
 #	define _CONF_ORGANIZATION	"organization"
 #	define _CONF_SERVER	"server"
@@ -792,12 +828,12 @@ enum resizer { cNo, cYes, cRedraw };
 /* Philip Hazel's Perl regular expressions library */
 #include	<pcre.h>
 
-#if defined(HAVE_ICONV) && !defined(LOCAL_CHARSET)
+#ifdef HAVE_ICONV
 #	define CHARSET_CONVERSION 1
 #	ifdef HAVE_ICONV_H
 #		include <iconv.h>
 #	endif /* HAVE_ICONV_H */
-#endif /* HAVE_ICONV && !LOCAL_CHARSET */
+#endif /* HAVE_ICONV */
 
 #ifdef HAVE_LANGINFO_H
 #	include <langinfo.h>
@@ -853,26 +889,56 @@ enum resizer { cNo, cYes, cRedraw };
 #define STRNCMPEQ(s1, s2, n)	(strncmp((s1), (s2), n) == 0)
 #define STRNCASECMPEQ(s1, s2, n)	(strncasecmp((s1), (s2), n) == 0)
 
+/*
+ * PATH_LEN    = max. path length (incl. terminating '\0')
+ * NAME_LEN    = max. filename length (not incl. terminating '\0')
+ * LEN         =
+ * HEADER_LEN  = max. size of a news/mail header-line
+ * NEWSRC_LINE =
+ */
 #if defined(VMS) || defined(M_AMIGA)
-#	define LEN	512
 #	define PATH_LEN	256
+#	define NAME_LEN	14
+#	define LEN	512
 #endif /* VMS || M_AMIGA */
 
 #ifdef M_UNIX
-#	ifndef MAXPATHLEN
-#		define MAXPATHLEN	256
-#	endif /* !MAXPATHLEN */
+#	ifdef PATH_MAX
+#		define PATH_LEN	PATH_MAX
+#	else
+#		ifdef MAXPATHLEN
+#			define PATH_LEN	MAXPATHLEN
+#		else
+#			ifdef _POSIX_PATH_MAX
+#				define PATH_LEN	_POSIX_PATH_MAX
+#			else
+#				define PATH_LEN	255
+#			endif /* _POSIX_PATH_MAX */
+#		endif /* MAXPATHLEN */
+#	endif /* PATH_MAX */
+#	ifdef HAVE_LONG_FILE_NAMES
+#		ifdef NAME_MAX
+#			define NAME_LEN	NAME_MAX
+#		else
+#			ifdef _POSIX_NAME_MAX
+#				define NAME_LEN	_POSIX_NAME_MAX
+#			else
+#				define NAME_LEN	14
+#			endif /* _POSIX_NAME_MAX */
+#		endif /* NAME_MAX */
+#	else
+#		define NAME_LEN	14
+#	endif /* HAVE_LONG_FILE_NAMES */
 #	define LEN	1024
-#	define PATH_LEN	MAXPATHLEN
 #endif /* M_UNIX */
 
 #define NEWSRC_LINE	8192
-
 #define HEADER_LEN	1024
 
-#define MODULO_COUNT_NUM	50
 #define TABLE_SIZE	1409
 
+#define MODULO_COUNT_NUM	50
+
 #define DAY	(60*60*24)		/* Seconds in a day */
 
 #define ctrl(c)	((c) & 0x1F)
@@ -942,13 +1008,13 @@ enum resizer { cNo, cYes, cRedraw };
  */
 #define NUM_MIME_ENCODINGS	4
 
-#define MIME_ENCODING_8BIT	0	/* not used */
+#define MIME_ENCODING_8BIT	0
 #define MIME_ENCODING_BASE64	1
 #define MIME_ENCODING_QP	2
 #define MIME_ENCODING_7BIT	3
 
 #ifdef CHARSET_CONVERSION			/* can/should do charset conversion via iconv() */
-#	define NUM_MIME_CHARSETS 28	/* # known 'outgoing' charsets */
+#	define NUM_MIME_CHARSETS 27	/* # known 'outgoing' charsets */
 #endif /* CHARSET_CONVERSION */
 
 #define NUM_MAILBOX_FORMATS 3		/* MBOX0, MBOXRD, MMDF */
@@ -964,6 +1030,11 @@ enum resizer { cNo, cYes, cRedraw };
 #define MAX_COLOR	15
 #define MAX_BACKCOLOR	7
 
+/*
+ * Count of available attributes for highlighting
+ */
+#define MAX_ATTR	6
+
 /*
  * Maximal permissible word mark type
  * 0 = nothing, 1 = mark, 2 = space
@@ -980,6 +1051,8 @@ enum resizer { cNo, cYes, cRedraw };
 #define REGEX_FMT (tinrc.wildcard ? "%s" : "*%s*")
 
 #define IGNORE_ART(i)	((tinrc.kill_level != KILL_THREAD && arts[i].killed) || (arts[i].thread == ART_EXPIRED))
+/* only used for threading */
+#define IGNORE_ART_THREAD(i)	(arts[i].thread != ART_NORMAL || (tinrc.kill_level == KILL_NOTHREAD && arts[i].killed))
 
 /*
  * Is this part text/plain ?
@@ -1014,8 +1087,6 @@ enum resizer { cNo, cYes, cRedraw };
  */
 #define PLURAL(x,y)			((x == 1) ? _(y##_singular) : _(y##_plural))
 
-#define POST_PROC_TYPE(x)	(ch_post_process[x])
-
 /*
  * News/Mail group types
  */
@@ -1050,8 +1121,9 @@ enum resizer { cNo, cYes, cRedraw };
 
 
 /*
- * used by feed_articles() & show_mini_help()
+ * used by feed_articles() & show_mini_help() & quick_filter & add_filter_rule
  */
+#define INTERNAL_LEVEL  0	/* do we need this? see comments in filter.c */
 #define SELECT_LEVEL	1
 #define GROUP_LEVEL	2
 #define THREAD_LEVEL	3
@@ -1064,7 +1136,7 @@ enum resizer { cNo, cYes, cRedraw };
 #define FEED_PIPE		2
 #define FEED_PRINT		3
 #define FEED_SAVE		4
-#define FEED_AUTOSAVE_TAGGED		5
+#define FEED_AUTOSAVE	5
 #define FEED_REPOST		6
 
 #if 0
@@ -1093,6 +1165,14 @@ enum resizer { cNo, cYes, cRedraw };
 #define SHOW_FROM_NAME		2
 #define SHOW_FROM_BOTH		3
 
+/*
+ * Values for show_infos
+ */
+#define SHOW_INFO_NOTHING	0
+#define SHOW_INFO_LINES		1
+#define SHOW_INFO_SCORE		2
+#define SHOW_INFO_BOTH		3
+
 /*
  * Values for thread_score
  */
@@ -1103,9 +1183,9 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * used in feed.c & save.c
  */
-#define POST_PROC_NONE		0
+#define POST_PROC_NO		0
 #define POST_PROC_SHAR		1
-#define POST_PROC_UUDECODE	2
+#define POST_PROC_YES		2
 
 /*
  * used in art.c
@@ -1147,7 +1227,7 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * Different extents to which we can hide killed articles
  */
-#define KILL_READ		0		/* Kill only read articles */
+#define KILL_UNREAD		0		/* Kill only unread articles */
 #define KILL_THREAD		1		/* Kill all articles and show as K */
 #define KILL_NOTHREAD	2		/* Kill all articles, never show them */
 
@@ -1176,6 +1256,14 @@ enum resizer { cNo, cYes, cRedraw };
 #define POSTED_OK		2			/* posted normally */
 
 
+/*
+ * used in pager
+ */
+#define UUE_NO			0		/* Don't hide uue data */
+#define UUE_YES			1		/* Hide uue data */
+#define UUE_ALL			2		/* Hide uue data harder */
+
+
 /*
  * index_point variable values used throughout tin
  */
@@ -1184,15 +1272,18 @@ enum resizer { cNo, cYes, cRedraw };
  * -1 is kind of overloaded as an error from which_thread() and other functions
  * where we wish to return to the next level up
  */
-#define GRP_RETURN		-1	/* Pager 'T' command only -> return to selection screen */
-#define GRP_QUIT		-2	/* Set by 'Q' when coming all the way out */
-#define GRP_NEXTUNREAD	-3	/* (After catchup) goto next unread item */
-#define GRP_NEXT		-4	/* (After catchup) move to next item */
-#define GRP_ARTFAIL		-5	/* show_page() only. Failed to get into the article */
-#define GRP_KILLED		-6	/* ?? Thread was killed at pager level */
-#define GRP_GOTOTHREAD	-7	/* show_page() only. Goto thread menu */
-#define GRP_ENTER		-8	/* New group is set, spin in read_groups() */
-#define GRP_EXIT		-9	/* Normal return to higher level */
+enum {
+	GRP_RETSELECT	= -1,	/* Pager 'T' command only -> return to selection screen */
+	GRP_QUIT		= -2,	/* Set by 'Q' when coming all the way out */
+	GRP_NEXTUNREAD	= -3,	/* (After catchup) goto next unread item */
+	GRP_NEXT		= -4,	/* (After catchup) move to next item */
+	GRP_ARTUNAVAIL	= -5,	/* show_page() Article is unavailable */
+	GRP_ARTABORT	= -6,	/* show_page() User aborted article read */
+	GRP_KILLED		= -7,	/* ?? Thread was killed at pager level */
+	GRP_GOTOTHREAD	= -8,	/* show_page() only. Goto thread menu */
+	GRP_ENTER		= -9,	/* New group is set, spin in read_groups() */
+	GRP_EXIT		= -10	/* Normal return to higher level */
+};
 
 #ifndef EXIT_SUCCESS
 #	define EXIT_SUCCESS	0	/* Successful exit status */
@@ -1239,6 +1330,13 @@ enum resizer { cNo, cYes, cRedraw };
 #define ART_WILL_RETURN		2
 #define ART_UNAVAILABLE		-1 /* Also used by msgid.article */
 
+/*
+ * art.killed
+ */
+#define ART_NOTKILLED		0
+#define ART_KILLED		1
+#define ART_KILLED_UNREAD	2
+
 /*
  * Additionally used for user aborts in art_open()
  */
@@ -1384,6 +1482,15 @@ typedef unsigned char	t_bitmap;
  */
 #define MSGID_HASH_SIZE		2609
 
+/*
+ * Archive-Name: header
+ */
+struct t_archive {
+	char *name;			/* name of archive */
+	char *partnum;			/* part/patch no. in archive */
+	t_bool ispart:1;		/* TRUE if part, FALSE if patch */
+};
+
 /*
  *	struct t_msgid - message id
  */
@@ -1405,9 +1512,10 @@ struct t_msgid {
  *	   directory for the group)
  *	>=0 points to another arts[] (struct t_article)
  *
- * article.inthread:
- *	FALSE for the first article in a thread, TRUE for all
- *	following articles in thread
+ * article.prev:
+ *	the previous article in thread
+ *	-1 (ART_NORMAL) initial default, no previous article
+ *	>=0 points to the previous arts[] (struct t_article)
  */
 struct t_article {
 	long artnum;			/* Article number in spool directory for group */
@@ -1422,19 +1530,18 @@ struct t_article {
 	char *refs;			/* References: article reference id's */
 	struct t_msgid *refptr;		/* Pointer to us in the reference tree */
 	int line_count;			/* Lines: number of lines in article */
-	char *archive;			/* Archive-name: line from mail header */
-	char *part;			/* part no. of archive */
-	char *patch;			/* patch no. of archive */
+	struct t_archive *archive;	/* Archive-Name: header */
 	int tagged;			/* 0 = not tagged, >0 = tagged */
 	int thread;
+	int prev;
 	int score;			/* score article has reached after filtering */
 	unsigned int status:2;	/* 0 = read, 1 = unread, 2 = will return */
-	unsigned int killed:1;	/* 0 = not killed, 1 = killed */
+	unsigned int killed:2;	/* 0 = not killed, 1 = killed, 2 = killed unread */
 	unsigned int zombie:1;	/* 1 = was alive (unread) before 'X' command */
 	unsigned int delete_it:1;	/* 1 = delete art when leaving group [mail group] */
-	unsigned int inthread:1;	/* 0 = thread head, 1 = thread follower */
 	t_bool selected:1;	/* FALSE = not selected, TRUE = selected */
 	t_bool inrange:1;	/* TRUE = article selected via # range command */
+	t_bool matched:1;	/* TRUE = article matched regex in feed.c */
 };
 
 /*
@@ -1455,6 +1562,7 @@ struct t_attribute {
 	char *from;				/* from line */
 	char *news_quote_format;		/* another way to begin a posting format */
 	char *quote_chars;			/* string to precede quoted text on each line */
+	char *mime_types_to_save;	/* MIME content major/minors we want to save */
 #ifdef HAVE_ISPELL
 	char *ispell;			/* path to ispell and options */
 #endif /* HAVE_ISPELL */
@@ -1585,10 +1693,6 @@ struct t_filter {
 	int  gnksa_num;			/* GNKSA code */
 	int  score;			/* score to give if rule matches */
 	char *xref;			/* groups in xref line */
-	int xref_max;			/* maximal number of groups in newsgroups line */
-	int xref_score_cnt;
-	int xref_scores[10];
-	char *xref_score_strings[10];
 	time_t time;			/* expire time in seconds */
 	struct t_filter *next;		/* next rule valid in group */
 	unsigned int inscope:4;		/* if group matches scope ie. 'comp.os.*' */
@@ -1628,10 +1732,8 @@ struct regex_cache {
 
 struct t_save {
 	char *path;
-	char *file;					/* => file part of *path */
-	struct t_article *artptr;	/* => article in arts[] */
-	t_bool saved:1;				/* Set if saved okay */
-	t_bool is_mailbox:1;			/* Set if path is a mailbox */
+	char *file;					/* ptr to file part of *path */
+	t_bool mailbox:1;			/* Set if path is a mailbox */
 };
 
 #ifndef USE_CURSES
@@ -1657,6 +1759,7 @@ struct t_art_stat {
 	int selected_total;	/* total selected count */
 	int selected_unread;	/* selected and unread */
 	int selected_seen;	/* selected and seen */
+	int killed;		/* killed */
 	int score;		/* maximum score */
 	int multipart_total; /* 0=not multipart, >0 = number of articles in the multipart */
 	int multipart_have; /* number of articles we actually have found */
@@ -1788,12 +1891,16 @@ typedef struct {
 #	endif /* HAVE_COMPTYPE_CHAR */
 #endif /* HAVE_COMPTYPE_VOID */
 
+/* Define a matching function pointer type */
+typedef int (*t_compfunc)(t_comptype, t_comptype);
+
 #define _CDECL
 
+/* Seperator between dir part of path & the filename */
 #ifdef VMS
-#	define SEPDIR ']'
+#	define DIRSEP	']'
 #else
-#	define SEPDIR	'/'
+#	define DIRSEP	'/'
 #endif /* VMS */
 
 /*
@@ -1804,7 +1911,6 @@ typedef struct {
 #define MOUSE_BUTTON_3		2
 
 #define TIN_EDITOR_FMT_OFF		"%E %F"
-#define NEWSGROUPS_FILE		"newsgroups"
 
 #ifdef M_AMIGA
 #	define REDIRECT_OUTPUT	"> NIL:"
@@ -1815,10 +1921,10 @@ typedef struct {
 #	define ENV_VAR_SHELL		"SHELL"
 #	define TIN_EDITOR_FMT_ON	"%E %F"
 #	define MAILER_FORMAT		"%M <%F -f %U"
-#	define METAMAIL_CMD		"%s -e -p -m \"tin\""
 #	define TMPDIR			"T:"
 #	define KEY_PREFIX		0x9b
 #endif /* M_AMIGA */
+
 #ifdef VMS
 #	define REDIRECT_OUTPUT	""
 #	define REDIRECT_PGP_OUTPUT	""
@@ -1826,9 +1932,8 @@ typedef struct {
 /*#	define ENV_VAR_SHELL		"SHELL"*/
 #	define ENV_VAR_POSTER		"TIN_POST"
 #	define TIN_EDITOR_FMT_ON	"%E %F"
-#	define METAMAIL_CMD		"%s -e -p -m \"tin\""
 #	define TMPDIR "SYS$SCRATCH:"
-#	ifdef	HAVE_KEY_PREFIX
+#	ifdef HAVE_KEY_PREFIX
 #		define KEY_PREFIX	0x9b
 #	endif /* HAVE_KEY_PREFIX */
 extern void joindir (char *result, const char *dir, const char *file);
@@ -1841,9 +1946,8 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define ENV_VAR_SHELL		"SHELL"
 #	define TIN_EDITOR_FMT_ON		"%E +%N %F"
 #	define MAILER_FORMAT		"%M -oi -t < %F"
-#	define METAMAIL_CMD		"%s -e -p -m \"tin\""
-#	define TMPDIR	_PATH_TMP
-#	ifdef	HAVE_KEY_PREFIX
+#	define TMPDIR	get_val("TMPDIR", _PATH_TMP)
+#	ifdef HAVE_KEY_PREFIX
 #		define KEY_PREFIX		0x8f: case 0x9b
 #	endif /* HAVE_KEY_PREFIX */
 #endif /* M_UNIX */
@@ -1868,9 +1972,6 @@ extern void joindir (char *result, const char *dir, const char *file);
 #ifndef MAILER_FORMAT
 #	define MAILER_FORMAT		""
 #endif /* !MAILER_FORMAT */
-#ifndef METAMAIL_CMD
-#	define METAMAIL_CMD		""
-#endif /* !METAMAIL_CMD */
 #ifndef TMPDIR
 #	define TMPDIR	_PATH_TMP
 #endif /* !TMPDIR */
@@ -2184,6 +2285,10 @@ extern struct tm *localtime(time_t *);
 #	define vsnprintf	plp_vsnprintf
 #endif /* HAVE_VSNPRINTF */
 
+/*
+ * TODO: might need an extra check for gcc-version >= 2.5
+ *       __GNUC__ __GNUC_MINOR__
+ */
 /* gcc-specific attributes */
 #if defined(__GNUC__) && !defined(__cplusplus) && !defined(__APPLE_CC__)
 #	define UNUSED(x) x __attribute__((unused))
