@@ -3,7 +3,7 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-22
+ *  Updated   : 2003-08-10
  *  Notes     : Configuration file routines
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -50,23 +50,28 @@
 #ifndef MENUKEYS_H
 #	include "menukeys.h"
 #endif /* !MENUKEYS_H */
+#ifndef TNNTP_H
+#	include "tnntp.h"
+#endif /* TNNTP_H */
 
 /*
  * local prototypes
  */
-static int check_upgrade(char *buf);
 static int get_option_num(int act_option);
 static int set_option_num(int option);
 static t_bool OptionOnPage(int option);
 static t_bool match_item(char *line, const char *pat, char *dst, size_t dstlen);
+static t_bool rc_update(FILE *fp);
 static void RepaintOption(int option);
 static void check_score_defaults(void);
 static void expand_rel_abs_pathname(int line, int col, char *str);
 static void highlight_option(int option);
 static void print_any_option(int act_option);
+static void print_option(enum option_enum the_option);
 static void redraw_screen(int option);
 static void show_config_page(void);
 static void unhighlight_option(int option);
+static void write_server_config(void);
 #ifdef HAVE_COLOR
 	static t_bool match_color(char *line, const char *pat, int *dst, int max);
 #endif /* HAVE_COLOR */
@@ -75,55 +80,10 @@ static void unhighlight_option(int option);
 #endif /* USE_CURSES */
 
 
-enum rc_state { IGNORE, CHECK, UPGRADE };
-
 #define DASH_TO_SPACE(mark)	((char) (mark == '_' ? ' ' : mark))
 #define SPACE_TO_DASH(mark)	((char) (mark == ' ' ? '_' : mark))
 
 
-/* FIXME: see doc/TODO and comments in the code */
-/*
- * If we don't find a matching version tag line at the top of the rc file,
- * give the user some upgrade guidance and silently update variables where
- * necessary. We use a simple state mechanism, starting with CHECK for the
- * 1st line, then switch to UPGRADE or IGNORE accordingly.
- */
-static int
-check_upgrade(
-	char *buf)
-{
-	char foo[60];
-	char bar[120]; /* should be enough */
-	int ch;
-
-	my_strncpy(foo, txt_tinrc_header, sizeof(foo) - 1);
-	snprintf(bar, sizeof(bar) - 1, foo, PRODUCT, TINRC_VERSION);
-
-	if (strncmp(buf, bar, MIN(strlen(bar), strlen(buf))) == 0)
-		return IGNORE;
-	else {
-		/*
-		 * TODO: update txt_warn_update (include a pointer to
-		 *       tinrcupdate.pl)
-		 */
-		error_message(_(txt_warn_update), VERSION);
-		error_message(_(txt_return_key));
-		ch = ReadCh();
-		/* TODO: document etc.pp. */
-		switch (ch) {
-			case iKeyQuit:
-			case iKeyQuitTin:
-			case iKeyAbort:
-				giveup();
-
-			default:
-				break;
-		}
-		return UPGRADE;
-	}
-}
-
-
 /*
  * read local & global configuration defaults
  */
@@ -135,18 +95,25 @@ read_config_file(
 	FILE *fp;
 	char newnews_info[PATH_LEN];
 	char buf[LEN];
-	int upgrade = CHECK;
+	int upgrade = RC_CHECK;
 
 	if ((fp = fopen(file, "r")) == NULL)
 		return FALSE;
 
+#if 0 /* batch_mode is not set at this stage, so checking for it is useless */
 	if (!batch_mode)
+#endif /* 0 */
 		wait_message(0, _(txt_reading_config_file), (global_file) ? _(txt_global) : "");
 
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
 		if (buf[0] == '#' || buf[0] == '\n') {
-			if (upgrade == CHECK)
-				upgrade = check_upgrade(buf);
+			if (upgrade == RC_CHECK && !global_file) {
+				upgrade = check_upgrade(buf, "# tin configuration file V", TINRC_VERSION);
+				if (upgrade != RC_IGNORE)
+					upgrade_prompt_quit(upgrade, CONFIG_FILE);
+				if (upgrade == RC_UPGRADE)
+					rc_update(fp);
+			}
 			continue;
 		}
 
@@ -206,10 +173,8 @@ read_config_file(
 				break;
 			}
 
-#ifdef HAVE_METAMAIL
 			if (match_boolean(buf, "ask_for_metamail=", &tinrc.ask_for_metamail))
 				break;
-#endif /* HAVE_METAMAIL */
 
 			if (match_boolean(buf, "auto_bcc=", &tinrc.auto_bcc))
 				break;
@@ -326,7 +291,7 @@ read_config_file(
 			if (match_string(buf, "default_savedir=", tinrc.savedir, sizeof(tinrc.savedir))) {
 				if (tinrc.savedir[0] == '.' && strlen(tinrc.savedir) == 1) {
 					get_cwd(buf);
-					my_strncpy(tinrc.savedir, buf, sizeof(tinrc.savedir));
+					my_strncpy(tinrc.savedir, buf, sizeof(tinrc.savedir) - 1);
 				}
 				break;
 			}
@@ -463,7 +428,7 @@ read_config_file(
 			break;
 
 		case 'h':
-			if (match_boolean(buf, "hide_uue=", &tinrc.hide_uue))
+			if (match_integer(buf, "hide_uue=", &tinrc.hide_uue, UUE_ALL))
 				break;
 
 			break;
@@ -484,30 +449,15 @@ read_config_file(
 			if (match_boolean(buf, "keep_dead_articles=", &tinrc.keep_dead_articles))
 				break;
 
-			if (match_boolean(buf, "keep_posted_articles=", &tinrc.keep_posted_articles))
-				break;
-
-			if (match_string(buf, "keep_posted_articles_file=", tinrc.keep_posted_articles_file, sizeof(tinrc.keep_posted_articles_file)))
-				break;
-
 			if (match_integer(buf, "kill_level=", &tinrc.kill_level, KILL_NOTHREAD))
 				break;
 
 			break;
 
-		case 'l':
-#ifdef LOCAL_CHARSET
-			if (match_boolean(buf, "local_charset=", &use_local_charset))
-				break;
-#endif /* LOCAL_CHARSET */
-
-			break;
-
 		case 'm':
 			if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
 				break;
 
-			/* option to toggle 8bit char. in header of mail message */
 			if (match_boolean(buf, "mail_8bit_header=", &tinrc.mail_8bit_header)) {
 				if (strcasecmp(txt_mime_encodings[tinrc.mail_mime_encoding], txt_8bit))
 					tinrc.mail_8bit_header = FALSE;
@@ -524,6 +474,9 @@ read_config_file(
 				break;
 #endif /* !CHARSET_CONVERSION */
 
+			if (match_boolean(buf, "mark_ignore_tags=", &tinrc.mark_ignore_tags))
+				break;
+
 			if (match_boolean(buf, "mark_saved_read=", &tinrc.mark_saved_read))
 				break;
 
@@ -536,6 +489,21 @@ read_config_file(
 			if (match_list(buf, "mailbox_format=", txt_mailbox_formats, NUM_MAILBOX_FORMATS, &tinrc.mailbox_format))
 				break;
 
+			if (match_string(buf, "metamail_prog=", tinrc.metamail_prog, sizeof(tinrc.metamail_prog)))
+				break;
+
+			if (match_integer(buf, "mono_markdash=", &tinrc.mono_markdash, MAX_ATTR))
+				break;
+
+			if (match_integer(buf, "mono_markstar=", &tinrc.mono_markstar, MAX_ATTR))
+				break;
+
+			if (match_integer(buf, "mono_markslash=", &tinrc.mono_markslash, MAX_ATTR))
+				break;
+
+			if (match_integer(buf, "mono_markstroke=", &tinrc.mono_markstroke, MAX_ATTR))
+				break;
+
 			break;
 
 		case 'n':
@@ -546,12 +514,18 @@ read_config_file(
 
 			/* pick which news headers to display */
 			if (match_string(buf, "news_headers_to_display=", tinrc.news_headers_to_display, sizeof(tinrc.news_headers_to_display))) {
+				if (news_headers_to_display_array)
+					FreeIfNeeded(*news_headers_to_display_array);
+				FreeIfNeeded(news_headers_to_display_array);
 				news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
 				break;
 			}
 
 			/* pick which news headers to NOT display */
 			if (match_string(buf, "news_headers_to_not_display=", tinrc.news_headers_to_not_display, sizeof(tinrc.news_headers_to_not_display))) {
+				if (news_headers_to_not_display_array)
+					FreeIfNeeded(*news_headers_to_not_display_array);
+				FreeIfNeeded(news_headers_to_not_display_array);
 				news_headers_to_not_display_array = ulBuildArgv(tinrc.news_headers_to_not_display, &num_headers_to_not_display);
 				break;
 			}
@@ -565,8 +539,8 @@ read_config_file(
 			if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
 				break;
 
-			/* option to toggle 8bit char. in header of news message */
 			if (match_boolean(buf, "post_8bit_header=", &tinrc.post_8bit_header)) {
+				/* if post_mime_encoding != 8bit, post_8bit_header is disabled */
 				if (strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_8bit))
 					tinrc.post_8bit_header = FALSE;
 				break;
@@ -580,14 +554,15 @@ read_config_file(
 			if (match_boolean(buf, "pos_first_unread=", &tinrc.pos_first_unread))
 				break;
 
-			if (match_integer(buf, "post_process_type=", &tinrc.post_process, POST_PROC_UUDECODE)) {
-				proc_ch_default = POST_PROC_TYPE(tinrc.post_process);
+			if (match_integer(buf, "post_process_type=", &tinrc.post_process, POST_PROC_YES))
 				break;
-			}
 
 			if (match_boolean(buf, "post_process_view=", &tinrc.post_process_view))
 				break;
 
+			if (match_string(buf, "posted_articles_file=", tinrc.posted_articles_file, sizeof(tinrc.posted_articles_file)))
+				break;
+
 			if (match_boolean(buf, "process_only_unread=", &tinrc.process_only_unread))
 				break;
 
@@ -682,10 +657,7 @@ read_config_file(
 			if (match_integer(buf, "scroll_lines=", &tinrc.scroll_lines, 0))
 				break;
 
-			if (match_boolean(buf, "show_lines=" , &tinrc.show_lines))
-				break;
-
-			if (match_boolean(buf, "show_score=" , &tinrc.show_score))
+			if (match_integer(buf, "show_info=", &tinrc.show_info, SHOW_INFO_BOTH))
 				break;
 
 			if (match_boolean(buf, "show_signatures=", &tinrc.show_signatures))
@@ -722,13 +694,8 @@ read_config_file(
 			break;
 
 		case 't':
-			if (match_integer(buf, "thread_articles=", &tinrc.thread_articles, THREAD_MAX)) {
-#if 0
-				if (upgrade == UPGRADE)
-					tinrc.thread_articles = THREAD_BOTH;
-#endif /* 0 */
+			if (match_integer(buf, "thread_articles=", &tinrc.thread_articles, THREAD_MAX))
 				break;
-			}
 
 			if (match_integer(buf, "thread_score=", &tinrc.thread_score, THREAD_SCORE_WEIGHT))
 				break;
@@ -770,11 +737,6 @@ read_config_file(
 				break;
 #endif /* HAVE_KEYPAD */
 
-#ifdef HAVE_METAMAIL
-			if (match_boolean(buf, "use_metamail=", &tinrc.use_metamail))
-				break;
-#endif /* HAVE_METAMAIL */
-
 #ifdef HAVE_COLOR
 			if (match_boolean(buf, "use_color=", &tinrc.use_color)) {
 				use_color = tinrc.use_color;
@@ -782,6 +744,11 @@ read_config_file(
 			}
 #endif /* HAVE_COLOR */
 
+#ifdef XFACE_ABLE
+			if (match_boolean(buf, "use_slrnface=", &tinrc.use_slrnface))
+				break;
+#endif /* XFACE_ABLE */
+
 			break;
 
 		case 'w':
@@ -798,10 +765,11 @@ read_config_file(
 			if (match_integer(buf, "wrap_column=", &tinrc.wrap_column, 0))
 				break;
 
-#ifdef HAVE_COLOR
+			if (match_boolean(buf, "wrap_on_next_unread=", &tinrc.wrap_on_next_unread))
+				break;
+
 			if (match_integer(buf, "word_h_display_marks=", &tinrc.word_h_display_marks, MAX_MARK))
 				break;
-#endif /* HAVE_COLOR */
 
 			break;
 
@@ -817,11 +785,6 @@ read_config_file(
 	}
 	fclose(fp);
 
-	/*
-	 * set posted_msgs_file
-	 */
-	joinpath(posted_msgs_file, tinrc.maildir, *tinrc.keep_posted_articles_file ? tinrc.keep_posted_articles_file : POSTED_FILE);
-
 	/*
 	 * sort out conflicting settings
 	 */
@@ -867,12 +830,7 @@ write_config_file(
 	char *file_tmp;
 	int i;
 
-	/*
-	 * TODO: is logic correct? shouldn't that be:
-	 * if (no_write || post_article_and_exit || post_postponed_and_exit)
-	 * 	return;
-	 */
-	if (no_write && !(post_article_and_exit || post_postponed_and_exit) && file_size(file) != -1L)
+	if ((no_write || post_article_and_exit || post_postponed_and_exit) && file_size(file) != -1L)
 		return;
 
 	/* generate tmp-filename */
@@ -916,6 +874,9 @@ write_config_file(
 	fprintf(fp, _(txt_confirm_choice.tinrc));
 	fprintf(fp, "confirm_choice=%s\n\n", txt_confirm_choices[tinrc.confirm_choice]);
 
+	fprintf(fp, _(txt_mark_ignore_tags.tinrc));
+	fprintf(fp, "mark_ignore_tags=%s\n\n", print_boolean(tinrc.mark_ignore_tags));
+
 	fprintf(fp, _(txt_auto_reconnect.tinrc));
 	fprintf(fp, "auto_reconnect=%s\n\n", print_boolean(tinrc.auto_reconnect));
 
@@ -1015,11 +976,8 @@ write_config_file(
 	fprintf(fp, _(txt_use_mailreader_i.tinrc));
 	fprintf(fp, "use_mailreader_i=%s\n\n", print_boolean(tinrc.use_mailreader_i));
 
-	fprintf(fp, _(txt_show_lines.tinrc));
-	fprintf(fp, "show_lines=%s\n\n", print_boolean(tinrc.show_lines));
-
-	fprintf(fp, _(txt_show_score.tinrc));
-	fprintf(fp, "show_score=%s\n\n", print_boolean(tinrc.show_score));
+	fprintf(fp, _(txt_show_info.tinrc));
+	fprintf(fp, "show_info=%d\n\n", tinrc.show_info);
 
 	fprintf(fp, _(txt_thread_score.tinrc));
 	fprintf(fp, "thread_score=%d\n\n", tinrc.thread_score);
@@ -1030,11 +988,8 @@ write_config_file(
 	fprintf(fp, _(txt_keep_dead_articles.tinrc));
 	fprintf(fp, "keep_dead_articles=%s\n\n", print_boolean(tinrc.keep_dead_articles));
 
-	fprintf(fp, _(txt_keep_posted_articles.tinrc));
-	fprintf(fp, "keep_posted_articles=%s\n\n", print_boolean(tinrc.keep_posted_articles));
-
-	fprintf(fp, _(txt_keep_posted_articles_file.tinrc));
-	fprintf(fp, "keep_posted_articles_file=%s\n\n", tinrc.keep_posted_articles_file);
+	fprintf(fp, _(txt_posted_articles_file.tinrc));
+	fprintf(fp, "posted_articles_file=%s\n\n", tinrc.posted_articles_file);
 
 	fprintf(fp, _(txt_add_posted_to_filter.tinrc));
 	fprintf(fp, "add_posted_to_filter=%s\n\n", print_boolean(tinrc.add_posted_to_filter));
@@ -1096,7 +1051,7 @@ write_config_file(
 	fprintf(fp, "tex2iso_conv=%s\n\n", print_boolean(tinrc.tex2iso_conv));
 
 	fprintf(fp, _(txt_hide_uue.tinrc));
-	fprintf(fp, "hide_uue=%s\n\n", print_boolean(tinrc.hide_uue));
+	fprintf(fp, "hide_uue=%d\n\n", tinrc.hide_uue);
 
 	fprintf(fp, _(txt_news_quote_format.tinrc));
 	fprintf(fp, "news_quote_format=%s\n", tinrc.news_quote_format);
@@ -1145,6 +1100,9 @@ write_config_file(
 	fprintf(fp, _(txt_auto_list_thread.tinrc));
 	fprintf(fp, "auto_list_thread=%s\n\n", print_boolean(tinrc.auto_list_thread));
 
+	fprintf(fp, _(txt_wrap_on_next_unread.tinrc));
+	fprintf(fp, "wrap_on_next_unread=%s\n\n", print_boolean(tinrc.wrap_on_next_unread));
+
 	fprintf(fp, _(txt_use_mouse.tinrc));
 	fprintf(fp, "use_mouse=%s\n\n", print_boolean(tinrc.use_mouse));
 
@@ -1242,16 +1200,21 @@ write_config_file(
 	fprintf(fp, "col_signature=%d\n\n", tinrc.col_signature);
 #endif /* HAVE_COLOR */
 
+#ifdef XFACE_ABLE
+	fprintf(fp, _(txt_use_slrnface.tinrc));
+	fprintf(fp, "use_slrnface=%s\n\n", print_boolean(tinrc.use_slrnface));
+#endif /*XFACE_ABLE */
+
 	fprintf(fp, _(txt_word_highlight.tinrc));
 	fprintf(fp, "word_highlight=%s\n\n", print_boolean(tinrc.word_highlight));
 
 	fprintf(fp, _(txt_wrap_column.tinrc));
 	fprintf(fp, "wrap_column=%d\n\n", tinrc.wrap_column);
 
-#ifdef HAVE_COLOR
 	fprintf(fp, _(txt_word_h_display_marks.tinrc));
 	fprintf(fp, "word_h_display_marks=%d\n\n", tinrc.word_h_display_marks);
 
+#ifdef HAVE_COLOR
 	fprintf(fp, _(txt_col_markstar.tinrc));
 	fprintf(fp, "col_markstar=%d\n", tinrc.col_markstar);
 	fprintf(fp, "col_markdash=%d\n", tinrc.col_markdash);
@@ -1259,6 +1222,12 @@ write_config_file(
 	fprintf(fp, "col_markstroke=%d\n\n", tinrc.col_markstroke);
 #endif /* HAVE_COLOR */
 
+	fprintf(fp, _(txt_mono_markstar.tinrc));
+	fprintf(fp, "mono_markstar=%d\n", tinrc.mono_markstar);
+	fprintf(fp, "mono_markdash=%d\n", tinrc.mono_markdash);
+	fprintf(fp, "mono_markslash=%d\n", tinrc.mono_markslash);
+	fprintf(fp, "mono_markstroke=%d\n\n", tinrc.mono_markstroke);
+
 	fprintf(fp, _(txt_mail_address.tinrc));
 	fprintf(fp, "mail_address=%s\n\n", tinrc.mail_address);
 
@@ -1275,12 +1244,6 @@ write_config_file(
 #	endif /* HAVE_ICONV_OPEN_TRANSLIT */
 #endif /* !CHARSET_CONVERSION */
 
-/* Not on Option Menu */
-#ifdef LOCAL_CHARSET
-	fprintf(fp, _(txt_tinrc_local_charset));
-	fprintf(fp, "local_charset=%s\n\n", print_boolean(use_local_charset));
-#endif /* LOCAL_CHARSET */
-
 	fprintf(fp, _(txt_post_mime_encoding.tinrc));
 	fprintf(fp, "post_mime_encoding=%s\n", txt_mime_encodings[tinrc.post_mime_encoding]);
 	fprintf(fp, "mail_mime_encoding=%s\n\n", txt_mime_encodings[tinrc.mail_mime_encoding]);
@@ -1291,13 +1254,11 @@ write_config_file(
 	fprintf(fp, _(txt_mail_8bit_header.tinrc));
 	fprintf(fp, "mail_8bit_header=%s\n\n", print_boolean(tinrc.mail_8bit_header));
 
-#ifdef HAVE_METAMAIL
-	fprintf(fp, _(txt_use_metamail.tinrc));
-	fprintf(fp, "use_metamail=%s\n\n", print_boolean(tinrc.use_metamail));
+	fprintf(fp, _(txt_metamail_prog.tinrc));
+	fprintf(fp, "metamail_prog=%s\n\n", tinrc.metamail_prog);
 
 	fprintf(fp, _(txt_ask_for_metamail.tinrc));
 	fprintf(fp, "ask_for_metamail=%s\n\n", print_boolean(tinrc.ask_for_metamail));
-#endif /* HAVE_METAMAIL */
 
 #ifdef HAVE_KEYPAD
 	fprintf(fp, _(txt_use_keypad.tinrc));
@@ -1355,8 +1316,18 @@ write_config_file(
 	fprintf(fp, _(txt_tinrc_newnews));
 	{
 		char timestring[30];
+		int j = find_newnews_index(nntp_server);
 
+		/*
+		 * Newnews timestamps in tinrc are bogus as of tin 1.5.19 because they
+		 * are now stored in a separate file to prevent overwriting them from
+		 * another instance running concurrently. Except for the current server,
+		 * however, we must remember them because otherwise we would lose them
+		 * after the first start of a tin 1.5.19 (or later) version.
+		 */
 		for (i = 0; i < num_newnews; i++) {
+			if (i == j)
+				continue;
 			if (my_strftime(timestring, sizeof(timestring) - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(newnews[i].time))))
 				fprintf(fp, "newnews=%s %lu (%s)\n", newnews[i].host, (unsigned long int) newnews[i].time, timestring);
 		}
@@ -1370,6 +1341,7 @@ write_config_file(
 		rename_file(file_tmp, file);
 
 	free(file_tmp);
+	write_server_config();
 }
 
 
@@ -1441,10 +1413,10 @@ fmt_option_prompt(
 	 *       requries changes in various prompt_*() functions (and lang.c)
 	 */
 	if (num) {
-		snprintf(dst, len, "%s %3d. %-*.*s: ", editing ? "->" : "  ", num,
-			OPTION_WIDTH, OPTION_WIDTH, _(option_table[option].txt->opt));
+		snprintf(dst, len, "%s %3d. %.*s: ", editing ? "->" : "  ", num,
+			OPTION_WIDTH, _(option_table[option].txt->opt));
 	} else
-		snprintf(dst, len, "  %-*.*s", OPTION_WIDTH, OPTION_WIDTH, _(option_table[option].txt->opt));
+		snprintf(dst, len, "  %.*s", cCOLS - 3, _(option_table[option].txt->opt));
 
 	return dst;
 }
@@ -1592,14 +1564,8 @@ unhighlight_option(
 
 /*
  * Refresh the config page which holds the actual option. If act_option is
- * zero fall back on the last given option; if even that is not present,
- * use 1 (first option) as fall back value. Note that act_option is the
- * number shown on the screen, not the index in option_table (which is
- * one smaller). Set force_redraw to TRUE if you want to enforce a refresh,
- * and to FALSE if you want to refresh the screen only when necessary
- * (needed by signal.c: config_resize(); the resizing could result in
- * first_option_on_page == actual_top_option even if there are now more/less
- * options on the screen than before).
+ * smaller zero fall back on the last given option (first option if there was
+ * no last option) and refresh the screen.
  */
 void
 refresh_config_page(
@@ -1638,6 +1604,11 @@ redraw_screen(
 
 /*
  * options menu so that the user can dynamically change parameters
+ *
+ * TODO: - why do we use ret_code when we never modify it?  what about calling
+ *         code which checks the return value?
+ *       - when we change something we need to update the related attributes
+ *         as well (see line 2009).
  */
 int
 change_config_file(
@@ -1872,6 +1843,7 @@ change_config_file(
 							break;
 
 						case OPT_POST_8BIT_HEADER:
+							/* if post_mime_encoding != 8bit, post_8bit_header is disabled */
 							if (strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_8bit)) {
 								tinrc.post_8bit_header = FALSE;
 								print_option(OPT_POST_8BIT_HEADER);
@@ -1899,30 +1871,40 @@ change_config_file(
 							break;
 #endif /* HAVE_COLOR */
 
+#ifdef XFACE_ABLE
+						/* use slrnface */
+						case OPT_USE_SLRNFACE:
+							if (!bool_equal(tinrc.use_slrnface, original_on_off_value)) {
+								if (!tinrc.use_slrnface)
+									slrnface_stop();
+								else
+									slrnface_start();
+							}
+							break;
+#endif /* XFACE_ABLE */
+
 						/*
-						 * the following do not need further action (if I'm right)
+						 * the following are boolean and do not need further action (if I'm right)
 						 *
 						 * case OPT_AUTO_BCC:
 						 * case OPT_AUTO_CC:
 						 * case OPT_AUTO_LIST_THREAD:
 						 * case OPT_AUTO_RECONNECT:
-						 * case OPT_CATCHUP_READ_GROUPS:
 						 * case OPT_AUTO_SAVE:
 						 * case OPT_BATCH_SAVE:
+						 * case OPT_CATCHUP_READ_GROUPS:
 						 * case OPT_FORCE_SCREEN_REDRAW:
 						 * case OPT_FULL_PAGE_SCROLL:
 						 * case OPT_GROUP_CATCHUP_ON_EXIT:
-						 * case OPT_KEEP_POSTED_ARTICLES:
+						 * case OPT_KEEP_DEAD_ARTICLES:
 						 * case OPT_MARK_SAVED_READ:
 						 * case OPT_NO_ADVERTISING:
 						 * case OPT_POS_FIRST_UNREAD:
+						 * case OPT_POSTED_ARTICLES:
 						 * case OPT_PRINT_HEADER:
 						 * case OPT_PROCESS_ONLY_UNREAD:
-						 * case OPT_SHOW_LINES:
-						 * case OPT_SHOW_SCORE:
 						 * case OPT_SHOW_ONLY_UNREAD_GROUPS:
 						 * case OPT_SHOW_XCOMMENTTO:
-						 * case OPT_HIDE_UUE:
 						 * case OPT_SIGDASHES:
 						 * case OPT_SPACE_GOTO_NEXT_UNREAD:
 						 * case OPT_START_EDITOR_OFFSET:
@@ -1933,15 +1915,12 @@ change_config_file(
 						 * case OPT_UNLINK_ARTICLE:
 						 * case OPT_USE_MAILREADER_I:
 						 * case OPT_USE_MOUSE:
+						 * case OPT_WORD_HIGHLIGHT_TINRC:
+						 * case OPT_WRAP_ON_NEXT_UNREAD:
 #ifdef HAVE_KEYPAD
 						 * case OPT_USE_KEYPAD:
 #endif
-#ifdef HAVE_METAMAIL
 						 * case OPT_ASK_FOR_METAMAIL:
-						 * case OPT_USE_METAMAIL:
-#endif
-						 * case OPT_KEEP_DEAD_ARTICLES:
-						 * case OPT_WORD_HIGHLIGHT_TINRC:
 #if defined(HAVE_ICONV_OPEN_TRANSLIT) && defined(CHARSET_CONVERSION)
 						 * case OPT_TRANSLIT:
 #endif
@@ -1973,8 +1952,15 @@ change_config_file(
 							 * so that rethreading will occur
 							 */
 							if (tinrc.thread_articles != original_list_value && group != NULL) {
+								int n, old_base_art = base[grpmenu.curr];
+
 								group->attribute->thread_arts = tinrc.thread_articles;
 								make_threads(group, TRUE);
+								/* in non-empty groups update cursor position */
+								if (grpmenu.max > 0) {
+									if ((n = which_thread(old_base_art)) >= 0)
+										grpmenu.curr = n;
+								}
 							}
 							clear_message();
 							break;
@@ -2002,7 +1988,9 @@ change_config_file(
 							break;
 
 						case OPT_POST_PROCESS:
-							proc_ch_default = POST_PROC_TYPE(tinrc.post_process);
+							glob_attributes.post_proc_type = tinrc.post_process;
+							if (group != NULL)
+								group->attribute->post_proc_type = tinrc.post_process;
 							break;
 
 						case OPT_SHOW_AUTHOR:
@@ -2017,7 +2005,7 @@ change_config_file(
 						case OPT_MAIL_MIME_ENCODING:
 						case OPT_POST_MIME_ENCODING:
 							mime_encoding = *(option_table[option].variable);
-							/* do not use 8 bit headers if mime encoding is not 8bit; ask J. Shin why */
+							/* do not use 8 bit headers if mime encoding is not 8bit */
 							if (strcasecmp(txt_mime_encodings[mime_encoding], txt_8bit)) {
 								if (option == (int) OPT_POST_MIME_ENCODING) {
 									tinrc.post_8bit_header = FALSE;
@@ -2031,20 +2019,81 @@ change_config_file(
 
 #ifdef CHARSET_CONVERSION
 						case OPT_MM_NETWORK_CHARSET:
-							if (tinrc.mm_network_charset != original_list_value && group != NULL)
-								group->attribute->mm_network_charset = tinrc.mm_network_charset;
-#if 0 /* TODO */
-							else
-								glob_attributes->attribute->mm_network_charset = tinrc.mm_network_charset;
-#endif /* 0 */
+							if (tinrc.mm_network_charset != original_list_value) {
+								glob_attributes.mm_network_charset = tinrc.mm_network_charset;
+								if (group)
+									group->attribute->mm_network_charset = tinrc.mm_network_charset;
 #	ifdef NO_LOCALE
-							strcpy(tinrc.mm_local_charset, txt_mime_charsets[tinrc.mm_network_charset]);
+								strcpy(tinrc.mm_local_charset, txt_mime_charsets[tinrc.mm_network_charset]);
 #	endif /* NO_LOCALE */
+							}
+							/*
+							 * check if we have selected a 7bit charset, otherwise
+							 * update encoding
+							 * we always do this (even if we did not change the
+							 * charset) to fixup flaws in the tinrc - once we do
+							 * the same while reading the tinrc this can go into
+							 * the != original_list_value case.
+							 */
+							{
+								int i;
+								t_bool change;
+
+								if (!strcasecmp(txt_mime_encodings[tinrc.post_mime_encoding], txt_7bit)) {
+									change = TRUE;
+									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
+										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
+											change = FALSE;
+											break;
+										}
+									}
+									if (change) {
+										tinrc.post_mime_encoding = MIME_ENCODING_8BIT;
+										RepaintOption(OPT_POST_MIME_ENCODING);
+									}
+								} else { /* and vice versa, if we have a 7bit chaset but a !7bit encoding, fix that */
+									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
+										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
+											tinrc.mail_mime_encoding = tinrc.post_mime_encoding = MIME_ENCODING_7BIT;
+											tinrc.mail_8bit_header = tinrc.post_8bit_header = FALSE;
+											RepaintOption(OPT_POST_MIME_ENCODING);
+											RepaintOption(OPT_MAIL_MIME_ENCODING);
+											RepaintOption(OPT_POST_8BIT_HEADER);
+											break;
+										}
+									}
+								}
+
+								if (!strcasecmp(txt_mime_encodings[tinrc.mail_mime_encoding], txt_7bit)) {
+									change = TRUE;
+									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
+										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
+											change = FALSE;
+											break;
+										}
+									}
+									if (change) {
+										tinrc.mail_mime_encoding = MIME_ENCODING_QP;
+										RepaintOption(OPT_MAIL_MIME_ENCODING);
+									}
+								} else { /* and vice versa, if we have a 7bit chaset but a !7bit encoding, fix that */
+									for (i = 0; *txt_mime_7bit_charsets[i]; i++) {
+										if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], txt_mime_7bit_charsets[i])) {
+											tinrc.mail_mime_encoding = tinrc.post_mime_encoding = MIME_ENCODING_7BIT;
+											tinrc.mail_8bit_header = tinrc.post_8bit_header = FALSE;
+											RepaintOption(OPT_POST_MIME_ENCODING);
+											RepaintOption(OPT_MAIL_MIME_ENCODING);
+											RepaintOption(OPT_POST_8BIT_HEADER);
+											break;
+										}
+									}
+								}
+							}
 							break;
 #endif /* CHARSET_CONVERSION */
 
 						/*
-						 * the following don't need any further action
+						 * the following are picklists and don't need any further action
 #ifdef HAVE_COLOR
 						 * case OPT_COL_BACK:
 						 * case OPT_COL_FROM:
@@ -2066,11 +2115,17 @@ change_config_file(
 						 * case OPT_COL_MARKDASH:
 						 * case OPT_COL_MARKSLASH:
 						 * case OPT_COL_MARKSTROKE:
-						 * case OPT_WORD_H_DISPLAY_MARKS:
 #endif
-						 * case OPT_DEFAULT_SORT_ART_TYPE:
+						 * case OPT_HIDE_UUE:
+						 * case OPT_WORD_H_DISPLAY_MARKS:
+						 * case OPT_MONO_MARKSTAR:
+						 * case OPT_MONO_MARKDASH:
+						 * case OPT_MONO_MARKSLASH:
+						 * case OPT_MONO_MARKSTROKE:
 						 * case OPT_CONFIRM_CHOICE:
+						 * case OPT_DEFAULT_SORT_ART_TYPE:
 						 * case OPT_MAILBOX_FORMAT:
+						 * case OPT_SHOW_INFO:
 						 *	break;
 						 */
 
@@ -2084,16 +2139,17 @@ change_config_file(
 						case OPT_EDITOR_FORMAT:
 						case OPT_INEWS_PROG:
 						case OPT_MAILER_FORMAT:
+						case OPT_MAIL_ADDRESS:
+						case OPT_MAIL_QUOTE_FORMAT:
+						case OPT_METAMAIL_PROG:
 #ifndef CHARSET_CONVERSION
 						case OPT_MM_CHARSET:
 #endif /* !CHARSET_CONVERSION */
-						case OPT_MAIL_QUOTE_FORMAT:
 						case OPT_NEWS_QUOTE_FORMAT:
 						case OPT_QUOTE_CHARS:
-						case OPT_XPOST_QUOTE_FORMAT:
-						case OPT_MAIL_ADDRESS:
 						case OPT_SPAMTRAP_WARNING_ADDRESSES:
 						case OPT_URL_HANDLER:
+						case OPT_XPOST_QUOTE_FORMAT:
 							prompt_option_string(option);
 							break;
 
@@ -2101,7 +2157,7 @@ change_config_file(
 							prompt_option_string(option);
 							if (news_headers_to_display_array)
 								FreeIfNeeded(*news_headers_to_display_array);
-							FreeIfNeeded((char *) news_headers_to_display_array);
+							FreeIfNeeded(news_headers_to_display_array);
 							news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
 							break;
 
@@ -2109,7 +2165,7 @@ change_config_file(
 							prompt_option_string(option);
 							if (news_headers_to_not_display_array)
 								FreeIfNeeded(*news_headers_to_not_display_array);
-							FreeIfNeeded((char *) news_headers_to_not_display_array);
+							FreeIfNeeded(news_headers_to_not_display_array);
 							news_headers_to_not_display_array = ulBuildArgv(tinrc.news_headers_to_not_display, &num_headers_to_not_display);
 							break;
 
@@ -2119,7 +2175,7 @@ change_config_file(
 						case OPT_MAILDIR:
 						case OPT_SAVEDIR:
 						case OPT_SIGFILE:
-						case OPT_KEEP_POSTED_ARTICLES_FILE:
+						case OPT_POSTED_ARTICLES_FILE:
 #ifdef M_AMIGA
 							if (tin_bbs_mode)
 								break;
@@ -2129,7 +2185,6 @@ change_config_file(
 								OPT_ARG_COLUMN + OPTION_WIDTH,
 								OPT_STRING_list[option_table[option].var_index]
 								);
-							joinpath(posted_msgs_file, tinrc.maildir, *tinrc.keep_posted_articles_file ? tinrc.keep_posted_articles_file : POSTED_FILE);
 							break;
 
 #ifdef HAVE_COLOR
@@ -2296,8 +2351,6 @@ change_config_file(
 
 /*
  * expand ~/News to /usr/username/News and print to screen
- *
- * TODO: fix the bug mentioned below
  */
 static void
 expand_rel_abs_pathname(
@@ -2305,28 +2358,13 @@ expand_rel_abs_pathname(
 	int col,
 	char *str)
 {
-	char buf[LEN];
+	char buf[PATH_LEN];
 
-	/* TODO: handle ~user/foo - use strfpath()? */
-	if (str[0] == '~') {
-		switch (str[1]) {
-			case '\0':
-				strcpy(str, homedir);
-				break;
-
-			case SEPDIR:
-				joinpath(buf, homedir, str + 2);
-				strcpy(str, buf);
-				break;
-
-			default:		/* TODO: handle ~foo */
-				break;
-		}
-	}
-	sprintf(buf, "%-.*s", cCOLS - col - 1, str);
+	strfpath(str, buf, sizeof(buf), &CURR_GROUP);
+	sprintf(str, "%-.*s", cCOLS - col - 1, buf);
 	MoveCursor(line, col);
 	CleartoEOLN();
-	my_fputs(buf, stdout);
+	my_fputs(str, stdout);
 	my_flush();
 }
 
@@ -2506,7 +2544,7 @@ match_item(
 	char *nline = my_strdup(line);
 	size_t patlen = strlen(pat);
 
-	nline[strlen(nline) -1] = '\0'; /* remove tailing \n */
+	nline[strlen(nline) - 1] = '\0'; /* remove tailing \n */
 
 	if (!strcasecmp(nline, pat)) {
 		strncpy(dst, &nline[patlen], dstlen);
@@ -2670,3 +2708,254 @@ check_score_defaults(
 	if (tinrc.score_select < tinrc.score_limit_select)
 		tinrc.score_select = tinrc.score_limit_select;
 }
+
+
+/*
+ * auto update tinrc
+ */
+static t_bool
+rc_update(
+	FILE *fp)
+{
+	char buf[1024];
+	const char *env;
+	t_bool confirm_to_quit = FALSE;
+	t_bool confirm_action = FALSE;
+	t_bool compress_quotes = FALSE;
+	t_bool hide_uue = FALSE;
+	t_bool keep_posted_articles = FALSE;
+	t_bool quote_empty_lines = FALSE;
+	t_bool quote_signatures = FALSE;
+	t_bool save_to_mmdf_mailbox = FALSE;
+	t_bool show_last_line_prev_page = FALSE;
+	t_bool show_lines = FALSE;
+	t_bool show_score = FALSE;
+	t_bool thread_articles = FALSE;
+	t_bool use_builtin_inews = FALSE;
+	t_bool use_getart_limit = FALSE;
+	t_bool use_metamail = FALSE;
+
+	if (!fp)
+		return FALSE;
+
+	/* rewind(fp); */
+	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
+		if (buf[0] == '#' || buf[0] == '\n')
+			continue;
+
+		switch (tolower((unsigned char) buf[0])) {
+			case 'c':
+				if (match_boolean(buf, "confirm_action=", &confirm_action))
+					break;
+				if (match_boolean(buf, "confirm_to_quit=", &confirm_to_quit))
+					break;
+				if (match_boolean(buf, "compress_quotes=", &compress_quotes))
+					break;
+				break;
+
+			case 'd':
+				/* simple rename */
+				if (match_string(buf, "default_regex_pattern=", tinrc.default_pattern, sizeof(tinrc.default_pattern)))
+					break;
+				break;
+
+			case 'h':
+				if (match_boolean(buf, "hide_uue=", &hide_uue))
+					break;
+				break;
+
+			case 'k':
+				if (match_boolean(buf, "keep_posted_articles=", &keep_posted_articles))
+					break;
+				break;
+
+			case 'q':
+				if (match_boolean(buf, "quote_signatures=" , &quote_signatures))
+					break;
+				if (match_boolean(buf, "quote_empty_lines=" , &quote_empty_lines))
+					break;
+				break;
+
+			case 's':
+				if (match_boolean(buf, "save_to_mmdf_mailbox=", &save_to_mmdf_mailbox))
+					break;
+				if (match_boolean(buf, "show_last_line_prev_page=", &show_last_line_prev_page))
+					break;
+				if (match_boolean(buf, "show_lines=", &show_lines))
+					break;
+				if (match_boolean(buf, "show_score=", &show_score))
+					break;
+				break;
+
+			case 't':
+				if (match_boolean(buf, "thread_articles=", &thread_articles))
+					break;
+				break;
+
+			case 'u':
+				if (match_boolean(buf, "use_builtin_inews=", &use_builtin_inews))
+					break;
+				if (match_boolean(buf, "use_getart_limit=", &use_getart_limit))
+					break;
+				if (match_boolean(buf, "use_metamail=", &use_metamail))
+					break;
+				break;
+
+			default:
+				break;
+		}
+	}
+
+	/* update the values */
+	tinrc.confirm_choice = (confirm_action ? 1 : 0 ) + (confirm_to_quit ? 3 : 0);
+
+	if (!use_getart_limit)
+		tinrc.getart_limit = 0;
+
+	if (hide_uue)
+		tinrc.hide_uue = 1;
+
+	if (keep_posted_articles)
+		strncpy(tinrc.posted_articles_file, "posted", sizeof(tinrc.posted_articles_file) - 1);
+
+	tinrc.quote_style = (compress_quotes ? QUOTE_COMPRESS : 0) + (quote_empty_lines ? QUOTE_EMPTY : 0) + (quote_signatures ? QUOTE_SIGS : 0);
+
+	tinrc.mailbox_format = (save_to_mmdf_mailbox ? 2 : 0);
+
+	tinrc.show_info = (show_lines ? SHOW_INFO_LINES : 0) + (show_score ? SHOW_INFO_SCORE : 0);
+
+	if (show_last_line_prev_page)
+		tinrc.scroll_lines = -1;
+
+	if (thread_articles)
+		tinrc.thread_articles = THREAD_BOTH;
+
+	if (use_builtin_inews)
+		strncpy(tinrc.inews_prog, INTERNAL_CMD, sizeof(tinrc.inews_prog) - 1);
+
+	env = getenv("NOMETAMAIL");
+	if (!use_metamail || (NULL == env))
+		strncpy(tinrc.metamail_prog, INTERNAL_CMD, sizeof(tinrc.metamail_prog) - 1);
+	else
+		my_strncpy(tinrc.metamail_prog, METAMAIL_CMD, sizeof(tinrc.metamail_prog) - 1);
+
+	rewind(fp);
+	return TRUE;
+}
+
+
+void
+read_server_config(
+	void)
+{
+	FILE *fp;
+	char *line;
+	char file[PATH_LEN];
+	char newnews_info[LEN];
+	char serverdir[PATH_LEN];
+	char version[LEN];
+	int upgrade = RC_CHECK;
+
+#ifdef NNTP_ABLE
+	if (read_news_via_nntp && !read_saved_news && nntp_tcp_port != IPPORT_NNTP)
+		snprintf(file, sizeof(file), "%s:%d", nntp_server, nntp_tcp_port);
+	else
+#endif /* NNTP_ABLE */
+	{
+		STRCPY(file, nntp_server);
+	}
+	JOINPATH(serverdir, rcdir, file);
+	joinpath(file, serverdir, SERVERCONFIG_FILE);
+
+	if ((fp = fopen(file, "r")) == NULL)
+		return;
+	while (NULL != (line = tin_fgets(fp, FALSE))) {
+		if (('#' == *line) || ('\0' == *line))
+			continue;
+
+		if (match_string(line, "last_newnews=", newnews_info, sizeof(newnews_info))) {
+			int tmp_len = strlen(nntp_server) + strlen(newnews_info) + 2;
+			char *tmp_info = my_malloc(tmp_len);
+
+			snprintf(tmp_info, tmp_len, "%s %s", nntp_server, newnews_info);
+			load_newnews_info(tmp_info);
+			free(tmp_info);
+			break;
+		}
+		if (match_string(line, "version=", version, sizeof(version))) {
+			if (RC_CHECK != upgrade)
+				/* ignore duplicate version lines; first match counts */
+				break;
+			upgrade = check_upgrade(line, "version=", SERVERCONFIG_VERSION);
+			if (RC_IGNORE == upgrade)
+				/* Expected version number; nothing to do -> continue */
+				break;
+
+			/* Nothing to do yet for RC_UPGRADE and RC_DOWNGRADE */
+			break;
+		}
+	}
+	fclose(fp);
+}
+
+
+static void
+write_server_config(
+	void)
+{
+	FILE *fp;
+	char *file_tmp;
+	char file[PATH_LEN];
+	char timestring[30];
+	char serverdir[PATH_LEN];
+	int i;
+	struct stat statbuf;
+
+	if (read_saved_news)
+		/* don't update server files while reading locally stored articles */
+		return;
+#ifdef NNTP_ABLE
+	if (read_news_via_nntp && nntp_tcp_port != IPPORT_NNTP)
+		snprintf(file, sizeof(file), "%s:%d", nntp_server, nntp_tcp_port);
+	else
+#endif /* NNTP_ABLE */
+	{
+		STRCPY(file, nntp_server);
+	}
+	JOINPATH(serverdir, rcdir, file);
+	joinpath(file, serverdir, SERVERCONFIG_FILE);
+
+	if ((no_write || post_article_and_exit || post_postponed_and_exit) && file_size(file) != -1L)
+		return;
+
+	if (-1 == stat(serverdir, &statbuf)) {
+		if (-1 == my_mkdir(serverdir, (mode_t) (S_IRWXU)))
+			/* Can't create directory TODO: Add error handling */
+			return;
+	}
+
+	/* generate tmp-filename */
+	file_tmp = get_tmpfilename(file);
+
+	if ((fp = fopen(file_tmp, "w")) == NULL) {
+		error_message(_(txt_filesystem_full_backup), SERVERCONFIG_FILE);
+		free(file_tmp);
+		return;
+	}
+
+	fprintf(fp, _(txt_serverconfig_header), PRODUCT, tin_progname, VERSION, RELEASEDATE, RELEASENAME, PRODUCT, PRODUCT);
+	fprintf(fp, "version=%s\n", SERVERCONFIG_VERSION);
+
+	if ((i = find_newnews_index(nntp_server)) >= 0)
+		if (my_strftime(timestring, sizeof(timestring) - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(newnews[i].time))))
+			fprintf(fp, "last_newnews=%lu (%s)\n", (unsigned long int) newnews[i].time, timestring);
+
+	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR)); /* rename_file() preserves mode */
+
+	if (ferror(fp) || fclose(fp))
+		error_message(_(txt_filesystem_full), SERVERCONFIG_FILE);
+	else
+		rename_file(file_tmp, file);
+
+	free(file_tmp);
+}
