@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2003-01-16
+ *  Updated   : 2003-07-21
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2003 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2003 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,72 +56,88 @@
 
 #define MATCH_REGEX(x,y,z)	(pcre_exec(x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
 
-static int put_rest(char **rest, char *dest, int max_line_len);
-static int read_decoded_base64_line(FILE *file, char *line, const int max_line_len, const int max_lines_to_read, char **rest);
-static int read_decoded_qp_line(FILE *file, char *line, const int max_line_len, const int max_lines_to_read, char **rest);
+
+static int put_rest(char **rest, char **line, int *max_line_len, const int offset);
+static int read_decoded_base64_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read, char **rest);
+static int read_decoded_qp_line(FILE *file, char **line, int *max_line_len, const int max_lines_to_read);
 static t_bool header_wanted(const char *line);
 static t_part *new_uue(t_part **part, char *name);
-static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part);
-static void put_cooked(t_bool wrap_lines, int flags, const char *fmt, ...);
+static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part, int hide_uue, int tabs);
+static void put_cooked(size_t buf_len, t_bool wrap_lines, int flags, const char *fmt, ...);
 static void set_rest(char **rest, const char *ptr);
+#ifdef DEBUG_ART
+	static void dump_cooked(void);
+#endif /* DEBUG_ART */
+
 
 /*
  * These are used globally within this module for access to the context
  * currently being built. They must not leak outside.
  */
-static int cook_width;
-static t_bool hide_uue;
 static t_openartinfo *art;
 
 
 /*
- * Rewrite frombuf into tobuf to a maximum length
  * Handle backspace, expand tabs, expand control chars to a literal ^[A-Z]
  * Allows \n through
  * Return TRUE if line contains a ^L (form-feed)
- * Anything after the (lenght-2)th character in the output string is
- * dropped, if the resultinglast character is not \n, \n is added
- * afterwards.
  */
 t_bool
 expand_ctrl_chars(
-	char *to,
-	const char *from,
-	int length,
+	char **line,
+	int *length,
 	size_t lcook_width)
 {
-	const char *p, *end;
-	char *q;
-	int i, j, space;
-	t_bool ctrl_L = FALSE;
+	int curr_len = LEN;
+	int i = 0, j;
+	char *buf = my_malloc(curr_len);
+	char *c;
+	t_bool ctrl_L = FALSE, resize = FALSE;
 
-	for (p = from, q = to, end = p, space = length - 2; space && *p; end = p, p++) {
-		if (*p == '\t') {			/* Expand tabs */
-			i = q - to;
-			j = ((i + lcook_width) / lcook_width) * lcook_width;
-/*			j = (i | (tabwidth - 1)) + 1; TODO more efficient? */
-
-			while (space && i++ < j) {
-				*q++ = ' ';
-				space--;
-			}
-		} else if (((*p) & 0xFF) < ' ' && *p != '\n' && (!IS_LOCAL_CHARSET("Big5") || *p != 27)) {	/* Literal ctrl chars */
-			*q++ = '^';
-			if (--space) {
-				*q++ = ((*p) & 0xFF) + '@';
-				space--;
-			}
-			if (*p == '\f')					/* ^L detected */
-				ctrl_L = TRUE;
-		} else {
-			*q++ = *p;
-			space--;
+	c = *line;
+	while (*c) {
+		if (resize) {
+			curr_len <<= 1;
+			buf = my_realloc(buf, curr_len);
+			resize = FALSE;
 		}
+		if (*c == '\t') { /* expand tabs */
+/*			j = ((i + lcook_width) / lcook_width) * lcook_width; */
+			j = i + lcook_width - (i % lcook_width);
+			if (j > curr_len - 2) {
+				resize = TRUE;
+				continue;
+			}
+			for (; i < j; i++)
+				buf[i] = ' ';
+		} else {
+			if (((*c) & 0xFF) < ' ' && *c != '\n' && (!IS_LOCAL_CHARSET("Big5") || *c != 27)) {	/* literal ctrl chars */
+				if (i > curr_len - 4) {
+					resize = TRUE;
+					continue;
+				}
+				buf[i++] = '^';
+				buf[i++] = ((*c) & 0xFF) + '@';
+				if (*c == '\f')	/* ^L detected */
+					ctrl_L = TRUE;
+			} else {
+				if (i > curr_len - 3) {
+					resize = TRUE;
+					continue;
+				}
+				buf[i++] = *c;
+			}
+		}
+		c++;
 	}
-	if (*end != '\n')
-		*q++ = '\n';	/* Force last char of string to be \n */
-
-	*q = '\0';
+	/* put_cooked() requires a newline at the end of the line */
+	if (buf[i - 1] != '\n')
+		buf[i++] = '\n';	/* Force last char of string to be \n */
+	buf[i] = '\0';
+	*length = i + 1;
+	*line = my_realloc(*line, *length);
+	strcpy(*line, buf);
+	free(buf);
 	return ctrl_L;
 }
 
@@ -130,17 +146,18 @@ expand_ctrl_chars(
  * Output text to the cooked stream. Wrap lines as necessary.
  * Update the line count and the array of line offsets
  * Extend the lineoffset array as needed in CHUNK amounts.
- * flags are 'hints' to the pager about line content
+ * flags are 'hints' to the pager about line content.
+ * buf_len is the size put_cooked should use for its buffer.
  */
 static void
 put_cooked(
+	size_t buf_len,
 	t_bool wrap_lines,
 	int flags,
 	const char *fmt,
 	...)
 {
-	char *p, *bufp;
-	char buf[LEN];
+	char *p, *bufp, *buf;
 	int wrap_column;
 	int space;
 	static int overflow = 0;
@@ -151,8 +168,10 @@ put_cooked(
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	va_list ap;
 
+	buf = my_malloc(buf_len + 1);
+
 	va_start(ap, fmt);
-	vsnprintf(buf, sizeof(buf) - 1, fmt, ap);
+	vsnprintf(buf, buf_len + 1, fmt, ap);
 
 	if (tinrc.wrap_column < 0)
 		wrap_column = ((tinrc.wrap_column > -cCOLS) ? cCOLS + tinrc.wrap_column : cCOLS);
@@ -162,7 +181,7 @@ put_cooked(
 	p = bufp = buf;
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wp = my_malloc ((MB_CUR_MAX + 1) * sizeof(wint_t));
+	wp = my_malloc((MB_CUR_MAX + 1) * sizeof(wint_t));
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	while (*p) {
@@ -231,6 +250,7 @@ put_cooked(
 	}
 
 	va_end(ap);
+	free(buf);
 }
 
 
@@ -245,16 +265,14 @@ set_rest(
 	char **rest,
 	const char *ptr)
 {
-	char *new_rest;
-	size_t new_rest_len;
+	char *old_rest = *rest;
 
-	new_rest_len = strlen(ptr);
-	if (new_rest_len) {
-		new_rest = my_malloc(new_rest_len + 1);
-		strcpy(new_rest, ptr);
-		*rest = new_rest;
-	} else
-		*rest = NULL;	/* no rest anymore */
+	if (ptr == NULL || strlen(ptr) == 0) {
+		FreeAndNull(*rest);
+		return;
+	}
+	*rest = my_strdup(ptr);
+	FreeIfNeeded(old_rest);
 }
 
 
@@ -262,68 +280,67 @@ set_rest(
  * FIXME: should go into rfc1521.c
  *
  * Copy things that were left over from the last decoding into the new line.
- * If there's a newline in the rest, copy everything up to and including
- * that newline in the expected buffer, adjust rest and return. If there's
- * no newline in the rest, copy all of it (modulo length of the buffer) to
- * the expected buffer and return.
+ * If there's a newline in the rest, copy everything up to and including that
+ * newline into the expected buffer, adjust rest and return. If there's no
+ * newline in the rest, copy all of it to the expected buffer and return.
+ *
+ * Side effects: resizes line if necessary, adjusts max_line_len
+ * accordingly.
  *
  * This function returns the number of character written to the line buffer.
- * The buffer is NULL terminated if a complete line was written or the line
- * buffer was filled up to its end. The buffer is NOT NULL terminated if
- * there was no newline in the rest.
+ * The buffer is LF + NULL terminated if a complete line was written. The
+ * buffer is NOT neccessarily NULL terminated if there was no newline in the
+ * rest. Use the return value to find out how long the string is.
  */
 static int
 put_rest(
 	char **rest,
-	char *dest,
-	int max_line_len)
+	char **line,
+	int *max_line_len,
+	const int offset)
 {
 	char *my_rest = *rest;
 	char *ptr;
-	int put_chars = 0;
+	char c;
+	int put_chars = offset;
 
-	ptr = my_rest;
-	if (my_rest) {
-		if (strlen(my_rest)) {
-			char c;
-
-			while ((c = *ptr++) && (c != '\n') && (put_chars < (max_line_len - 2))) {
-				if ((c == '\r') && (*ptr == '\n'))
-					continue;	/* step over CRLF */
-				*dest++ = c;
-				put_chars++;
-			}
-			if ((c == '\n') || (put_chars >= max_line_len - 2)) {
-				/*
-				 * FIXME: Adding a newline may be not correct. At least it may
-				 * be not what the author of that article intended.
-				 * Unfortunately, a newline is expected at the end of a line by
-				 * some other code in cook.c and even those functions invoking
-				 * this one rely on it.
-				 */
-				*dest++ = '\n';
-				put_chars++;	/* don't count the termining NULL! */
-				*dest = '\0';
-				if (c != '\n') {
-					/*
-					 * Line buffer was too small -- "ungetc" last character,
-					 * because we didn't put it into the line but left the loop.
-					 */
-					ptr--;
-				}
-				set_rest(rest, ptr);
-				FreeIfNeeded(my_rest);
-			} else {
-				/* rest is now empty */
-				*rest = NULL;
-				FreeIfNeeded(my_rest);
-			}
-		} else {
-			/* empty rest, clean up */
-			free(my_rest);
-			*rest = NULL;
-		}
+	if ((ptr = my_rest) == NULL)
+		return 0;
+	if (strlen(my_rest) == 0) {
+		FreeAndNull(*rest);
+		return 0;
 	}
+
+	while ((c = *ptr++) && (c != '\n')) {
+		if ((c == '\r') && (*ptr == '\n'))
+			continue;	/* step over CRLF */
+		/*
+		 * Resize line if necessary. Keep in mind that we add LF and \0 later.
+		 */
+		if (put_chars >= *max_line_len - 2) {
+			if (*max_line_len == 0)
+				*max_line_len = LEN;
+			else
+				*max_line_len <<= 1;
+			*line = my_realloc(*line, *max_line_len);
+		}
+		(*line)[put_chars++] = c;
+	}
+	if (c == '\n') {
+		/*
+		 * FIXME: Adding a newline may be not correct. At least it may
+		 * be not what the author of that article intended.
+		 * Unfortunately, a newline is expected at the end of a line by
+		 * some other code in cook.c and even those functions invoking
+		 * this one rely on it.
+		 */
+		(*line)[put_chars++] = '\n';
+		(*line)[put_chars] = '\0';	/* don't count the termining NULL! */
+		set_rest(rest, ptr);
+	} else
+		/* rest is now empty */
+		FreeAndNull(*rest);
+
 	return put_chars;
 }
 
@@ -337,22 +354,22 @@ put_rest(
  * This function keeps track of all these cases and always copies only one
  * decoded line to the line buffer.
  *
+ * Side effects: resizes line if necessary, adjusts max_line_len
+ * accordingly.
+ *
  * This function returns the number of physical lines read or a negative
  * value on error.
  */
 static int
 read_decoded_base64_line(
 	FILE *file,
-	char *line,
-	const int max_line_len,
+	char **line,
+	int *max_line_len,
 	const int max_lines_to_read,
 	char **rest)
 {
-	char *dest = line;
-	char *ptr;
-	char c;
-	char buf2[HEADER_LEN];	/* holds the entire decoded line */
-	char buf[HEADER_LEN];	/* holds the entire encoded line; allowed are 76 char + CRLF (cf. RFC 2045, section 6.8), so "this should be enough for everyone" */
+	char *buf2;	/* holds the entire decoded line */
+	char *buf;	/* holds the entire encoded line*/
 	int count = 0;
 	int lines_read = 0;
 	int put_chars = 0;
@@ -364,18 +381,22 @@ read_decoded_base64_line(
 	 * no newline in the rest, copy all of it (modulo length of the buffer) to
 	 * the expected buffer and continue as if there was no rest.
 	 */
-	put_chars = put_rest(rest, dest, max_line_len);
-	if (put_chars && (line[put_chars - 1] == '\n'))
+	put_chars = put_rest(rest, line, max_line_len, 0);
+	if (put_chars && ((*line)[put_chars - 1] == '\n'))
 		return 0;	/* we didn't read any new lines but filled the line */
-	dest = &line[put_chars];
 
 	/*
 	 * At this point, either there was no rest or there was no newline in the
 	 * rest. In any case, we need to read further encoded lines and decode
-	 * them until we find a newline or the line buffer is filled up
-	 * completely or there are no more (encoded or physical) lines in this
-	 * part of the posting.
+	 * them until we find a newline or there are no more (encoded or physical)
+	 * lines in this part of the posting. To be sure, now allocate memory for
+	 * the output if it wasn't already done.
 	 */
+	if (*max_line_len == 0) {
+		*max_line_len = LEN;
+		*line = my_malloc(*max_line_len);
+	}
+
 	/*
 	 * max_lines_to_read==0 occurs at end of an encoded part and if there was
 	 * no trailing newline in the encoded text. So we put one there and exit.
@@ -385,8 +406,8 @@ read_decoded_base64_line(
 	 */
 	if (max_lines_to_read <= 0) {
 		if (put_chars) {
-			*dest++ = '\n';
-			*dest++ = '\0';
+			(*line)[put_chars++] = '\n';
+			(*line)[put_chars] = '\0';
 		}
 		return max_lines_to_read;
 	}
@@ -394,7 +415,7 @@ read_decoded_base64_line(
 	 * Ok, now read a new line from the original article.
 	 */
 	do {
-		if (fgets(buf, sizeof(buf), file) == NULL) {
+		if ((buf = tin_fgets(file, FALSE)) == NULL) {
 			/*
 			 * Premature end of file (or file error), leave loop. To prevent
 			 * re-invoking of this function, set the numbers of read lines to
@@ -405,32 +426,24 @@ read_decoded_base64_line(
 			 * newline is expected at the end of a line by some other code in
 			 * cook.c.
 			 */
-			*dest++ = '\n';
-			*dest = '\0';
+			if (put_chars > *max_line_len - 2) {
+				*max_line_len <<= 1;
+				*line = my_realloc(*line, *max_line_len);
+			}
+			(*line)[put_chars++] = '\n';
+			(*line)[put_chars] = '\0';
 			return max_lines_to_read;
 		}
 		lines_read++;
+		buf2 = my_malloc(strlen(buf) + 1); /* decoded string is always shorter than encoded string, so this is safe */
 		count = mmdecode(buf, 'b', '\0', buf2);
 		buf2[count] = '\0';
-		ptr = &buf2[0];
-		while ((c = *ptr++) && (c != '\n') && (put_chars < (max_line_len - 2))) {
-			if ((c == '\r') && (*ptr == '\n'))
-				continue;	/* step over CRLF */
-			*dest++ = c;
-			put_chars++;
-		}
-		if ((c == '\n') || (put_chars >= max_line_len - 2)) {
-			if (put_chars && (line[put_chars - 1] == '\r') && (c == '\n'))
-				dest--;	/* remove CR before LF */
-			*dest++ = '\n';
-			*dest = '\0';
-			if (c != '\n')
-				ptr--;
-			set_rest(rest, ptr);
+		FreeIfNeeded(*rest);
+		*rest = buf2;
+		put_chars = put_rest(rest, line, max_line_len, put_chars);
+		if (put_chars && ((*line)[put_chars - 1] == '\n')) /* end of logical line reached */
 			return lines_read;
-		}
-	} while ((lines_read < max_lines_to_read) && (put_chars < max_line_len));
-	set_rest(rest, ptr);	/* should not be needed */
+	} while (lines_read < max_lines_to_read);
 	return lines_read;
 }
 
@@ -443,83 +456,53 @@ read_decoded_base64_line(
  * so this function collects all affected lines, concatenates and decodes
  * them.
  *
+ * Side effects: resizes line if necessary, adjusts max_line_len
+ * accordingly.
+ *
  * This function returns the number of physical lines read or a negative
  * value on error.
  */
 static int
 read_decoded_qp_line(
 	FILE *file,
-	char *line,							/* where to copy the decoded line */
-	const int max_line_len,			/* maximum line length */
-	const int max_lines_to_read,	/* don't read more physical lines than told here */
-	char **rest)						/* rest of line if line is too small */
+	char **line,					/* where to copy the decoded line */
+	int *max_line_len,				/* (maximum) line length */
+	const int max_lines_to_read)	/* don't read more physical lines than told here */
 {
-	char *buf, *buf2, *endptr;
-	char *dest = line;
+	char *buf, *buf2;
 	char *ptr;
 	char c;
-	int buflen = 0;
-	int chars_added = 0;
+	int buflen = LEN;
+	size_t chars_to_add = 0;
 	int count = 0;
 	int lines_read = 0;
-	int max_buf_len = 3 * max_line_len;	/* worst case: every char is qp-encoded (3 chars) */
-	int put_chars = 0;
 
-	/*
-	 * If there was anything left over from the last invokation, put it out
-	 * now.
-	 */
-	put_chars = put_rest(rest, dest, max_line_len);
-	if (put_chars && (line[put_chars - 1] == '\n'))
-		return 0;	/* we didn't read any new lines but filled the line */
-
-	dest = &line[put_chars];
-	if ((max_lines_to_read < 1) || (max_line_len < 2))
-		return max_lines_to_read;
-	buf = my_malloc((size_t) max_buf_len);
+	buf = my_malloc(buflen); /* initial internal line buffer */
 	*buf = '\0';
-	endptr = buf;
 	do {
-		if (fgets(endptr, max_buf_len - strlen(buf), file) == NULL) {
+		if ((buf2 = tin_fgets(file, FALSE)) == NULL) {
 			/*
-			 * Premature end of file (or file error), leave loop. To prevent
-			 * re-invokation of this function, set the numbers of read lines to
-			 * the expected maximum that should be read at most.
+			 * Premature end of file (or file error, leave loop. To prevent
+			 * re-invokation of this function, set the numbers of read lines
+			 * to the expected maximum that should be read at most.
 			 */
-#if 1
-			/*
-			 * FIXME: If the last line ended in '=' (soft break), skip that
-			 * character because the QP decoder will happily strip the
-			 * following newline that put_cooked() relies on. This should
-			 * probably be fixed in put_cooked(), not here.
-			 */
-			if (*endptr == '=')
-				endptr--;
-#endif /* 1 */
 			lines_read = max_lines_to_read;
 			break;
 		}
 		lines_read++;
-		chars_added = strlen(buf) - (endptr - buf);
-		if (!(buflen = strlen(buf)))
-			/*
-			 * Empty line. Should not occur, at least newline should be there.
-			 * We'll add it when we left the loop.
-			 */
+		if ((chars_to_add = strlen(buf2)) == 0) /* Empty line, leave loop. */
 			break;
-		endptr = &buf[buflen - 1];
-		buf[buflen] = '\0';
-		c = *endptr;
+
 		/*
-		 * Strip trailing white space at the end of the line, but no more than
-		 * the number of characters that were newly read.
+		 * Strip trailing white space at the end of the line.
 		 * See RFC 2045, section 6.7, #3
 		 */
-		while ((chars_added > 0) && ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r'))) {
-			c = *--endptr;
-			buflen--;
-			chars_added--;
+		c = buf2[chars_to_add - 1];
+		while ((chars_to_add > 0) && ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r'))) {
+			--chars_to_add;
+			c = (chars_to_add > 0 ? buf2[chars_to_add - 1] : '\0');
 		}
+
 		/*
 		 * '=' at the end of a line indicates a soft break meaning
 		 * that the following physical line "belongs" to this one.
@@ -529,22 +512,34 @@ read_decoded_qp_line(
 		 * loop is not left but the next line is read and concatenated
 		 * with this one while the '=' is overwritten.
 		 */
-	} while ((c == '=') && (lines_read < max_lines_to_read) && (buflen < max_buf_len));
+		if (c == '=') /* c is 0 when chars_to_add is 0 so this is safe */
+			buf2[--chars_to_add] = '\0';
+
+		/*
+		 * Join physical lines to a logical one; keep in mind that a LF is
+		 * added afterwards.
+		 */
+		if (chars_to_add > buflen - strlen(buf) - 2) {
+			buflen <<= 1;
+			buf = my_realloc(buf, buflen);
+		}
+		strncat(buf, buf2, buflen);
+	} while ((c == '=') && (lines_read < max_lines_to_read));
 	/*
 	 * re-add newline and NULL termination at end of line
 	 * FIXME: Adding a newline may be not correct. At least it may be not
 	 * what the author of that article intended. Unfortunately, a newline is
 	 * expected at the end of a line by some other code in cook.c.
 	 */
-	*++endptr = '\n';
-	*++endptr = '\0';
+	strcat(buf, "\n");
 
 	/*
 	 * Now decode complete (logical) line from buf to buf2 and copy it to the
 	 * buffer where the invoking function expects it. Don't decode directly
-	 * to the buffer of the other function to prevent buffer overruns.
+	 * to the buffer of the other function to prevent buffer overruns and to
+	 * decide if the encoding was ok.
 	 */
-	buf2 = my_malloc(strlen(buf) + 1);
+	buf2 = my_malloc(strlen(buf) + 1); /* Don't use realloc here, tin_fgets relies on its internal state! */
 	count = mmdecode(buf, 'q', '\0', buf2);
 
 	if (count >= 0) {
@@ -553,10 +548,12 @@ read_decoded_qp_line(
 	} else	/* error in encoding: copy raw line */
 		ptr = buf;
 
-	strncpy(dest, ptr, max_line_len - put_chars - 1);
-	line[max_line_len - 1] = '\0'; /* be sure to terminate string */
-	if ((signed int) (strlen(ptr) + put_chars) > max_line_len - 1)
-		set_rest(rest, &ptr[max_line_len - put_chars - 1]);
+	if (*max_line_len < (int) strlen(ptr) + 1) {
+		*max_line_len = strlen(ptr) + 1;
+		*line = my_realloc(*line, *max_line_len);
+	}
+	strncpy(*line, ptr, *max_line_len);
+	(*line)[*max_line_len - 1] = '\0'; /* be sure to terminate string */
 	free(buf);
 	free(buf2);
 	return lines_read;
@@ -587,6 +584,7 @@ new_uue(
 	ptr->encoding = ENCODING_UUE;	/* treat as x-uuencode */
 
 	ptr->offset = ftell(art->raw);
+	ptr->depth = (*part)->depth;	/* uue is at the same depth as the envelope */
 
 	/*
 	 * If an extension is present, try and add a Content-Type
@@ -615,7 +613,7 @@ get_filename(
 			return NULL;
 	}
 
-	/* TODO: Use base_name()? */
+	/* TODO: Use base_name()? or at least DIRSEP */
 	if (((p = strrchr(name, '/'))) || ((p = strrchr(name, '\\'))))
 		return p + 1;
 
@@ -623,6 +621,28 @@ get_filename(
 }
 
 
+#define PUT_UUE(part, qualifier_text)					\
+	put_cooked(LEN, wrap_lines, C_UUE, _(txt_uue),			\
+		part->depth * 4, "",							\
+		content_types[part->type], part->subtype,		\
+		qualifier_text, part->line_count, get_filename(part->params))
+
+#define PUT_ATTACH(part, depth, name, charset) \
+	put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_attach),	\
+		depth, "",	\
+		content_types[part->type], part->subtype,	\
+		content_encodings[part->encoding],	\
+		charset ? _(txt_attach_charset) : "", BlankIfNull(charset),	\
+		part->line_count,	\
+		name ? _(txt_name) : "", BlankIfNull(name));	\
+		\
+	if (part->description)	\
+		put_cooked(LEN, wrap_lines, C_ATTACH,	\
+			_(txt_attach_description),	\
+			depth, "",	\
+			part->description);	\
+	put_cooked(1, wrap_lines, C_ATTACH, "\n")
+
 /*
  * Decodes text bodies, remove sig's, detects uuencoded sections
  */
@@ -630,18 +650,26 @@ static void
 process_text_body_part(
 	t_bool wrap_lines,
 	FILE *in,
-	t_part *part)
+	t_part *part,
+	int hide_uue,
+	int tabs)
 {
 	char *rest = NULL;
-	char *line, *buf;
-	char to[LEN];
-	int max_line_len = LEN;
+	char *line = NULL, *buf;
+	int max_line_len = 0;
 	int flags, len, lines_left;
+	int offsets[6];
+	int size_offsets = ARRAY_SIZE(offsets);
 	t_bool in_sig = FALSE;			/* Set when in sig portion */
 	t_bool in_uue = FALSE;			/* Set when in uuencoded section */
+	t_bool is_uubody;				/* Set when current line looks like a uuencoded line */
 	t_part *curruue = NULL;
 
-	line = buf = my_malloc(max_line_len);
+	if (part->uue) {				/* These are redone each time we recook/resize etc.. */
+		free_parts(part->uue);
+		part->uue = NULL;
+	}
+
 	fseek(in, part->offset, SEEK_SET);
 
 	if (part->encoding == ENCODING_BASE64)
@@ -651,17 +679,38 @@ process_text_body_part(
 	while ((lines_left > 0) || rest) {
 		switch (part->encoding) {
 			case ENCODING_BASE64:
-				lines_left -= read_decoded_base64_line(in, buf, max_line_len, lines_left, &rest);
-				line = buf;
+				lines_left -= read_decoded_base64_line(in, &line, &max_line_len, lines_left, &rest);
 				break;
 
 			case ENCODING_QP:
-				lines_left -= read_decoded_qp_line(in, buf, max_line_len, lines_left, &rest);
-				line = buf;
+				lines_left -= read_decoded_qp_line(in, &line, &max_line_len, lines_left);
 				break;
 
 			default:
-				line = fgets(buf, max_line_len, in);
+				if ((buf = tin_fgets(in, FALSE)) == NULL) {
+					FreeAndNull(line);
+					break;
+				}
+
+				/*
+				 * tin_fgets() uses the returned space also internally
+				 * so it's not advisable to use it for our own purposes
+				 * especially if we must resize it.
+				 * So copy buf to line (and resize line if necessary).
+				 */
+				if (max_line_len < (int) strlen(buf) + 2) {
+					max_line_len = strlen(buf) + 2;
+					line = my_realloc(line, max_line_len);
+				};
+				strcpy(line, buf);
+
+				/*
+				 * FIXME: Some code in cook.c expects a '\n' at the end
+				 * of the line. As tin_fgets() strips trailing '\n', re-add it.
+				 * This should problably be fixed in that other code.
+				 */
+				strcat(line, "\n");
+
 				lines_left--;
 				break;
 		}
@@ -670,96 +719,107 @@ process_text_body_part(
 
 		/* convert network to local charset, tex2iso, iso2asc etc. */
 		process_charsets(&line, &max_line_len, get_param(part->params, "charset"), tinrc.mm_local_charset, CURR_GROUP.attribute->tex2iso_conv && art->tex2iso);
-		/* we might have realloced line, so make sure we use this new buffer */
-		buf = line;
+
+		len = (int) strlen(line);
 
 		/*
 		 * Detect and skip signatures if necessary
-		 * Detect and process uuencoded sections
-		 * Check quoting depth
 		 */
-		len = (int) strlen(line);
-
 		if (!in_sig) {
 			if (strcmp(line, SIGDASHES) == 0) {
 				in_sig = TRUE;
 				if (in_uue) {
 					in_uue = FALSE;
-					put_cooked(wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
+					if (hide_uue)
+						PUT_UUE(curruue, _(txt_incomplete));
 				}
 			}
 		}
 
-		if (hide_uue) {
-			int offsets[6];
-			int size_offsets = ARRAY_SIZE(offsets);
-			t_bool is_uubody = FALSE;		/* Set if this line looks like a uuencoded line */
-
-			/*
-			 * Look for the start or the end of a uuencoded section
-			 */
-			if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
-				in_uue = TRUE;
-				curruue = new_uue(&part, line + offsets[1]);
-				continue;
-			} else if (strncmp(line, "end\n", 4) == 0) {
-				if (in_uue) {
-					in_uue = FALSE;
-					/* TODO include type/subtype/encoding in txt_uue as in txt_attach? */
-					put_cooked(wrap_lines, C_UUE, txt_uue, "", curruue->line_count, get_filename(curruue->params));
-					continue;				/* To stop 'end' line appearing */
-				}
-			}
-
-			/*
-			 * See if this line looks like a uuencoded line
-			 */
-			if (MATCH_REGEX (uubody_regex, line, len)) {
-				int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
-
-				/*
-				 * sum = 0 in a uubody only on the last line, a single `
-				 */
-				if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
-					is_uubody = TRUE;
-				else if (len == sum + 1 + 1)
-					is_uubody = TRUE;
-#ifdef DEBUG_ART
-				if (debug == 2)
-					fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
-#endif /* DEBUG_ART */
-			}
+		if (in_sig && !tinrc.show_signatures)
+			continue;					/* No further processing needed */
 
+		/*
+		 * Detect and process uuencoded sections
+		 * Look for the start or the end of a uuencoded section
+		 */
+		if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
+			in_uue = TRUE;
+			curruue = new_uue(&part, line + offsets[1]);
+			if (hide_uue)
+				continue;				/* Don't cook the 'begin' line */
+		} else if (strncmp(line, "end\n", 4) == 0) {
 			if (in_uue) {
-				if (is_uubody)
-					curruue->line_count++;
-				else {
-					if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
-#ifdef DEBUG_ART
-						fprintf(stderr, "not a uue line while reading a uue body?\n");
-#endif /* DEBUG_ART */
-						in_uue = FALSE;
-						put_cooked(wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
-					}
+				in_uue = FALSE;
+				if (hide_uue) {
+					PUT_UUE(curruue, "");
+					continue;			/* Don't cook the 'end' line */
 				}
-			} else {
-#if 0
-				if (is_uubody) {
-					char name[] = "(missing)";
-					curruue = new_uue(&part, name);
-					curruue->line_count++;				/* We never saw a begin line */
-					in_uue = TRUE;
-					continue;
-				}
-#endif /* 0 */
 			}
 		}
 
-		if (in_uue || (in_sig && !tinrc.show_signatures))
-			continue;
+		/*
+		 * See if this line looks like a uuencoded 'body' line
+		 */
+		is_uubody = FALSE;
+
+		if (MATCH_REGEX (uubody_regex, line, len)) {
+			int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
+
+			/* sum = 0 in a uubody only on the last line, a single ` */
+			if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
+				is_uubody = TRUE;
+			else if (len == sum + 1 + 1)
+				is_uubody = TRUE;
+#ifdef DEBUG_ART
+			if (debug == 2)
+				fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
+#endif /* DEBUG_ART */
+		}
+
+		if (in_uue) {
+			if (is_uubody)
+				curruue->line_count++;
+			else {
+				if (line[0] == '\n') {		/* Blank line in a uubody - definitely a failure */
+					/* fprintf(stderr, "not a uue line while reading a uue body?\n"); */
+					in_uue = FALSE;
+					if (hide_uue)
+						/* don't continue here, so we see the line that 'broke' in_uue */
+						PUT_UUE(curruue, _(txt_incomplete));
+				}
+			}
+		} else {
+			/*
+			 * UUE_ALL = 'Try harder' - we never saw a begin line, but useful
+			 * when uue sections are split across > 1 article
+			 */
+			if (is_uubody && hide_uue == UUE_ALL) {
+				char name[] = N_("(unknown)");
+
+				curruue = new_uue(&part, name);
+				curruue->line_count++;
+				in_uue = TRUE;
+				continue;
+			}
+		}
+
+		/*
+		 * Skip output if we're hiding uue or the sig
+		 */
+		if (in_uue && hide_uue)
+			continue;						/* No further processing needed */
 
 		flags = in_sig ? C_SIG : C_BODY;
 
+		/*
+		 * Don't do any further handling of uue lines - the data is binary after all
+		 */
+		if (in_uue) {
+			put_cooked(max_line_len, wrap_lines, flags, "%s", line);
+			continue;
+		}
+
 #ifdef HAVE_COLOR
 		if (quote_regex3.re) {
 			if (MATCH_REGEX(quote_regex3, line, len))
@@ -789,17 +849,16 @@ process_text_body_part(
 		 * TODO: integrate into expand_ctrl_chars
 		 */
 		convert_body2printable(line);
-		if (expand_ctrl_chars(to, line, sizeof(to) - 1, cook_width))
+		if (expand_ctrl_chars(&line, &max_line_len, tabs))
 			flags |= C_CTRLL;				/* Line contains form-feed */
-
-		put_cooked(wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", to);
+		put_cooked(max_line_len, wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", line);
 	} /* while */
 
 	/*
 	 * Were we reading uue and ran off the end ?
 	 */
-	if (in_uue)
-		put_cooked(wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
+	if (in_uue && hide_uue)
+		PUT_UUE(curruue, _(txt_incomplete));
 
 	free(line);
 }
@@ -870,13 +929,15 @@ dump_cooked(
  * trying to keep this simple.
  *
  * Expects:
- *		Rewound open cooked stream
  *		Fresh article context to write into
+ *		parse_uue is set only when the art is opened to create t_parts for
+ *		uue sections found, when resizing this is not needed
+ *		hide_uue determines the folding of uue sections
  * Handles:
  *		multipart articles
  *		stripping of non text sections if skip_alternative
  *		Q and B decoding of text sections
- *		detection of uuencoded sections
+ *		handling of uuencoded sections
  *		stripping of sigs if !show_signatures
  * Returns:
  *		TRUE on success
@@ -886,15 +947,15 @@ cook_article(
 	t_bool wrap_lines,
 	t_openartinfo *artinfo,
 	int tabs,
-	t_bool uue)
+	int hide_uue)
 {
+	const char *charset;
+	const char *name;
 	char *line;
 	struct t_header *hdr = &artinfo->hdr;
 	t_bool header_put = FALSE;
 
 	art = artinfo;				/* Global saves lots of passing artinfo around */
-	cook_width = tabs;
-	hide_uue = uue;
 
 	if (!(art->cooked = tmpfile()))
 		return FALSE;
@@ -911,17 +972,20 @@ cook_article(
 			if (STRIP_ALTERNATIVE(artinfo)) {
 				if (header_wanted(_(txt_info_x_conversion_note))) {
 					header_put = TRUE;
-					put_cooked(wrap_lines, C_HEADER, _(txt_info_x_conversion_note));
+					put_cooked(LEN, wrap_lines, C_HEADER, _(txt_info_x_conversion_note));
 				}
 			}
 			if (header_put)
-				put_cooked(TRUE, 0, "\n");		/* put a newline after headers */
+				put_cooked(1, TRUE, 0, "\n");		/* put a newline after headers */
 			break;
 		}
 
 		if (header_wanted(line)) {	/* Put cooked data */
 			header_put = TRUE;
-			put_cooked(wrap_lines, C_HEADER, "%s\n", rfc1522_decode(line));
+			/*
+			 * FIXME: don't decode addr-part of From:/Cc:/ etc.pp.
+			 */
+			put_cooked(LEN, wrap_lines, C_HEADER, "%s\n", rfc1522_decode(line));
 		}
 	}
 
@@ -934,41 +998,41 @@ cook_article(
 	 */
 	if (hdr->mime && hdr->ext->type == TYPE_MULTIPART) {
 		t_part *ptr;
-		const char *name;
 
 		for (ptr = hdr->ext->next; ptr != NULL; ptr = ptr->next) {
-			name = get_filename(ptr->params);
-
 			/*
 			 * Ignore non text/plain sections with alternative handling
 			 */
 			if (STRIP_ALTERNATIVE(artinfo) && !IS_PLAINTEXT(ptr))
 				continue;
 
-			put_cooked(wrap_lines, C_ATTACH, _(txt_attach),
-				ptr->depth * 4, "",
-				content_types[ptr->type], ptr->subtype,
-				content_encodings[ptr->encoding], ptr->line_count,
-				(name) ? _(", name: ") : "", (name) ? name : "");	/* FIXME: -> lang.c */
+			name = get_filename(ptr->params);
+			if (!strcmp(content_types[ptr->type], "text"))
+				charset = get_param(ptr->params, "charset");
+			else
+				charset = NULL;
+			PUT_ATTACH(ptr, ptr->depth * 4, name, charset);
 
 			/* Try to view anything of type text, may need to review this */
 			if (IS_PLAINTEXT(ptr))
-				process_text_body_part(wrap_lines, artinfo->raw, ptr);
+				process_text_body_part(wrap_lines, artinfo->raw, ptr, hide_uue, tabs);
 		}
 	} else {
 		/*
 		 * A regular single-body article
 		 */
 		if (IS_PLAINTEXT(hdr->ext))
-			process_text_body_part(wrap_lines, artinfo->raw, hdr->ext);
-		else {						/* Non-textual main body */
-			const char *name = get_filename(hdr->ext->params);
-
-			put_cooked(wrap_lines, C_ATTACH, _(txt_attach),
-					0, "",
-					content_types[hdr->ext->type], hdr->ext->subtype,
-					content_encodings[hdr->ext->encoding], hdr->ext->line_count,
-					(name) ? _(", name: ") : "", (name) ? name : "");	/* FIXME: -> lang.c */
+			process_text_body_part(wrap_lines, artinfo->raw, hdr->ext, hide_uue, tabs);
+		else {
+			/*
+			 * Non-textual main body
+			 */
+			name = get_filename(hdr->ext->params);
+			if (!strcmp(content_types[hdr->ext->type], "text"))
+				charset = get_param(hdr->ext->params, "charset");
+			else
+				charset = NULL;
+			PUT_ATTACH(hdr->ext, 0, name, charset);
 		}
 	}
 
@@ -980,5 +1044,5 @@ cook_article(
 		art->cookl = my_realloc(art->cookl, sizeof(t_lineinfo) * art->cooked_lines);
 
 	rewind(art->cooked);
-	return TRUE;
+	return (tin_errno != 0) ? FALSE : TRUE;
 }
