@@ -3,7 +3,7 @@
  *  Module    : curses.c
  *  Author    : D. Taylor & I. Lea
  *  Created   : 1986-01-01
- *  Updated   : 2002-12-18
+ *  Updated   : 2003-08-03
  *  Notes     : This is a screen management library borrowed with permission
  *              from the Elm mail system. This library was hacked to provide
  *              what tin needs.
@@ -21,13 +21,17 @@
 #	include "tnntp.h"
 #endif /* !TNNTP_H */
 
+/* local prototype */
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && defined(M_UNIX)
+	static wint_t convert_c2wc (const char *input);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE && M_UNIX */
 
 #ifdef USE_CURSES
 
 #define ReadCh cmdReadCh
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE) && defined(M_UNIX)
 #	define ReadWch cmdReadWch
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE && M_UNIX */
 
 void my_dummy(void) { }	/* ANSI C requires non-empty file */
 t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
@@ -63,8 +67,6 @@ static int _inraw = FALSE;	/* are we IN rawmode? */
 static int xclicks = FALSE;	/* do we have an xterm? */
 t_bool have_linescroll = FALSE;
 
-#define BACKSPACE	'\b'
-#define VERY_LONG_STRING	2500
 #define TTYIN	0
 
 #ifdef HAVE_CONFIG_H
@@ -92,8 +94,10 @@ t_bool have_linescroll = FALSE;
 #				include <sgtty.h>
 #				define USE_SGTTY 1
 #				define TTY struct sgttyb
-#			else
-#				error "No termios.h, termio.h or sgtty.h found"
+/*
+	#			else
+	#				error "No termios.h, termio.h or sgtty.h found"
+*/
 #			endif /* HAVE_SGTTY_H */
 #		endif /* HAVE_TERMIO_H */
 #	endif /* HAVE_TERMIOS_H && HAVE_TCGETATTR && HAVE_TCSETATTR */
@@ -167,7 +171,8 @@ static char *_clearscreen, *_moveto, *_cleartoeoln, *_cleartoeos,
 			*_setinverse, *_clearinverse, *_setunderline, *_clearunderline,
 			*_xclickinit, *_xclickend, *_cursoron, *_cursoroff,
 			*_terminalinit, *_terminalend, *_keypadlocal, *_keypadxmit,
-			*_scrollregion, *_scrollfwd, *_scrollback;
+			*_scrollregion, *_scrollfwd, *_scrollback,
+			*_reset, *_reversevideo, *_blink, *_dim, *_bold;
 
 #ifdef M_AMIGA
 static char *_getwinsize;
@@ -176,7 +181,6 @@ static char *_getwinsize;
 static int _columns, _line, _lines;
 
 #ifdef M_UNIX
-
 #	undef SET_TTY
 #	undef GET_TTY
 #	if USE_POSIX_TERMIOS
@@ -190,15 +194,18 @@ static int _columns, _line, _lines;
 #			if USE_SGTTY
 #				define SET_TTY(arg) stty(TTYIN, arg)
 #				define GET_TTY(arg) gtty(TTYIN, arg)
-#			else
-#				error "No termios.h, termio.h or sgtty.h found"
+/*
+	#			else
+	#				error "No termios.h, termio.h or sgtty.h found"
+*/
 #			endif /* USE_SGTTY */
 #		endif /* USE_TERMIO */
 #	endif /* USE_POSIX_TERMIOS */
-
 #endif /* M_UNIX */
 
-static int in_inverse;			/* 1 when in inverse, 0 otherwise */
+#if 0
+	static int in_inverse;			/* 1 when in inverse, 0 otherwise */
+#endif /* 0 */
 
 /*
  * Local prototypes
@@ -301,6 +308,12 @@ get_termcaps(
 		{ &_setunderline,	CAPNAME("us", "smul") },
 		{ &_terminalend,	CAPNAME("te", "rmcup") },
 		{ &_terminalinit,	CAPNAME("ti", "smcup") },
+		/* extra caps needed for word highlighting */
+		{ &_reset,		CAPNAME("me", "sgr0") },
+		{ &_reversevideo,	CAPNAME("mr", "rev") },
+		{ &_blink,		CAPNAME("mb", "blink") },
+		{ &_dim,		CAPNAME("mh", "dim") },
+		{ &_bold,		CAPNAME("mb", "bold") },
 	};
 
 	static char _terminal[1024];		/* Storage for terminal entry */
@@ -441,6 +454,12 @@ InitScreen(
 	_terminalinit	= NULL;
 	_terminalend	= "";
 #	endif /* VMS */
+	/* needed for word highlighting */
+	_reset		= "\033[0m";
+	_reversevideo	= "\033[7m";
+	_blink		= "\033[5m";
+	_dim		= "\033[2m";
+	_bold		= "\033[1m";
 
 	_lines = _columns = -1;
 
@@ -753,7 +772,7 @@ void
 StartInverse(
 	void)
 {
-	in_inverse = 1;
+/*	in_inverse = 1; */
 	if (_setinverse && tinrc.inverse_okay) {
 #	ifdef HAVE_COLOR
 		if (use_color) {
@@ -777,7 +796,7 @@ void
 EndInverse(
 	void)
 {
-	in_inverse = 0;
+/*	in_inverse = 0; */
 	if (_clearinverse && tinrc.inverse_okay) {
 #	ifdef HAVE_COLOR
 		if (use_color) {
@@ -794,6 +813,7 @@ EndInverse(
 }
 
 
+#if 0 /* doesn't work correct with ncurses4.x */
 /*
  * toggle inverse video mode
  */
@@ -806,6 +826,7 @@ ToggleInverse(
 	else
 		EndInverse();
 }
+#endif /* 0 */
 
 
 /*
@@ -837,7 +858,7 @@ Raw(
 #else
 #	if defined(M_AMIGA) && defined(__SASC)
 	_inraw = state;
-	rawcon (state);
+	rawcon(state);
 #	else
 	if (!state && _inraw) {
 		SET_TTY(&_original_tty);
@@ -900,7 +921,7 @@ PROFILE_ON();
 				if (result <= 0) return EOF;
 			} while (buf[buflen++] < 0x40);
 
-			switch (buf[buflen-1]) {
+			switch (buf[buflen - 1]) {
 				char *ptr;
 				long class;
 
@@ -1074,13 +1095,31 @@ highlight_string(
 	int col,
 	int size)
 {
-	char tmp[LEN];
+	char output[LEN];
+
+	my_strncpy(output, &(screen[row].col[col]), size);
+
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	/*
+	 * In a multibyte locale we get byte offsets instead of character
+	 * offsets; calculate now the correct starting column
+	 */
+	if (col > 0) {
+		char tmp[LEN];
+		wchar_t wtmp[LEN];
+
+		my_strncpy(tmp, &(screen[row].col[0]), sizeof(tmp) - 1);
+		tmp[col] = '\0';
+		if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+			wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
+			col = wcswidth(wtmp, ARRAY_SIZE(wtmp));
+		}
+	}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	MoveCursor(row, col);
-	my_strncpy(tmp, &(screen[row].col[col]), size);
-
 	StartInverse();
-	my_fputs(tmp, stdout);
+	my_fputs(output, stdout);
 	my_flush();
 	EndInverse();
 
@@ -1088,7 +1127,6 @@ highlight_string(
 }
 
 
-#ifdef HAVE_COLOR
 /*
  * Color 'size' chars at (row,col) with 'color' and handle marks
  */
@@ -1099,36 +1137,106 @@ word_highlight_string(
 	int size,
 	int color)
 {
-	char tmp[LEN];
+	/*
+	 * Mapping of the tinrc.mono_mark* values to the corresponding escape sequences
+	 */
+	char *attributes[MAX_ATTR + 1];
+	char *dest, *src;
+	char output[LEN];
+	int byte_offset = col;
+	int wsize = size;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t wtmp[LEN];
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	attributes[0] = _reset;	/* Normal */
+	attributes[1] = _setinverse;	/* Best highlighting */
+	attributes[2] = _setunderline;	/* Underline */
+	attributes[3] = _reversevideo;	/* Reverse video */
+	attributes[4] = _blink;	/* Blink */
+	attributes[5] = _dim;	/* Dim */
+	attributes[6] = _bold;	/* Bold */
+
+	my_strncpy(output, &(screen[row].col[byte_offset]), size);
+
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	/*
+	 * In a multibyte locale we get byte offsets instead of character
+	 * offsets; calculate now the correct starting column and
+	 * width
+	 */
+	if (byte_offset > 0) {
+		char tmp[LEN];
+		my_strncpy(tmp, &(screen[row].col[0]), sizeof(tmp) - 1);
+		tmp[byte_offset] = '\0';
+		if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+			wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
+			col = wcswidth(wtmp, ARRAY_SIZE(wtmp));
+		}
+	}
+	if (mbstowcs(wtmp, output, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+		wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
+		wsize = wcswidth(wtmp, ARRAY_SIZE(wtmp));
+	}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	MoveCursor(row, col);
-	my_strncpy(tmp, &(screen[row].col[col]), size);
 
-	/* safegurad against bogus regexps */
-	if ((tmp[0] == '*' && tmp[size - 1] == '*') ||
-		 (tmp[0] == '/' && tmp[size - 1] == '/') ||
-		 (tmp[0] == '_' && tmp[size - 1] == '_') ||
-		 (tmp[0] == '-' && tmp[size - 1] == '-')) {
+	/* safeguard against bogus regexps */
+	if ((output[0] == '*' && output[size - 1] == '*') ||
+		 (output[0] == '/' && output[size - 1] == '/') ||
+		 (output[0] == '_' && output[size - 1] == '_') ||
+		 (output[0] == '-' && output[size - 1] == '-')) {
 
 		switch (tinrc.word_h_display_marks) {
-			case 0: /* FIXME */
+			case 0: /* remove marks */
+				MoveCursor(row, col + wsize - 2);
+				CleartoEOLN();
+				my_fputs(&(screen[row].col[byte_offset + size]), stdout);
+				output[0] = output[size - 1] = ' ';
+				str_trim(output);
+				strncpy(&(screen[row].col[byte_offset]), output, size - 2);
+				src = &(screen[row].col[byte_offset + size]);
+				dest = &(screen[row].col[byte_offset + size - 2]);
+				while (*src)
+					*dest++ = *src++;
+				*dest++ = '\0';
+				MoveCursor(row, col);
 				break;
 
 			case 2: /* print space */
-				tmp[0] = tmp[size - 1] = ' ';
+				MoveCursor(row, col + wsize - 1);
+				my_fputs(" ", stdout);
+				MoveCursor(row, col);
+				my_fputs(" ", stdout);
+				output[0] = output[size - 1] = ' ';
+				str_trim(output);
+				screen[row].col[byte_offset] = ' ';
+				screen[row].col[byte_offset + size - 1] = ' ';
 				break;
 
 			default:	/* print mark (case 1) */
 				break;
 		}
 	}
-	fcol(color);
-	my_fputs(tmp, stdout);
+#	ifdef HAVE_COLOR
+	if (use_color)
+		fcol(color);
+	else
+#	endif /* HAVE_COLOR */
+		if (color > 0 && color <= MAX_ATTR && !NO_CAP(attributes[color]))
+			tputs(attributes[color], 1, outchar);
+	my_fputs(output, stdout);
 	my_flush();
-	fcol(tinrc.col_text);
+#	ifdef HAVE_COLOR
+	if (use_color)
+		fcol(tinrc.col_text);
+	else
+#	endif /* HAVE_COLOR */
+		if (!NO_CAP(_reset))
+			tputs(_reset, 1, outchar);
 	stow_cursor();
 }
-#endif /* HAVE_COLOR */
 
 
 /*
@@ -1180,7 +1288,7 @@ input_pending(
 #	if defined(HAVE_POLL) && !defined(HAVE_SELECT)
 	static int Timeout;
 	static long nfds = 1;
-	static struct pollfd fds[]= {{ STDIN_FILENO, POLLIN, 0 }};
+	static struct pollfd fds[] = {{ STDIN_FILENO, POLLIN, 0 }};
 
 	Timeout = delay;
 	if (poll(fds, nfds, Timeout) < 0) /* Error on poll */
@@ -1380,7 +1488,7 @@ int
 ReadCh(
 	void)
 {
-	register int result;
+	int result;
 #	ifndef READ_CHAR_HACK
 	char ch;
 #	endif /* READ_CHAR_HACK */
@@ -1426,19 +1534,41 @@ ReadCh(
 
 
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+/*
+ * A helper function for ReadWch()
+ * converts the read input to a wide-char
+ */
+static wint_t
+convert_c2wc(
+	const char *input)
+{
+	int res;
+	wchar_t wc;
+
+	res = mbtowc(&wc, input, MB_CUR_MAX);
+	if (res == -1)
+		return WEOF;
+	else
+		return (wint_t) wc;
+}
+
+
 wint_t
 ReadWch(
 	void)
 {
 	char *mbs = my_malloc(MB_CUR_MAX + 1);
-	int result, offset;
+	int result, to_read;
 	wchar_t wch = 0;
 
 	fflush(stdout);
 
-#		ifndef USE_CURSES
-		/* read only the first char */
-#			ifdef EINTR
+	/*
+	 * Independent of the pressed key and the used charset we have to
+	 * read at least one byte. The further processing is depending of
+	 * the read byte and the used charset.
+	 */
+#		ifdef EINTR
 	allow_resize(TRUE);
 	while ((result = read(0, mbs, 1)) < 0 && errno == EINTR) { /* spin on signal interrupts */
 		if (need_resize) {
@@ -1447,9 +1577,10 @@ ReadWch(
 		}
 	}
 	allow_resize(FALSE);
-#			else
-	result = read(0, mbs, sizeof(mbs));
-#			endif /* EINTR */
+#		else
+	result = read(0, mbs, 1);
+#		endif /* EINTR */
+
 	if (result <= 0) {
 		free(mbs);
 		return WEOF;
@@ -1461,48 +1592,78 @@ ReadWch(
 		return (wint_t) ESC;
 	}
 
-	/* the pressed key can be represented in one byte */
-	if (!input_pending(0)) {
-		char out = mbs[0] & 0xff;
-
+	/*
+	 * In an one-byte charset we don't need to read further chars but
+	 * we still need to convert the char to a correct wide-char
+	 */
+	if (MB_CUR_MAX == 1) {
+		mbs[1] = '\0';
+		wch = convert_c2wc(mbs);
 		free(mbs);
-		return (wint_t) out;
+
+		return (wint_t) wch;
 	}
 
-	/* read the rest of the multibyte-sequence */
-	offset = 1;
-#		else
-	/* read the complete input */
-	offset = 0;
-#		endif /* !USE_CURSES */
+	/*
+	 * we use a multi-byte charset
+	 */
 
-#		ifdef EINTR
-	allow_resize(TRUE);
-	while ((result = read(0, mbs + offset, MB_CUR_MAX)) < 0 && errno == EINTR) { /* spin on signal interrupts */
-		if (need_resize) {
-			handle_resize((need_resize == cRedraw) ? TRUE :  FALSE);
-			need_resize = cNo;
-		}
-	}
-	allow_resize(FALSE);
-#		else
-	result = read(0, mbs + offset, MB_CUR_MAX);
-#		endif /* EINTR */
+	/*
+	 * UTF-8
+	 */
+	if (!strcasecmp(tinrc.mm_local_charset, "UTF-8")) {
+		int ch = mbs[0] & 0xFF;
 
-	if (result <= 0) {
-		free(mbs);
-		return WEOF;
-	} else {
-		int res;
-
-		mbs[result + offset] = '\0';
-		res = mbtowc(&wch, mbs, MB_CUR_MAX);
-		free(mbs);
-		if (res == -1)
+		/* determine the count of bytes we have still have to read */
+		if (ch <= 0x7F) {
+			/* ASCII char */
+			to_read = 0;
+		} else if (ch >= 0xC2 && ch <= 0xDF) {
+			/* 2-byte sequence */
+			to_read = 1;
+		} else if (ch >= 0xE0 && ch <= 0xEF) {
+			/* 3-byte sequence */
+			to_read = 2;
+		} else if (ch >= 0xF0 && ch <= 0xF4) {
+			/* 4-byte sequence */
+			to_read = 3;
+		} else {
+			/* invalid sequence */
+			free(mbs);
 			return WEOF;
-		else
-			return (wint_t) wch;
+		}
+
+		/* read the missing bytes of the multi-byte sequence */
+		if (to_read > 0) {
+#		ifdef EINTR
+			allow_resize(TRUE);
+			while ((result = read(0, mbs + 1, to_read)) < 0 && errno == EINTR) { /* spin on signal interrupts */
+				if (need_resize) {
+					handle_resize((need_resize == cRedraw) ? TRUE :  FALSE);
+					need_resize = cNo;
+				}
+			}
+			allow_resize(FALSE);
+#		else
+			result = read(0, mbs + 1, to_read);
+#		endif /* EINTR */
+			if (result < 0) {
+				free(mbs);
+
+				return WEOF;
+			}
+		}
+		mbs[to_read + 1] = '\0';
+		wch = convert_c2wc(mbs);
+		free (mbs);
+
+		return (wint_t) wch;
+
 	}
+
+	/* FIXME: add support for other multi-byte charsets */
+
+	return WEOF;
 }
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #endif /* M_UNIX */
