@@ -3,7 +3,7 @@
  *  Module    : feed.c
  *  Author    : I. Lea
  *  Created   : 1991-08-31
- *  Updated   : 2002-12-09
+ *  Updated   : 2003-07-19
  *  Notes     : provides same interface to mail,pipe,print,save & repost commands
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -54,40 +54,40 @@
 #endif /* !RFC2046_H */
 
 
-t_bool is_mailbox = FALSE;
-
-/*
- * Post-processing type when saving
- * Set to <0 before post proc type is selected
- * Set to 0 if user aborted save process
- * We do this hackery to honour the "don't postprocess mailboxes" rule
- */
-static signed char proc_ch;
-static t_bool confirm;
+static char proc_ch;					/* Post-processing type when saving */
+static t_bool confirm;					/* only used for FEED_MAIL */
+static t_bool is_mailbox = FALSE;
 static t_bool redraw_screen = FALSE;
-static t_bool supersede = FALSE;			/* for reposting only */
+static t_bool supersede = FALSE;		/* for reposting only */
 #ifndef DONT_HAVE_PIPING
 	static FILE *pipe_fp = (FILE *) 0;
 #endif /* !DONT_HAVE_PIPING */
 
 
+struct t_counters {
+	int success;		/* # arts fed okay */
+	int total;			/* # arts fed */
+	int max;			/* initial guesstimate of total */
+};
+
 /*
  * Local prototypes
  */
-static char *get_save_filename(struct t_group *group, int function, char *filename, int filelen);
+static char *get_save_filename(struct t_group *group, int function, char *filename, int filelen, int respnum);
+static int get_feed_key(int function, int level, struct t_group *group, struct t_art_stat *thread, int respnum);
 static int get_post_proc_type(void);
-static t_bool feed_article(int art, int function, int *curr, int max, t_bool use_current, const char *data, const char *path);
+static t_bool feed_article(int art, int function, struct t_counters *counter, t_bool use_current, const char *data, struct t_group *group);
+static void print_save_summary(char type, int fed);
 #ifndef DISABLE_PRINTING
 	static t_bool print_file(const char *command, int respnum, t_openartinfo *artinfo);
 #endif /* !DISABLE_PRINTING */
 
 #ifndef DONT_HAVE_PIPING
-#	define handle_SIGPIPE() if (got_sig_pipe) goto got_sig_pipe_while_piping
+#	define handle_SIGPIPE()	if (got_sig_pipe) goto got_sig_pipe_while_piping
 #else
 #	define handle_SIGPIPE() /*nothing*/
 #endif /* DONT_HAVE_PIPING */
 
-
 /*
  * 'filename' holds 'filelen' amount of storage in which to place the
  * filename to save-to. The filename is also returned after basic syntax
@@ -99,15 +99,22 @@ get_save_filename(
 	struct t_group *group,
 	int function,
 	char *filename,
-	int filelen)
+	int filelen,
+	int respnum)
 {
 	char default_savefile[PATH_LEN];
 
 	filename[0] = '\0';
 
+	/*
+	 * Group attribute savefile overrides tinrc default savefile
+	 */
 	strncpy(default_savefile, (group->attribute->savefile ? group->attribute->savefile : tinrc.default_save_file), sizeof(default_savefile) - 1);
 
-	if (function != FEED_AUTOSAVE_TAGGED) {			/* ie, FEED_SAVE */
+	/*
+	 * We don't ask when auto'S'aving or Archive-Name saving with auto_save
+	 */
+	if (!(function == FEED_AUTOSAVE || (group->attribute->auto_save && arts[respnum].archive))) {
 		if (!prompt_default_string(_(txt_save_filename), filename, filelen, default_savefile, HIST_SAVE_FILE)) {
 			clear_message();
 			return NULL;
@@ -115,15 +122,21 @@ get_save_filename(
 		str_trim(filename);
 	}
 
+	/*
+	 * Update attribute/tinrc default savefiles if changed
+	 */
 	if (*filename) {
 		if (group->attribute->savefile) {
 			free(group->attribute->savefile);
 			group->attribute->savefile = my_strdup(filename);
 		}
-		my_strncpy(tinrc.default_save_file, filename, sizeof(tinrc.default_save_file));
+		my_strncpy(tinrc.default_save_file, filename, sizeof(tinrc.default_save_file) - 1);
 	} else {									/* No file was specified, try default */
+		/*
+		 * None chosen (or AUTOSAVING), use tinrc default
+		 */
 		if (*default_savefile)
-			my_strncpy(filename, default_savefile, sizeof(filename) - 1);
+			my_strncpy(filename, default_savefile, filelen - 1);
 		else {									/* No default either */
 			info_message(_(txt_no_filename));
 			return NULL;
@@ -143,25 +156,25 @@ get_save_filename(
 
 /*
  * Find out what post-processing to perform.
- * This is not used when saving to mailboxes
- * Also not used when using the auto-save feature as a default value is taken
- * from the group attributes
- * Return a post_proc char or 0 if aborting the save process
+ * This is not used when saving to mailboxes (we don't postprocess mailboxen)
+ * Also not used when using the auto-save feature because a default value is
+ * taken from the group attributes
+ * Return a post_proc_char or 0 if aborting the save process
  */
 static int
 get_post_proc_type(
 	void)
 {
 	char ch;
-	char keynone[MAXKEYLEN], keyquit[MAXKEYLEN], keyshar[MAXKEYLEN];
-	char keyuud[MAXKEYLEN];
+	char keyno[MAXKEYLEN], keyyes[MAXKEYLEN], keyquit[MAXKEYLEN];
+	char keyshar[MAXKEYLEN];
 
-	ch = (char) prompt_slk_response(proc_ch_default,
+	ch = (char) prompt_slk_response(ch_post_process[curr_group->attribute->post_proc_type],
 				&menukeymap.feed_post_process_type,
 				_(txt_choose_post_process_type),
-				printascii(keynone, map_to_local(iKeyPProcNone, &menukeymap.feed_post_process_type)),
+				printascii(keyno, map_to_local(iKeyPProcNo, &menukeymap.feed_post_process_type)),
+				printascii(keyyes, map_to_local(iKeyPProcYes, &menukeymap.feed_post_process_type)),
 				printascii(keyshar, map_to_local(iKeyPProcShar, &menukeymap.feed_post_process_type)),
-				printascii(keyuud, map_to_local(iKeyPProcUUDecode, &menukeymap.feed_post_process_type)),
 				printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.feed_post_process_type)));
 
 	if (ch == iKeyQuit || ch == iKeyAbort) {			/* exit */
@@ -173,167 +186,23 @@ get_post_proc_type(
 
 
 /*
- * This is the handler that processes a single article for all the
- * various FEED_ functions. 'art' is the index in arts[]
- * Assumes no article is open when we enter - opens and closes the art being
- * processed. As a performance hack this is not done if 'use_current' is set.
- * Returns TRUE if the article was processed okay
+ * Return the key mapping for what we are intending to process or 0 if save
+ * process is being aborted
+ * Key can be (current) article, (current) thread, tagged articles,
+ * hot articles, or articles matching a pattern
+ * This is automatic in the various auto-save cases, in other
+ * cases this is prompted for based on a chosen default
  */
-static t_bool
-feed_article(
-	int art,
-	int function,
-	int *curr,				/* Current # being processed */
-	int max,				/* Total # items being processed, if known */
-	t_bool use_current,		/* Use already open pager article */
-	const char *data,		/* Extra data if needed, print command or save filename */
-	const char *path)
-{
-	t_bool ok = TRUE;		/* Assume success */
-	t_openartinfo openart;
-	t_openartinfo *openartptr = &openart;
-
-	/*
-	 * The save function add_to_save_list() only queues up a batch and doesn't
-	 * need an open article. The actual transfer of data comes later
-	 */
-	if (use_current)
-		openartptr = &pgart;			/* Use art open in pager */
-	else {
-		if (function == FEED_SAVE || function == FEED_AUTOSAVE_TAGGED) {
-			/* When saving, just check that it exists for now */
-			if (!stat_article(arts[art].artnum, path))
-				return FALSE;
-		} else {
-			memset(openartptr, 0, sizeof(t_openartinfo));
-			if (art_open(FALSE, &arts[art], path, openartptr, TRUE) == ART_UNAVAILABLE)
-				return FALSE;
-		}
-	}
-
-	switch (function) {
-		case FEED_MAIL:
-			switch (mail_to_someone(tinrc.default_mail_address, confirm, openartptr)) {
-				case POSTED_REDRAW:
-					redraw_screen = TRUE;
-					/* FALLTHROUGH */
-				case POSTED_NONE:
-					ok = FALSE;
-					break;
-
-				case POSTED_OK:
-					break;
-			}
-
-			confirm = bool_not(ok);		/* Only confirm the next one after a failure */
-			break;
-
-#ifndef DONT_HAVE_PIPING
-		case FEED_PIPE:
-			/* TODO - looks odd because screen mode is raw */
-			if (max)
-				show_progress(_(txt_piping), (*curr) + 1, max);
-			else
-				wait_message(0, "%s %d\n", _(txt_piping), (*curr) + 1);
-
-			rewind(openartptr->raw);
-			ok = copy_fp(openartptr->raw, pipe_fp);	/* Check for SIGPIPE on return */
-			break;
-#endif /* !DONT_HAVE_PIPING */
-
-#ifndef DISABLE_PRINTING
-		case FEED_PRINT:
-			/* TODO - looks odd because screen mode is raw */
-			if (max)
-				show_progress(_(txt_printing), (*curr) + 1, max);
-			else
-				wait_message(0, "%s %d\n", _(txt_printing), (*curr) + 1);
-
-			ok = print_file(data /*print_command*/, art, openartptr);
-			break;
-#endif /* !DISABLE_PRINTING */
-
-		/* add_to_save_list() is instant, the actual saving happens later */
-		case FEED_SAVE:
-			if (!(is_mailbox = add_to_save_list(&arts[art], data /*filename*/))) {
-				if (proc_ch < 0 && (proc_ch = get_post_proc_type()) == 0)
-					ok = FALSE;
-			}
-			break;
-
-		case FEED_AUTOSAVE_TAGGED:
-			add_to_save_list(&arts[art], data /*filename*/);
-			break;
-
-		case FEED_REPOST:
-			if (repost_article(tinrc.default_repost_group, art, supersede, openartptr) == POSTED_NONE)
-				ok = FALSE;
-			else			/* POSTED_REDRAW, POSTED_OK */
-				redraw_screen = TRUE;
-			break;
-
-		default:
-			break;
-	}
-	if (ok)
-		(*curr)++;
-
-	/*
-	 * Don't mark read when saving - this is only done by save_art_to_file()
-	 * Likewise, no art is open at this stage when saving
-	 */
-	if (!(function == FEED_SAVE || function == FEED_AUTOSAVE_TAGGED)) {
-		if (!use_current)
-			art_close(openartptr);
-		if (ok && tinrc.mark_saved_read)
-			art_mark_read(&CURR_GROUP, &arts[art]);
-	}
-	return ok;
-}
-
-
-void
-feed_articles(
+static int
+get_feed_key(
 	int function,
 	int level,
 	struct t_group *group,
+	struct t_art_stat *thread,
 	int respnum)
 {
-	char group_path[PATH_LEN];
-	char output[PATH_LEN];
 	constext *prompt;
 	int ch, ch_default;
-	int i, j;
-	int count = 0;
-	int thread_base;
-	int saved_curr_line = -1;
-	t_bool use_current = FALSE;
-	t_bool processed_ok = TRUE;
-	t_bool ret1 = FALSE;
-	t_bool ret2 = FALSE;
-
-#ifdef DONT_HAVE_PIPING
-	if (function == FEED_PIPE) {
-		error_message(_(txt_piping_not_enabled));
-		clear_message();
-		return;
-	}
-#endif /* DONT_HAVE_PIPING */
-
-	set_xclick_off();
-
-	thread_base = which_thread(respnum);
-
-	proc_ch = -1;			/* No selection made yet */
-
-	/*
-	 * try and work out what default the user wants
-	 * This is dumb. If you have _any_ selected arts, then it picks 'h'
-	 * No it's not, that's exactly what you want (in binary groups)
-	 */
-	ch_default = (num_of_tagged_arts ? iKeyFeedTag :
-					(level == GROUP_LEVEL && HAS_FOLLOWUPS(thread_base) ? iKeyFeedThd :
-						(num_of_selected_arts ? iKeyFeedHot : iKeyFeedArt)));
 
 	switch (function) {
 		case FEED_MAIL:
@@ -352,6 +221,7 @@ feed_articles(
 			break;
 #endif /* !DISABLE_PRINTING */
 
+		/* FEED_AUTOSAVE doesn't prompt */
 		case FEED_SAVE:
 			prompt = txt_save;
 			break;
@@ -359,7 +229,7 @@ feed_articles(
 		case FEED_REPOST:
 			if (!can_post) {				/* Get this over with before asking any Q's */
 				info_message(_(txt_cannot_post));
-				return;
+				return 0;
 			}
 			prompt = txt_repost;
 			break;
@@ -369,45 +239,299 @@ feed_articles(
 			break;
 	}
 
-	make_group_path(group->name, group_path);
+	/*
+	 * Try and work out what default the user wants
+	 * thread->total = # arts in thread
+	 */
+	ch_default = (num_of_tagged_arts ? iKeyFeedTag :
+					(arts_selected() ? iKeyFeedHot :
+					((level == GROUP_LEVEL && thread->total > 1) ? iKeyFeedThd :
+					(thread->selected_total ? iKeyFeedHot :
+					iKeyFeedArt))));
 
 	/*
-	 * If not automatic, ask what the user wants to save
+	 * Don't bother querying when:
+	 *  auto'S'aving and there are tagged or selected(hot) articles
+	 *  using the auto_save feature on Archive postings
 	 */
-	if ((!(group->attribute->auto_save && arts[respnum].archive) || (group->attribute->auto_save && function != FEED_SAVE) || ch_default == iKeyFeedTag) && function != FEED_AUTOSAVE_TAGGED) {
+	if ((function == FEED_AUTOSAVE && (num_of_tagged_arts || arts_selected())) ||
+			(group->attribute->auto_save && arts[respnum].archive))
+		ch = ch_default;
+	else {
 		char buf[LEN];
 		char keyart[MAXKEYLEN], keythread[MAXKEYLEN], keyhot[MAXKEYLEN];
 		char keypat[MAXKEYLEN], keytag[MAXKEYLEN], keyquit[MAXKEYLEN];
 
-		snprintf(buf, sizeof(buf) - 1, _(txt_art_thread_regex_tag),
+		snprintf(buf, sizeof(buf), _(txt_art_thread_regex_tag),
 			printascii(keyart, map_to_local(iKeyFeedArt, &menukeymap.feed_art_thread_regex_tag)),
 			printascii(keythread, map_to_local(iKeyFeedThd, &menukeymap.feed_art_thread_regex_tag)),
 			printascii(keyhot, map_to_local(iKeyFeedHot, &menukeymap.feed_art_thread_regex_tag)),
 			printascii(keypat, map_to_local(iKeyFeedPat, &menukeymap.feed_art_thread_regex_tag)),
 			printascii(keytag, map_to_local(iKeyFeedTag, &menukeymap.feed_art_thread_regex_tag)),
 			printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.feed_art_thread_regex_tag)));
-		ch = prompt_slk_response(ch_default,
-				&menukeymap.feed_art_thread_regex_tag,
-				"%s %s", _(prompt), buf);
-	} else
-		ch = ch_default;
+
+		ch = prompt_slk_response(ch_default, &menukeymap.feed_art_thread_regex_tag, "%s %s", _(prompt), buf);
+	}
 
 	switch (ch) {
 		case iKeyQuit:
 		case iKeyAbort:
 			clear_message();
-			return;
+			return 0;
 
 		case iKeyFeedPat:
 			sprintf(mesg, _(txt_feed_pattern), tinrc.default_pattern);
 			if (!(prompt_string_default(mesg, tinrc.default_pattern, _(txt_no_match), HIST_REGEX_PATTERN)))
-				return;
+				return 0;
 			break;
 
 		default:
 			break;
 	}
 
+	return ch;
+}
+
+
+/*
+ * Print a message like:
+ * -- [Article|Thread|Tagged Articles] saved to [mailbox] [filenames] --
+ * 'fed' is the number of articles we tried to save
+ */
+static void
+print_save_summary(
+	char type,
+	int fed)
+{
+	const char *first, *last;
+	char buf[LEN];
+	char what[LEN];
+
+	if (fed != num_save)
+		wait_message(2, _(txt_warn_not_all_arts_saved), fed, num_save);
+
+	switch (type) {
+		case iKeyFeedHot:
+			snprintf(what, sizeof(what), _(txt_prefix_hot), PLURAL(fed, txt_article));
+			break;
+
+		case iKeyFeedTag:
+			snprintf(what, sizeof(what), _(txt_prefix_tagged), PLURAL(fed, txt_article));
+			break;
+
+		case iKeyFeedThd:
+			STRCPY(what, _(txt_thread_upper));
+			break;
+
+		case iKeyFeedArt:
+		case iKeyFeedPat:
+		default:
+			snprintf(what, sizeof(what), "%s", PLURAL(fed, txt_article));
+			break;
+	}
+
+	first = (save[0].mailbox) ? save[0].path : save[0].file;
+	last = (save[num_save - 1].mailbox) ? save[num_save - 1].path : save[num_save - 1].file;
+
+	/*
+	 * We report the range of saved-to files for regular saves of > 1 articles
+	 */
+	if (num_save == 1 || save[0].mailbox)
+		snprintf(buf, sizeof(buf), _(txt_saved_to),
+			what, (save[0].mailbox ? _(txt_mailbox) : ""), first);
+	else
+		snprintf(buf, sizeof(buf), _(txt_saved_to_range),
+			what, first, last);
+
+	wait_message((tinrc.beginner_level) ? 2 : 1, buf);
+
+	return;
+}
+
+
+/*
+ * This is the handler that processes a single article for all the various
+ * FEED_ functions.
+ * Assumes no article is open when we enter - opens and closes the art being
+ * processed. As a performance hack this is not done if 'use_current' is set.
+ * Returns TRUE or FALSE
+ * TODO: option to mail/pipe/print/repost raw vs. cooked?
+ *       (all currently raw only) or should we feed according to what
+ *       is currently on screen?
+ */
+static t_bool
+feed_article(
+	int art,				/* index in arts[] */
+	int function,
+	struct t_counters *counter,	/* Accounting */
+	t_bool use_current,		/* Use already open pager article */
+	const char *data,		/* Extra data if needed, print command or save filename */
+	struct t_group *group)
+{
+	t_bool ok = TRUE;		/* Assume success */
+	t_openartinfo openart;
+	t_openartinfo *openartptr = &openart;
+
+	counter->total++;
+
+	/*
+	 * Update the on-screen progress before art_open(), which is the bottleneck
+	 * timewise
+	 */
+	switch (function) {
+#ifndef DONT_HAVE_PIPING
+		case FEED_PIPE:
+			/* TODO: looks odd because screen mode is raw */
+			wait_message(0, "%s (%d/%d)", _(txt_piping), counter->total, counter->max);
+			break;
+#endif /* !DONT_HAVE_PIPING */
+
+#ifndef DISABLE_PRINTING
+		case FEED_PRINT:
+			/* TODO: looks odd because screen mode is raw */
+			wait_message(0, "%s (%d/%d)", _(txt_printing), counter->total, counter->max);
+			break;
+#endif /* !DISABLE_PRINTING */
+
+		case FEED_SAVE:
+		case FEED_AUTOSAVE:
+			wait_message(0, "%s (%d/%d)", _(txt_saving), counter->total, counter->max);
+			break;
+	}
+
+	if (use_current)
+		openartptr = &pgart;			/* Use art already open in pager */
+	else {
+		if (art_open(FALSE, &arts[art], group, openartptr, TRUE) < 0)	/* User abort or an error */
+			return FALSE;
+	}
+
+	switch (function) {
+		case FEED_MAIL:
+			switch (mail_to_someone(tinrc.default_mail_address, confirm, openartptr)) {
+				case POSTED_REDRAW:
+					redraw_screen = TRUE;
+					/* FALLTHROUGH */
+				case POSTED_NONE:
+					ok = FALSE;
+					break;
+
+				case POSTED_OK:
+					break;
+			}
+			confirm = bool_not(ok);		/* Only confirm the next one after a failure */
+			break;
+
+#ifndef DONT_HAVE_PIPING
+		case FEED_PIPE:
+			rewind(openartptr->raw);
+			ok = copy_fp(openartptr->raw, pipe_fp);	/* Check for SIGPIPE on return */
+			break;
+#endif /* !DONT_HAVE_PIPING */
+
+#ifndef DISABLE_PRINTING
+		case FEED_PRINT:
+			ok = print_file(data /*print_command*/, art, openartptr);
+			break;
+#endif /* !DISABLE_PRINTING */
+
+		case FEED_SAVE:
+		case FEED_AUTOSAVE:
+			ok = save_and_process_art(openartptr, &arts[art], is_mailbox, data /*filename*/, counter->max, (proc_ch != iKeyPProcNo));
+			break;
+
+		case FEED_REPOST:
+			if (repost_article(tinrc.default_repost_group, art, supersede, openartptr) == POSTED_NONE)
+				ok = FALSE;
+			else			/* POSTED_REDRAW, POSTED_OK */
+				redraw_screen = TRUE;
+			break;
+
+		default:
+			break;
+	}
+	if (ok)
+		counter->success++;
+
+	/*
+	 * Mark read for the SAVE cases (but not print/pipe etc..)
+	 */
+	if (function == FEED_SAVE || function == FEED_AUTOSAVE) {
+		if (ok && tinrc.mark_saved_read)
+			art_mark(&CURR_GROUP, &arts[art], ART_READ);
+	}
+
+	if (!use_current)
+		art_close(openartptr);
+	return ok;
+}
+
+
+/*
+ * Single entry point for 'feed'ing article(s) to a backend
+ * Function:
+ *	FEED_PIPE, FEED_MAIL, FEED_PRINT, FEED_REPOST
+ *	FEED_SAVE, FEED_AUTOSAVE
+ * Level:
+ *	GROUP_LEVEL, THREAD_LEVEL, PAGE_LEVEL
+ * Respnum:
+ *	Index in arts[] of starting article
+ *
+ * The following 'groups' of article can be processed:
+ *	Single (current) article
+ *	Current thread
+ *	Tagged articles
+ *	Hot articles
+ *	Articles matching a pattern
+ *
+ * The selection of Function depends on the key used to get here.
+ * The selection of which article 'group' to process is managed
+ * inside here, or by defaults.
+ */
+void
+feed_articles(
+	int function,
+	int level,
+	struct t_group *group,
+	int respnum)
+{
+	char outpath[PATH_LEN];
+	int art;
+	int feed_type;
+	int i;
+	int saved_curr_line = -1;
+	int thread_base;
+	struct t_art_stat sbuf;
+	struct t_counters counter = { 0, 0, 0 };
+	t_bool use_current = FALSE;
+	t_bool ret1 = FALSE;
+	t_bool post_processed_ok = FALSE;
+
+#ifdef DONT_HAVE_PIPING
+	if (function == FEED_PIPE) {
+		error_message(_(txt_piping_not_enabled));
+		clear_message();
+		return;
+	}
+#endif /* DONT_HAVE_PIPING */
+
+	if (function == FEED_AUTOSAVE) {
+		if (num_of_tagged_arts == 0 && !arts_selected()) {
+			info_message(_(txt_no_marked_arts));
+			return;
+		}
+	}
+
+	set_xclick_off();		/* TODO: there is no corresponding set_xclick_on()? */
+	thread_base = which_thread(respnum);
+	stat_thread(thread_base, &sbuf);
+
+	if ((feed_type = get_feed_key(function, level, group, &sbuf, respnum)) == 0)
+		return;
+
+	/*
+	 * Get whatever information is needed to proceed
+	 */
 	switch (function) {
 		/* Setup mail - get address to mail to */
 		case FEED_MAIL:
@@ -437,21 +561,45 @@ feed_articles(
 #endif /* !DONT_HAVE_PIPING */
 
 #ifndef DISABLE_PRINTING
+		/* Setup printing - get print command line */
 		case FEED_PRINT:
-			/* Setup print - get print command line */
-			sprintf(output, "%s %s", tinrc.printer, REDIRECT_OUTPUT);
+			sprintf(outpath, "%s %s", tinrc.printer, REDIRECT_OUTPUT);
 			break;
 #endif /* !DISABLE_PRINTING */
 
-		case FEED_SAVE:		/* ask user for filename to save to */
-		case FEED_AUTOSAVE_TAGGED:
-			if (!group->attribute->auto_save || (arts[respnum].archive == NULL)) {
-				if (get_save_filename(group, function, output, sizeof(output)) == NULL)
+		/*
+		 * Setup saving, some of these are generated automatically
+		 *	Determine path/file to save to
+		 *	Determine post-processing type
+		 *	Determine if post processed file deletion required
+		 */
+		case FEED_SAVE:
+		case FEED_AUTOSAVE:
+			{
+				char savefile[PATH_LEN];
+
+				/* This will force automatic selection unless changed by user */
+				savefile[0] = '\0';
+
+				if (get_save_filename(group, function, savefile, sizeof(savefile), respnum) == NULL)
+					return;
+
+				proc_ch = ch_post_process[curr_group->attribute->post_proc_type];
+
+				/* We don't postprocess mailboxen */
+				if ((is_mailbox = expand_save_filename(outpath, savefile)) == TRUE)
+					proc_ch = iKeyPProcNo;
+				else {
+					if (function != FEED_AUTOSAVE && (proc_ch = get_post_proc_type()) == 0)
+						return;
+				}
+				if (!create_path(outpath))
 					return;
 			}
 			break;
 
-		case FEED_REPOST:	/* repost article */
+		/* repost (or supersede) article */
+		case FEED_REPOST:
 			{
 #ifndef FORGERY
 				char from_name[PATH_LEN];
@@ -511,99 +659,84 @@ feed_articles(
 	 * Performance hack - If we feed a single art from the pager then we can
 	 * re-use the currently open article
 	 */
-	if (level == PAGE_LEVEL && ch == iKeyFeedArt) {
+	if (level == PAGE_LEVEL && feed_type == iKeyFeedArt) {
 		saved_curr_line = curr_line;		/* Save where we were in pager */
 		use_current = TRUE;
 	}
 
-	switch (ch) {
+	/*
+	 * This is the main loop
+	 * The general idea is to feed_article() for every article to be processed
+	 */
+	switch (feed_type) {
 		case iKeyFeedArt:		/* article */
-			if (!feed_article(respnum, function, &count, 1, use_current, output, group_path)) {
+			counter.max = 1;
+			if (!feed_article(respnum, function, &counter, use_current, outpath, group))
 				handle_SIGPIPE();
-				/* No point testing proc_ch when handling only a single art */
-				break;
-			}
-
-			if (proc_ch && function == FEED_SAVE) {
-				if (use_current) {
-					if (create_path(save[0].path))
-						processed_ok = save_art_to_file(0, &pgart);
-				} else
-					processed_ok = save_batch(ch, group_path);
-			}
 			break;
 
 		case iKeyFeedThd:		/* thread */
-			/* Could stat_thread() and pass in 'max' to feed_article() */
-			for_each_art_in_thread(i, thread_base) {
-				if (tinrc.process_only_unread && arts[i].status == ART_READ)
-					continue;
-
-				/* Ignore errors */
-				feed_article(i, function, &count, 0, use_current, output, group_path);
-				if (proc_ch == 0)
-					break;
-
-				handle_SIGPIPE();
+			/* Get accurate count first */
+			for_each_art_in_thread(art, which_thread(respnum)) {
+				if (!(tinrc.process_only_unread && arts[art].status == ART_READ))
+					counter.max++;
 			}
-			if (proc_ch && function == FEED_SAVE) {
-				/* Can't see any need for this with real threading */
-				/*	sort_save_list(); */
-				processed_ok = save_batch(ch, group_path);
+
+			for_each_art_in_thread(art, which_thread(respnum)) {
+				if (!(tinrc.process_only_unread && arts[art].status == ART_READ)) {
+					/* Keep going - don't abort on errors */
+					if (!feed_article(art, function, &counter, use_current, outpath, group))
+						handle_SIGPIPE();
+				}
 			}
 			break;
 
 		case iKeyFeedTag:		/* tagged articles */
+			counter.max = num_of_tagged_arts;
 			for (i = 1; i <= num_of_tagged_arts; i++) {
-				for_each_art(j) {
-					if (arts[j].tagged == i) {
-						/* Ignore errors */
-						feed_article(j, function, &count, num_of_tagged_arts, use_current, output, group_path);
-						if (proc_ch == 0) {
-							i = num_of_tagged_arts + 1;		/* Force break out of outer-loop */
-							break;
-						}
-						handle_SIGPIPE();
+				for_each_art(art) {
+					/* process_only_unread does NOT apply on tagged arts */
+					if (arts[art].tagged == i) {
+						/* Keep going - don't abort on errors */
+						if (!feed_article(art, function, &counter, use_current, outpath, group))
+							handle_SIGPIPE();
 					}
 				}
 			}
-
-			if (proc_ch && (function == FEED_SAVE || function == FEED_AUTOSAVE_TAGGED))
-				processed_ok = save_batch(ch, group_path);
-
-			untag_all_articles();
+			untag_all_articles();	/* TODO: this will untag even on partial failure */
 			break;
 
 		case iKeyFeedHot:		/* hot (auto-selected) articles */
 		case iKeyFeedPat:		/* pattern matched articles */
+			for_each_art(art) {
+				if (feed_type == iKeyFeedPat) {
+					if (!REGEX_MATCH(arts[art].subject, tinrc.default_pattern, TRUE))
+						continue;
+				} else if (!arts[art].selected)
+					continue;
+
+				if (tinrc.process_only_unread && arts[art].status == ART_READ)
+					continue;
+
+				arts[art].matched = TRUE;
+				counter.max++;
+			}
+
+			/* I think we nest like this to preserve any 'ordering' of the arts */
 			for (i = 0; i < grpmenu.max; i++) {
-				for_each_art_in_thread(j, i) {
-					if (ch == iKeyFeedPat) {
-						if (!REGEX_MATCH(arts[j].subject, tinrc.default_pattern, TRUE))
-							continue;
-					} else if (!arts[j].selected)
+				for_each_art_in_thread(art, i) {
+					if (!arts[art].matched)
 						continue;
+					arts[art].matched = FALSE;
 
-					if (tinrc.process_only_unread && arts[j].status == ART_READ)
-						continue;
-
-					if (feed_article(j, function, &count, 0, use_current, output, group_path)) {
-						if (ch == iKeyFeedHot) {
-							arts[j].selected = FALSE;
-							num_of_selected_arts--;
-						}
-					} else {
-						if (proc_ch == 0) {
-							i = grpmenu.max;		/* Force break out of outer-loop */
-							break;
-						}
+					/* Keep going - don't abort on errors */
+					if (feed_article(art, function, &counter, use_current, outpath, group)) {
+						if (feed_type == iKeyFeedHot)
+							arts[art].selected = FALSE;
+					} else
 						handle_SIGPIPE();
-					}
 				}
 			}
-			if (proc_ch && function == FEED_SAVE)
-				processed_ok = save_batch(ch, group_path);
-
 			break;
 
 		default:			/* Should never get here */
@@ -611,7 +744,8 @@ feed_articles(
 	} /* switch (ch) */
 
 	/*
-	 * Now work out what (if anything) needs to be redrawn
+	 * Invoke post-processing if needed
+	 * Work out what (if anything) needs to be redrawn
 	 */
 	if (!tinrc.use_mailreader_i)
 		redraw_screen |= mail_check();	/* in case of sending to oneself */
@@ -633,13 +767,25 @@ got_sig_pipe_while_piping:
 #endif /* !DONT_HAVE_PIPING */
 
 		case FEED_SAVE:
-		case FEED_AUTOSAVE_TAGGED:
-			if (proc_ch != 'n' && !is_mailbox && processed_ok) {
-				if (proc_ch < 0) {
-					if (proc_ch_default != 'n')
-					  ret2 = post_process_files(proc_ch_default, (function == FEED_SAVE ? FALSE : TRUE));
-				} else
-					ret2 = post_process_files(proc_ch, (function == FEED_SAVE ? FALSE : TRUE));
+		case FEED_AUTOSAVE:
+			if (num_save == 0) {
+				wait_message(1, _(txt_saved_nothing));
+				break;
+			}
+
+			print_save_summary(feed_type, counter.total);
+			if (proc_ch != iKeyPProcNo) {
+				t_bool delete_post_proc = FALSE;
+
+				if (CURR_GROUP.attribute->delete_tmp_files)
+					delete_post_proc = TRUE;
+				else {
+					if (function != FEED_AUTOSAVE) {
+						if (prompt_yn(cLINES, _(txt_delete_processed_files), TRUE) == 1)
+							delete_post_proc = TRUE;
+					}
+				}
+				post_processed_ok = post_process_files(proc_ch, delete_post_proc);
 			}
 			free_save_array();		/* NB: This is where num_save etc.. gets purged */
 			break;
@@ -651,7 +797,7 @@ got_sig_pipe_while_piping:
 	if (level != PAGE_LEVEL)
 		ret1 = tinrc.mark_saved_read;
 
-	if (ret1 || ret2)
+	if (ret1 || post_processed_ok)
 		redraw_screen = TRUE;
 
 	if (level == PAGE_LEVEL) {
@@ -675,22 +821,25 @@ got_sig_pipe_while_piping:
 			currmenu->redraw();
 	}
 
+	/*
+	 * Finally print a status message
+	 */
 	switch (function) {
 		case FEED_MAIL:
 			if (tinrc.use_mailreader_i)
 				info_message(_(txt_external_mail_done));
 			else
-				info_message(_(txt_articles_mailed), count, PLURAL(count, txt_article));
+				info_message(_(txt_articles_mailed), counter.success, PLURAL(counter.success, txt_article));
 			break;
 
 #ifndef DISABLE_PRINTING
 		case FEED_PRINT:
-			info_message(_(txt_articles_printed), count, PLURAL(count, txt_article));
+			info_message(_(txt_articles_printed), counter.success, PLURAL(counter.success, txt_article));
 			break;
 #endif /* !DISABLE_PRINTING */
 
-		case FEED_SAVE:		/* Reporting handled by save code */
-		case FEED_AUTOSAVE_TAGGED:
+		case FEED_SAVE:		/* Reporting done earlier */
+		case FEED_AUTOSAVE:
 		default:
 			break;
 	}
@@ -708,12 +857,12 @@ print_file(
 	t_bool ok = TRUE;
 	struct t_header *hdr = &artinfo->hdr;
 #	ifdef DONT_HAVE_PIPING
-	char cmd[255], file[255];
+	char cmd[PATH_LEN], file[PATH_LEN];
 #	endif /* DONT_HAVE_PIPING */
 
 #	ifdef DONT_HAVE_PIPING
-	snprintf(file, sizeof(file) - 1, TIN_PRINTFILE, respnum);
-	if ((fp = fopen(file, "w")) == NULL)
+	snprintf(file, sizeof(file), TIN_PRINTFILE, respnum);
+	if ((fp = fopen(file, "w")) == NULL) /* TODO: issue a more correct error message here */
 #	else
 	if ((fp = popen(command, "w")) == NULL)
 #	endif /* DONT_HAVE_PIPING */
@@ -745,7 +894,7 @@ print_file(
 	strncpy(cmd, command, sizeof(cmd) - 2);
 	strcat(cmd, " ");
 	strncat(cmd, file, sizeof(cmd) - strlen(cmd) - 1);
-	cmd[254] = '\0';
+	cmd[sizeof(cmd) - 1] = '\0';
 	invoke_cmd(cmd);
 	unlink(file);
 #	else
