@@ -3,7 +3,7 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-22
+ *  Updated   : 2003-08-10
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -58,12 +58,13 @@
  * local prototypes
  */
 static int read_site_config(void);
-
 #ifdef HAVE_COLOR
 	static void preinit_colors(void);
 #endif /* HAVE_COLOR */
 
 
+char **news_headers_to_display_array;	/* array of which headers to display */
+char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char active_times_file[PATH_LEN];
 char article[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
 char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
@@ -80,47 +81,42 @@ char global_config_file[PATH_LEN];
 char homedir[PATH_LEN];
 char host_name[MAXHOSTNAMELEN];
 char index_maildir[PATH_LEN];
-char index_newsdir[PATH_LEN];
+char index_newsdir[PATH_LEN];	/* directory for private overview data */
 char index_savedir[PATH_LEN];
 char inewsdir[PATH_LEN];
 char libdir[PATH_LEN];			/* directory where news config files are (ie. active) */
 char local_attributes_file[PATH_LEN];
 char local_config_file[PATH_LEN];
-char filter_file[PATH_LEN];
 char local_input_history_file[PATH_LEN];
 char local_newsgroups_file[PATH_LEN];	/* local copy of NNTP newsgroups file */
 char local_newsrctable_file[PATH_LEN];
 char lock_file[PATH_LEN];		/* contains name of index lock file */
+char filter_file[PATH_LEN];
 char mail_active_file[PATH_LEN];
 char mail_news_user[LEN];		/* mail new news to this user address */
 char mailbox[PATH_LEN];			/* system mailbox for each user */
 char mailer[PATH_LEN];			/* mail program */
-#ifdef HAVE_MH_MAIL_HANDLING
-	char mailgroups_file[PATH_LEN];
-#endif /* HAVE_MH_MAIL_HANDLING */
-char **news_headers_to_display_array;	/* array of which headers to display */
-char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char newnewsrc[PATH_LEN];
 char news_active_file[PATH_LEN];
 char newsgroups_file[PATH_LEN];
 char newsrc[PATH_LEN];
-#ifndef NNTP_ONLY
-	char novrootdir[PATH_LEN];		/* root directory of nov index files */
-	char novfilename[PATH_LEN];		/* file name of a single nov index files */
-#endif /* !NNTP_ONLY */
 char page_header[LEN];			/* page header of pgm name and version */
 char posted_info_file[PATH_LEN];
-char posted_msgs_file[PATH_LEN];
 char postponed_articles_file[PATH_LEN];	/* ~/.tin/postponed.articles file */
-char tin_progname[PATH_LEN];		/* program name */
 char rcdir[PATH_LEN];
 char save_active_file[PATH_LEN];
 char spooldir[PATH_LEN];		/* directory where news is */
 char subscriptions_file[PATH_LEN];
+char tin_progname[PATH_LEN];		/* program name */
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
-
-char proc_ch_default;			/* set in change_config_file() */
+#ifdef HAVE_MH_MAIL_HANDLING
+	char mailgroups_file[PATH_LEN];
+#endif /* HAVE_MH_MAIL_HANDLING */
+#ifndef NNTP_ONLY
+	char novfilename[PATH_LEN];		/* file name of a single nov index file */
+	char novrootdir[PATH_LEN];		/* root directory of nov index files */
+#endif /* !NNTP_ONLY */
 
 #ifdef VMS
 	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
@@ -133,31 +129,30 @@ int i_key_search_last;			/* for repeated search */
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int num_headers_to_display;		/* num headers to display -- swp */
 int num_headers_to_not_display;		/* num headers to not display -- swp */
-int num_of_killed_arts;
-int num_of_selected_arts;		/* num articles marked 'hot' */
 int system_status;
 int xmouse, xrow, xcol;			/* xterm button pressing information */
 
-#ifdef HAVE_COLOR
-	t_bool use_color;		/* enables/disables ansi-color support under linux-console and color-xterm */
-#endif /* HAVE_COLOR */
-
-t_bool word_highlight;		/* word highlighting on/off */
-
-pid_t process_id;			/* Useful to have around for .suffixes */
 mode_t real_umask;
+pid_t process_id;			/* Useful to have around for .suffixes */
 
-t_bool no_write = FALSE;		/* do not write newsrc on quit (-X cmd-line flag) */
+t_bool (*wildcard_func) (const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
+t_bool batch_mode;			/* update index files only mode */
 t_bool check_for_new_newsgroups;	/* don't check for new newsgroups */
 t_bool cmd_line;			/* batch / interactive mode */
 t_bool created_rcdir;			/* checks if first time tin is started */
 t_bool dangerous_signal_exit;		/* no get_respcode() in nntp_command when dangerous signal exit */
 t_bool disable_gnksa_domain_check;	/* disable checking TLD in From: etc. */
 t_bool disable_sender;			/* disable generation of Sender: header */
-t_bool got_sig_pipe = FALSE;
 t_bool filtered_articles;		/* locally killed / auto-selected articles */
+#ifdef NO_POSTING
+	t_bool force_no_post = TRUE;		/* force no posting mode */
+#else
+	t_bool force_no_post = FALSE;	/* don't force no posting mode */
+#endif /* NO_POSTING */
+t_bool got_sig_pipe = FALSE;
 t_bool list_active;
 t_bool newsrc_active;
+t_bool no_write = FALSE;		/* do not write newsrc on quit (-X cmd-line flag) */
 t_bool post_article_and_exit;		/* quick post of an article then exit(elm like) */
 t_bool post_postponed_and_exit;		/* post postponed articles and exit */
 t_bool reread_active_for_posted_arts;
@@ -165,14 +160,12 @@ t_bool read_local_newsgroups_file;	/* read newsgroups file locally or via NNTP *
 t_bool read_news_via_nntp = FALSE;	/* read news locally or via NNTP */
 t_bool read_saved_news = FALSE;		/* tin -R read saved news from tin -S */
 t_bool show_description = TRUE;		/* current copy of tinrc flag */
-t_bool batch_mode;			/* update index files only mode */
 t_bool verbose = FALSE;			/* update index files only mode */
-t_bool (*wildcard_func) (const char *str, char *patt, t_bool icase);		/* Wildcard matching function */
-t_bool xover_supported = FALSE;
+t_bool word_highlight;		/* word highlighting on/off */
 t_bool xref_supported = TRUE;
-#ifdef LOCAL_CHARSET
-	t_bool use_local_charset = TRUE;
-#endif /* LOCAL_CHARSET */
+#ifdef HAVE_COLOR
+	t_bool use_color;		/* enables/disables ansi-color support under linux-console and color-xterm */
+#endif /* HAVE_COLOR */
 #ifdef NNTP_ABLE
 	t_bool force_auth_on_conn_open = FALSE;	/* authenticate on connection startup */
 	unsigned short nntp_tcp_port;
@@ -188,13 +181,6 @@ char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 	struct utsname system_info;
 #endif /* HAVE_SYS_UTSNAME_H */
 
-#if 0
-#ifndef M_AMIGA
-	static struct passwd *myentry;
-	static struct passwd pwdentry;
-#endif /* !M_AMIGA */
-#endif /* 0 */
-
 struct regex_cache
 		strip_re_regex, strip_was_regex,
 		uubegin_regex, uubody_regex,
@@ -252,6 +238,11 @@ struct t_config tinrc = {
 	0,			/* mailbox_format = MBOXO */
 #endif /* SCO_UNIX */
 	"",		/* mail_address */
+#ifdef HAVE_METAMAIL
+	METAMAIL_CMD,		/* metamail_prog */
+#else
+	INTERNAL_CMD,		/* metamail_prog */
+#endif /* HAVE_METAMAIL */
 #ifndef CHARSET_CONVERSION
 	"",		/* mm_charset, defaults to $MM_CHARSET */
 #else
@@ -289,10 +280,11 @@ struct t_config tinrc = {
 	0,		/* getart_limit */
 	2,		/* recent_time */
 	32,		/* groupname_max_length */
-	KILL_READ,		/* kill_level */
+	UUE_NO,	/* hide_uue */
+	KILL_UNREAD,		/* kill_level */
 	MIME_ENCODING_7BIT,		/* mail_mime_encoding */
 	MIME_ENCODING_7BIT,		/* post_mime_encoding */
-	POST_PROC_NONE,			/* post_process */
+	POST_PROC_NO,			/* post_process */
 	REREAD_ACTIVE_FILE_SECS,	/* reread_active_file_secs */
 	1,		/* scroll_lines */
 	SHOW_FROM_NAME,				/* show_author */
@@ -329,8 +321,12 @@ struct t_config tinrc = {
 	0,		/* col_subject (initialised later) */
 	0,		/* col_text (initialised later) */
 	0,		/* col_title (initialised later) */
-	2,		/* word_h_display_marks */
 #endif /* HAVE_COLOR */
+	2,		/* word_h_display_marks */
+	2,		/* mono_markdash */
+	6,		/* mono_markstar */
+	5,		/* mono_markslash */
+	3,		/* mono_markstroke */
 	TRUE,		/* word_highlight */
 	0,		/* wrap_column */
 #ifdef HAVE_COLOR
@@ -363,9 +359,9 @@ struct t_config tinrc = {
 	TRUE,		/* inverse_okay */
 #endif /* USE_INVERSE_HACK */
 	TRUE,		/* keep_dead_articles */
-	TRUE,		/* keep_posted_articles */
-	POSTED_FILE,	/* keep_posted_articles_file */
+	POSTED_FILE,	/* posted_articles_file */
 	FALSE,		/* mail_8bit_header */
+	FALSE,		/* mark_ignore_tags */
 	TRUE,		/* mark_saved_read */
 	TRUE,		/* pgdn_goto_next */
 	TRUE,		/* pos_first_unread */
@@ -378,12 +374,10 @@ struct t_config tinrc = {
 	FALSE,		/* prompt_followupto */
 	QUOTE_COMPRESS|QUOTE_EMPTY,	/* quote_style */
 	TRUE,		/* show_description */
-	TRUE,		/* show_lines */
+	1,		/* show_infos */
 	TRUE,		/* show_only_unread_arts */
 	FALSE,		/* show_only_unread_groups */
-	FALSE,		/* show_score */
 	TRUE,		/* show_signatures */
-	FALSE,		/* hide_uue */
 	TRUE,		/* sigdashes */
 	TRUE,		/* signature_repost */
 	FALSE,		/* space_goto_next_unread */
@@ -404,15 +398,16 @@ struct t_config tinrc = {
 #ifdef HAVE_KEYPAD
 	FALSE,		/* use_keypad */
 #endif /* HAVE_KEYPAD */
-#ifdef HAVE_METAMAIL
+	TRUE,		/* wrap_on_next_unread */
 	FALSE,		/* ask_for_metamail */
-	FALSE,		/* use_metamail */
-#endif /* HAVE_METAMAIL */
 	FALSE,		/* default_filter_kill_case */
 	FALSE,		/* default_filter_kill_expire */
 	TRUE,		/* default_filter_kill_global */
 	FALSE,		/* default_filter_select_case */
 	FALSE,		/* default_filter_select_expire */
+#ifdef XFACE_ABLE
+	FALSE,		/* use_slrnface */
+#endif /* XFACE_ABLE */
 	TRUE		/* default_filter_select_global */
 };
 
@@ -507,14 +502,15 @@ init_selfinfo(
 	host_name[0] = '\0';
 	domain_name[0] = '\0';
 
-#ifndef M_AMIGA
-#	ifdef HAVE_SYS_UTSNAME_H
+#ifndef M_AMIGA /* TODO: why do we exclude M_AMIGA here but not in main.c:read_cmd_line_options() */
+#	if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
 	if (uname(&system_info) < 0) {
 		strcpy(system_info.sysname, "unknown");
 		*system_info.machine = '\0';
 		*system_info.release = '\0';
+		*system_info.nodename = '\0';
 	}
-#	endif /* HAVE_SYS_UTSNAME_H */
+#	endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
 #endif /* !M_AMIGA */
 
 	if ((cptr = get_host_name()) != NULL)
@@ -538,17 +534,6 @@ init_selfinfo(
 	real_umask = umask(0);
 	(void) umask(real_umask);
 
-#if defined(HAVE_SETLOCALE) && defined(LC_ALL) && !defined(NO_LOCALE)
-	if (!setlocale(LC_ALL, "")) {
-		/*
-		 * TODO: issue a warning here like
-		 *       "Can't set the specified locale! Check $LANG, $LC_CTYPE, $LC_ALL"
-		 */
-		;
-	}
-#endif /* HAVE_SETLOCALE && LC_ALL && !NO_LOCALE */
-
-/* FIXME: move to get_user_name() [header.c] */
 #ifndef M_AMIGA
 #	ifndef VMS
 	if ((myentry = getpwuid(getuid())) == NULL) {
@@ -556,7 +541,6 @@ init_selfinfo(
 		giveup();
 	}
 #	else
-	/* TODO: get_user_name() entirely reles on $USER */
 	if (((ptr = getlogin()) != NULL) && strlen(ptr))
 		myentry = getpwnam(ptr);
 	else {
@@ -571,50 +555,48 @@ init_selfinfo(
 	lower(userid);
 #	endif /* VMS */
 
-	if ((ptr = getenv("TIN_HOMEDIR")) != NULL) {
-		my_strncpy(homedir, ptr, sizeof(homedir));
-	} else if ((ptr = getenv("HOME")) != NULL) {
-		my_strncpy(homedir, ptr, sizeof(homedir));
-	} else if (!myentry) {
-		strcpy(homedir, "/tmp");
-	} else
-		my_strncpy(homedir, myentry->pw_dir, sizeof(homedir));
-
 #else
-	/* TODO: get_user_name() uses $USER, not $USERNAME */
 	if ((ptr = getenv("USERNAME")) == NULL) {
 		error_message(_(txt_env_var_not_found), "USERNAME");
 		giveup();
 	}
-	my_strncpy(userid, ptr, sizeof(userid));
-
-	/* TODO: why not also check for $TIN_HOMEDIR? */
-	if ((ptr = getenv("HOME")) == NULL) {
-		error_message(_(txt_env_var_not_found), "HOME");
-		giveup();
-	}
-	my_strncpy(homedir, ptr, sizeof(homedir));
+	my_strncpy(userid, ptr, sizeof(userid) - 1);
 #endif /* !M_AMIGA */
 
+	if (((ptr = getenv("TIN_HOMEDIR")) != NULL) && strlen(ptr)) {
+		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
+	} else if (((ptr = getenv("HOME")) != NULL) && strlen(ptr)) {
+		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
+	}
+#ifndef M_AMIGA
+	else if (strlen(myentry->pw_dir)) {
+		strncpy(homedir, myentry->pw_dir, sizeof(homedir) - 1);
+	}
+#endif /* !M_AMIGA */
+	 else
+		strncpy(homedir, TMPDIR, sizeof(homedir) - 1);
+
 	cmdline_nntpserver[0] = '\0';
 	created_rcdir = FALSE;
 	dangerous_signal_exit = FALSE;
 	disable_gnksa_domain_check = FALSE;
 #ifdef MAC_OS_X	/* usualy they don't have a valid FQDN */
+	/*
+	 * TODO: check if gnksa_check_domain(get_fqdn()) returns ok
+	 *       and if it does, don't disable Sender
+	 */
 	disable_sender = TRUE;
 #else
 	disable_sender = FALSE;
 #endif /* MAC_OS_X */
 	filtered_articles = FALSE;
 	iso2asc_supported = atoi(get_val("ISO2ASC", DEFAULT_ISO2ASC));
-	if (iso2asc_supported > NUM_ISO_TABLES)
-		iso2asc_supported = 0;
+	if (iso2asc_supported > NUM_ISO_TABLES || iso2asc_supported < 0) /* TODO: issue a warning here? */
+		iso2asc_supported = -1;
 	list_active = FALSE;
 	newsrc_active = FALSE;
 	num_headers_to_display = 0;
 	num_headers_to_not_display = 0;
-	num_of_selected_arts = 0;
-	num_of_killed_arts = 0;
 	post_article_and_exit = FALSE;
 	post_postponed_and_exit = FALSE;
 	read_local_newsgroups_file = FALSE;
@@ -623,12 +605,6 @@ init_selfinfo(
 	batch_mode = FALSE;
 	check_for_new_newsgroups = TRUE;
 	wildcard_func = wildmat;
-#ifdef HAVE_METAMAIL
-#	ifdef M_AMIGA
-		/* for all those AmigaElm users ... ;-) */
-		tinrc.use_metamail = (getenv("NoMetaMail") != NULL) ? TRUE : FALSE;
-#	endif /* M_AMIGA */
-#endif /* HAVE_METAMAIL */
 
 #ifdef HAVE_COLOR
 	preinit_colors();
@@ -648,47 +624,36 @@ init_selfinfo(
 	sprintf(cvers, txt_copyright_notice, page_header);
 
 	default_organization[0] = '\0';
-	proc_ch_default = 'n';
 	news_headers_to_display_array = ulBuildArgv(tinrc.news_headers_to_display, &num_headers_to_display);
 	news_headers_to_not_display_array = NULL;
 
-	/* TODO: nuke $BUG_ADDRESS entirely? */
-	strcpy(bug_addr, get_val("BUG_ADDRESS", BUG_REPORT_ADDRESS));
+	strncpy(bug_addr, BUG_REPORT_ADDRESS, sizeof(bug_addr) - 1);
 
 	bug_nntpserver1[0] = '\0';
 	bug_nntpserver2[0] = '\0';
 
-	/*
-	 * Amiga uses assigns which end in a ':' and won't work with a '/'
-	 * tacked on after them: e.g. we want UULIB:active, and not
-	 * UULIB:/active. For this reason I have changed the sprintf calls
-	 * to joinpath. This is defined to sprintf(result,"%s/%s",dir,file)
-	 * on all UNIX systems.
-	 */
-
 #ifdef INEWSDIR
-	strcpy(inewsdir, INEWSDIR);
+	strncpy(inewsdir, INEWSDIR, sizeof(inewsdir) - 1);
 #else
 	inewsdir[0] = '\0';
 #endif /* INEWSDIR */
 
 #ifdef apollo
-	strcpy(default_organization, get_val("NEWSORG", ""));
+	my_strncpy(default_organization, get_val("NEWSORG", ""), sizeof(default_organization) - 1);
 #else
-	strcpy(default_organization, get_val("ORGANIZATION", ""));
+	my_strncpy(default_organization, get_val("ORGANIZATION", ""), sizeof(default_organization) - 1);
 #endif /* apollo */
 
 #ifdef USE_INN_NNTPLIB
-	ptr = GetConfigValue(_CONF_ORGANIZATION);
-	if (ptr != NULL)
-		my_strncpy(default_organization, ptr, sizeof(default_organization));
+	if ((ptr = GetConfigValue(_CONF_ORGANIZATION)) != NULL)
+		my_strncpy(default_organization, ptr, sizeof(default_organization) - 1);
 #endif /* USE_INN_NNTPLIB */
 
 #ifndef NNTP_ONLY
-	strcpy(libdir, get_val("TIN_LIBDIR", NEWSLIBDIR)); /* moved inside ifdef */
-	strcpy(novrootdir, get_val("TIN_NOVROOTDIR", NOVROOTDIR));
-	strcpy(novfilename, get_val("TIN_NOVFILENAME", OVERVIEW_FILE));
-	strcpy(spooldir, get_val("TIN_SPOOLDIR", SPOOLDIR));
+	my_strncpy(libdir, get_val("TIN_LIBDIR", NEWSLIBDIR), sizeof(libdir) - 1);
+	my_strncpy(novrootdir, get_val("TIN_NOVROOTDIR", NOVROOTDIR), sizeof(novrootdir) - 1);
+	my_strncpy(novfilename, get_val("TIN_NOVFILENAME", OVERVIEW_FILE), sizeof(novfilename) - 1);
+	my_strncpy(spooldir, get_val("TIN_SPOOLDIR", SPOOLDIR), sizeof(spooldir) - 1);
 #endif /* !NNTP_ONLY */
 	/* clear news_active_file, active_time_file, newsgroups_file */
 	news_active_file[0] = '\0';
@@ -712,14 +677,18 @@ init_selfinfo(
 	 * if it's still unset exit tin.
 	 */
 	if (domain_name[0] == '\0') {
-		error_message(txt_error_no_domain_name);
-		giveup();
-		/*
-		 * TODO: ask the user to fall into no posting mode (can_post=FALSE)
-		 *       instead of exeting.
-		 */
+		error_message(_(txt_error_no_domain_name));
+		force_no_post = TRUE;
 	}
 
+	/*
+	 * Amiga uses assigns which end in a ':' and won't work with a '/'
+	 * tacked on after them: e.g. we want UULIB:active, and not
+	 * UULIB:/active. For this reason I have changed the sprintf calls
+	 * to joinpath. This is defined to sprintf(result,"%s/%s",dir,file)
+	 * on all UNIX systems.
+	 */
+
 	/*
 	 * only set the following variables if they weren't set from within
 	 * read_site_config()
@@ -746,7 +715,7 @@ init_selfinfo(
 					*ptr = '\0';
 			}
 			fclose(fp);
-			my_strncpy(default_organization, buf, sizeof(default_organization));
+			my_strncpy(default_organization, buf, sizeof(default_organization) - 1);
 		}
 	}
 
@@ -794,11 +763,8 @@ init_selfinfo(
 		created_rcdir = TRUE;
 		my_mkdir(rcdir, (mode_t) (S_IRWXU));
 	}
-#if defined(M_UNIX) || defined(M_AMIGA) || defined(VMS)
 	strcpy(tinrc.mailer_format, MAILER_FORMAT);
-#else
-	strcpy(tinrc.mailer_format, mailer);
-#endif /* M_UNIX || M_AMIGA || VMS */
+	my_strncpy(mailer, get_val(ENV_VAR_MAILER, DEFAULT_MAILER), sizeof(mailer) - 1);
 #ifndef DISABLE_PRINTING
 	strcpy(tinrc.printer, DEFAULT_PRINTER);
 #	ifdef M_AMIGA
@@ -807,10 +773,9 @@ init_selfinfo(
 #	endif /* M_AMIGA */
 #endif /* !DISABLE_PRINTING */
 	strcpy(tinrc.inews_prog, PATH_INEWS);
-	strcpy(mailer, get_val(ENV_VAR_MAILER, DEFAULT_MAILER));
 	joinpath(article, homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
-	sprintf(article + strlen(article), ".%d", (int) process_id);
+	snprintf(article + strlen(article), sizeof(article) - strlen(article), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 	joinpath(dead_article, homedir, "dead.article");
 	joinpath(dead_articles, homedir, "dead.articles");
@@ -845,18 +810,13 @@ init_selfinfo(
 	joinpath(newsrc, homedir, NEWSRC_FILE);
 	joinpath(newnewsrc, homedir, NEWNEWSRC_FILE);
 #ifdef APPEND_PID
-	sprintf(newnewsrc + strlen(newnewsrc), "%d", (int) process_id);
+	snprintf(newnewsrc + strlen(newnewsrc), sizeof(newnewsrc) - strlen(newnewsrc), ".%d", (int) process_id);
 #endif /* APPEND_PID */
 	joinpath(posted_info_file, rcdir, POSTED_FILE);
-	joinpath(posted_msgs_file, tinrc.maildir, POSTED_FILE);
 	joinpath(postponed_articles_file, rcdir, POSTPONED_FILE);
 	joinpath(save_active_file, rcdir, ACTIVE_SAVE_FILE);
 
-#ifdef HAVE_LONG_FILE_NAMES
-	sprintf(lock_file, "%stin.%s.LCK", TMPDIR, userid);
-#else
-	sprintf(lock_file, "%s%s.LCK", TMPDIR, userid);
-#endif /* HAVE_LONG_FILE_NAMES */
+	snprintf(lock_file, sizeof(lock_file), INDEX_LOCK, TMPDIR, userid);
 
 #ifdef NNTP_ABLE
 	nntp_tcp_port = (unsigned short) atoi(get_val("NNTPPORT", NNTP_TCP_PORT));
@@ -864,7 +824,7 @@ init_selfinfo(
 
 	if (stat(posted_info_file, &sb) == -1) {
 		if ((fp = fopen(posted_info_file, "w")) != NULL) {
-			fprintf(fp, txt_posted_info_file);
+			fprintf(fp, _(txt_posted_info_file));
 			fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
 			fclose(fp);
 		}
@@ -882,74 +842,6 @@ init_selfinfo(
 }
 
 
-/*
- * If we're caching overview files and the user specified an NNTP server
- * with the '-g' option, make the directory name specific to the NNTP server
- * and make sure the directory exists.
- */
-void
-set_up_private_index_cache(
-	void)
-{
-	struct stat sb;
-
-	if (xover_supported && !tinrc.cache_overview_files)
-		return;
-
-	if (cmdline_nntpserver[0] != 0) {
-		char *from;
-		char *to;
-		int c;
-
-		to = index_newsdir + strlen(index_newsdir);
-		*(to++) = '-';
-		for (from = cmdline_nntpserver; (c = *from) != 0; ++from)
-			*(to++) = tolower(c);
-		*to = 0;
-	}
-
-	if (stat(index_newsdir, &sb) == -1)
-		my_mkdir(index_newsdir, (mode_t) S_IRWXU);
-
-#	ifdef DEBUG
-	debug_nntp("set_up_private_index_cache", index_newsdir);
-#	endif /* DEBUG */
-	joinpath(local_newsgroups_file, index_newsdir, NEWSGROUPS_FILE);
-	return;
-}
-
-
-/*
- * Create default mail & save directories if they do not exist
- */
-t_bool
-create_mail_save_dirs(
-	void)
-{
-	t_bool created = FALSE;
-	char path[PATH_LEN];
-	struct stat sb;
-
-	if (!strfpath(tinrc.maildir, path, sizeof(path), NULL))
-		joinpath(path, homedir, DEFAULT_MAILDIR);
-
-	if (stat(path, &sb) == -1) {
-		my_mkdir(path, (mode_t) (S_IRWXU));
-		created = TRUE;
-	}
-
-	if (!strfpath(tinrc.savedir, path, sizeof(path), NULL))
-		joinpath(path, homedir, DEFAULT_SAVEDIR);
-
-	if (stat(path, &sb) == -1) {
-		my_mkdir(path, (mode_t) (S_IRWXU));
-		created = TRUE;
-	}
-
-	return created;
-}
-
-
 /*
  * read_site_config()
  *
