@@ -3,7 +3,7 @@
  *  Module    : misc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-16
+ *  Updated   : 2003-06-17
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -71,20 +71,10 @@ static int gnksa_check_localpart(const char *localpart);
 static int gnksa_dequote_plainphrase(char *realname, char *decoded, int addrtype);
 static int strfeditor(char *editor, int linenum, const char *filename, char *s, size_t maxsize, char *format);
 static void write_input_history_file(void);
-#ifdef LOCAL_CHARSET
-	static int to_local(int c);
-	static int to_network(int c);
-#endif /* LOCAL_CHARSET */
 #ifdef CHARSET_CONVERSION
 	static char *utf8_valid(char *line);
-#endif /* CHARSET_CONVERSION */
-#ifdef LOCAL_CHARSET
-	static void buffer_to_local(char *line);
-#else
-#	ifdef CHARSET_CONVERSION
 	static t_bool buffer_to_local(char **line, int *max_line_len, const char *network_charset, const char *local_charset);
-#	endif /* CHARSET_CONVERSION */
-#endif /* LOCAL_CHARSET */
+#endif /* CHARSET_CONVERSION */
 #if (defined(MIME_STRICT_CHARSET) && !defined(NO_LOCALE)) || defined(CHARSET_CONVERSION)
 	static void buffer_to_ascii(char *c);
 #endif /* (MIME_STRICT_CHARSET && !NO_LOCALE) || CHARSET_CONVERSION */
@@ -400,15 +390,15 @@ invoke_editor(
 t_bool
 invoke_ispell(
 	const char *nam,
-	struct t_group *psGrp) /* return value is always ignored */
+	struct t_group *group) /* return value is always ignored */
 {
 	FILE *fp_all, *fp_body, *fp_head;
-	char buf[PATH_LEN], nam_body[100], nam_head[100];
+	char buf[PATH_LEN], nam_body[PATH_LEN], nam_head[PATH_LEN];
 	char ispell[PATH_LEN];
 	t_bool retcode;
 
-	if (psGrp && psGrp->attribute->ispell != NULL)
-		STRCPY(ispell, psGrp->attribute->ispell);
+	if (group && group->attribute->ispell != NULL)
+		STRCPY(ispell, group->attribute->ispell);
 	else
 		STRCPY(ispell, get_val("ISPELL", PATH_ISPELL));
 
@@ -416,11 +406,13 @@ invoke_ispell(
 	 * Now seperating the header and body in two different files so that
 	 * the header is not checked by ispell
 	 */
-	strncpy(nam_body, nam, sizeof(nam_body) - 6);
-	strcat(nam_body, ".body");
-
-	strncpy(nam_head, nam, sizeof(nam_head) - 6);
-	strcat(nam_head, ".head");
+#ifdef HAVE_LONG_FILE_NAMES
+	snprintf(nam_body, sizeof(nam_body), "%s%s", nam, ".body");
+	snprintf(nam_head, sizeof(nam_head), "%s%s", nam, ".head");
+#else
+	snprintf(nam_body, sizeof(nam_body), "%s%s", nam, ".b");
+	snprintf(nam_head, sizeof(nam_head), "%s%s", nam, ".h");
+#endif /* HAVE_LONG_FILE_NAMES */
 
 	if ((fp_all = fopen(nam, "r")) == NULL) {
 		perror_message(_(txt_cannot_open), nam);
@@ -482,14 +474,14 @@ shell_escape(
 		continue;
 
 	if (*p)
-		my_strncpy(tinrc.default_shell_command, p, sizeof(tinrc.default_shell_command));
+		my_strncpy(tinrc.default_shell_command, p, sizeof(tinrc.default_shell_command) - 1);
 	else {
-		my_strncpy(shell, (*tinrc.default_shell_command ? tinrc.default_shell_command : (get_val(ENV_VAR_SHELL, DEFAULT_SHELL))), sizeof(shell));
+		my_strncpy(shell, (*tinrc.default_shell_command ? tinrc.default_shell_command : (get_val(ENV_VAR_SHELL, DEFAULT_SHELL))), sizeof(shell) - 1);
 		p = shell;
 	}
 
 	ClearScreen();
-	sprintf(mesg, "Shell Command (%s)", p); /* FIXME: -> lang.c */
+	sprintf(mesg, _(txt_shell_command), p);
 	center_line(0, TRUE, mesg);
 	MoveCursor(INDEX_TOP, 0);
 
@@ -553,7 +545,7 @@ tin_done(
 					} else
 						break;
 				}
-				wait_message(0, _("Catchup %s..."), group->name);	/* FIXME: -> lang.c */
+				wait_message(0, _(txt_catchup_group), group->name);
 				grp_mark_read(group, NULL);
 			}
 		}
@@ -566,7 +558,7 @@ tin_done(
 	if (!no_write) {
 		forever {
 			if (((wrote_newsrc_lines = write_newsrc()) >= 0L) && (wrote_newsrc_lines >= read_newsrc_lines)) {
-				if (!batch_mode || (batch_mode && verbose))
+				if (!batch_mode || verbose)
 					my_fputs(_(txt_newsrc_saved), stdout);
 				break;
 			}
@@ -589,7 +581,7 @@ tin_done(
 		}
 
 		write_input_history_file();
-#if 0 /* FIXME */
+#if 0
 		write_attributes_file(local_attributes_file);
 #endif /* 0 */
 
@@ -598,6 +590,10 @@ tin_done(
 #endif /* HAVE_MH_MAIL_HANDLING */
 	}
 
+#ifdef XFACE_ABLE
+	slrnface_stop();
+#endif /* XFACE_ABLE */
+
 	/* Do this sometime after we save the newsrc in case this hangs up for any reason */
 	if (ret != NNTP_ERROR_EXIT)
 		nntp_close();			/* disconnect from NNTP server */
@@ -649,81 +645,6 @@ tin_done(
 }
 
 
-/*
- * strip_double_ngs()
- * Strip duplicate newsgroups from within a given list of comma
- * separated groups
- *
- * 14-Jun-'96 Sven Paulus <sven@oops.sub.de>
- *
- */
-void
-strip_double_ngs(
-	char *ngs_list)
-{
-	char *ptr;			/* start of next (outer) newsgroup */
-	char *ptr2;			/* temporary pointer */
-	char ngroup1[HEADER_LEN];	/* outer newsgroup to compare */
-	char ngroup2[HEADER_LEN];	/* inner newsgroup to compare */
-	char cmplist[HEADER_LEN];	/* last loops output */
-	char newlist[HEADER_LEN];	/* the newly generated list without */
-										/* any duplicates of the first nwsg */
-	int ncnt1;			/* counter for the first newsgroup */
-	int ncnt2;			/* counter for the second newsgroup */
-	t_bool over1;		/* TRUE when the outer loop is over */
-	t_bool over2;		/* TRUE when the inner loop is over */
-
-	/* shortcut, check if there is only 1 group */
-	if (strchr(ngs_list, ',') != NULL) {
-		over1 = FALSE;
-		ncnt1 = 0;
-		strcpy(newlist, ngs_list);		/* make a "working copy" */
-		ptr = newlist;						/* the next outer newsg. is the 1st */
-
-		while (!over1) {
-			ncnt1++;							/* inc. outer counter */
-			strcpy(cmplist, newlist);	/* duplicate groups for inner loop */
-			ptr2 = strchr(ptr, ',');	/* search "," ... */
-			if (ptr2 != NULL) {	/* if found ... */
-				*ptr2 = '\0';
-				strcpy(ngroup1, ptr);	/* chop off first outer newsgroup */
-				ptr = ptr2 + 1;			/* pointer points to next newsgr. */
-			} else {							/* ... if not: last group */
-				over1 = TRUE;				/* wow, everything is done after . */
-				strcpy(ngroup1, ptr);	/* ... this last outer newsgroup */
-			}
-
-			over2 = FALSE;
-			ncnt2 = 0;
-
-			/*
-			 * now compare with each inner newsgroup on the list,
-			 * which is behind the momentary outer newsgroup
-			 * if it is different from the outer newsgroup, append
-			 * to list, strip double-commas
-			 */
-
-			while (!over2) {
-				ncnt2++;
-				strcpy(ngroup2, cmplist);
-				ptr2 = strchr(ngroup2, ',');
-				if (ptr2 != NULL) {
-					strcpy(cmplist, ptr2 + 1);
-					*ptr2 = '\0';
-				} else
-					over2 = TRUE;
-
-				if ((ncnt2 > ncnt1) && (strcasecmp(ngroup1, ngroup2)) && (strlen(ngroup2) != 0)) {
-					strcat(newlist, ",");
-					strcat(newlist, ngroup2);
-				}
-			}
-		}
-		strcpy(ngs_list, newlist);	/* move string to its real location */
-	}
-}
-
-
 int
 my_mkdir(
 	char *path,
@@ -733,7 +654,7 @@ my_mkdir(
 	char buf[LEN];
 	struct stat sb;
 
-	sprintf(buf, "mkdir %s", path); /* redirect stderr to /dev/null? */
+	sprintf(buf, "mkdir %s", path); /* redirect stderr to /dev/null? use invoke_cmd()? */
 	if (stat(path, &sb) == -1) {
 		system(buf);
 		return chmod(path, mode);
@@ -856,12 +777,17 @@ rename_file(
 #endif /* M_AMIGA */
 
 
+/*
+ * Note that we exit screen/curses mode when invoking
+ * external commands
+ */
 t_bool
 invoke_cmd(
 	const char *nam)
 {
 	int ret;
 	t_bool save_cmd_line = cmd_line;
+	t_bool success;
 
 	if (!save_cmd_line) {
 		EndWin();
@@ -870,13 +796,15 @@ invoke_cmd(
 	set_signal_catcher(FALSE);
 
 	TRACE(("called system(%s)", _nc_visbuf(nam)));
-#ifdef USE_SYSTEM_STATUS
-	system(nam);
-	ret = system_status;
-#else
 	ret = system(nam);
-#endif /* USE_SYSTEM_STATUS */
-	TRACE(("return %d", ret));
+#ifdef VMS
+	system_status = 0;
+#else
+#	ifndef USE_SYSTEM_STATUS
+	system_status = (ret >= 0 && WIFEXITED(ret)) ? WEXITSTATUS(ret) : 0;
+#	endif /* !USE_SYSTEM_STATUS */
+#endif /* VMS */
+	TRACE(("return %d (%d)", ret, system_status));
 
 	set_signal_catcher(TRUE);
 	if (!save_cmd_line) {
@@ -885,14 +813,16 @@ invoke_cmd(
 		need_resize = cYes;		/* Flag a redraw */
 	}
 
-	if (ret != 0)
+#ifdef VMS
+	success = (ret != 0);
+#else
+	success = (ret == 0);
+#endif /* VMS */
+
+	if (!success || system_status != 0)
 		error_message(_(txt_command_failed), nam);
 
-#ifdef VMS
-	return ret != 0;
-#else
-	return ret == 0;
-#endif /* VMS */
+	return success;
 }
 
 
@@ -910,7 +840,7 @@ draw_percent_mark(
 		return;
 
 	clear_message();
-	snprintf(buf, sizeof(buf) - 1, "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num);
+	snprintf(buf, sizeof(buf), "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num);
 	MoveCursor(cLINES, (cCOLS - (int) strlen(buf)) - (1 + BLANK_PAGE_COLS));
 	StartInverse();
 	my_fputs(buf, stdout);
@@ -919,31 +849,55 @@ draw_percent_mark(
 }
 
 
+/*
+ * grab file portion of fullpath
+ */
 void
 base_name(
-	char *fullpath,		/* argv[0] */
-	char *program)		/* tin_progname is returned */
+	const char *fullpath,		/* /foo/bar/baz */
+	char *file)				/* baz */
 {
 	size_t i;
 #ifdef VMS
 	char *cp;
 #endif /* VMS */
 
-	strcpy(program, fullpath);
+	strcpy(file, fullpath);
 
 	for (i = strlen(fullpath) - 1; i; i--) {
-		if (fullpath[i] == SEPDIR) {
-			strcpy(program, fullpath + i + 1);
+		if (fullpath[i] == DIRSEP) {
+			strcpy(file, fullpath + i + 1);
 			break;
 		}
 	}
 #ifdef VMS
-	if ((cp = strrchr(program, '.')) != 0)
+	if ((cp = strrchr(file, '.')) != 0)
 		*cp = '\0';
 #endif /* VMS */
 }
 
 
+/*
+ * grab dir portion of fullpath
+ */
+void
+dir_name(
+	const char *fullpath,	/* /foo/bar/baz */
+	char *dir)		/* /foo/bar/ */
+{
+	char *d, *f, *p;
+
+	d = my_strdup(fullpath);
+	f = my_strdup(fullpath);
+	base_name(d, f);
+	if ((p = strrstr(d, f)) != NULL)
+		*p = '\0';
+	strcpy(dir, d);
+	free(f);
+	free(d);
+}
+
+
 /*
  * Return TRUE if new mail has arrived
  */
@@ -1007,7 +961,7 @@ mail_check(
 /*
  * Return a pointer into s eliminating any leading Re:'s. Example:
  *
- *		Re: Reorganization of misc.jobs
+ * 	Re: Reorganization of misc.jobs
  *		^   ^
  * 	Re^2: Reorganization of misc.jobs
  *
@@ -1051,10 +1005,6 @@ my_isprint(
 	/* use locale */
 	return isprint(c);
 #else
-#	ifdef LOCAL_CHARSET
-		/* use some conversation table */
-		return (isprint(c) || (c >= 0x80 && c <= 0xff));
-#	else
 	if (IS_LOCAL_CHARSET("ISO-8859"))
 		return (isprint(c) || (c >= 0xa0 && c <= 0xff));
 	else if (IS_LOCAL_CHARSET("ISO-2022"))
@@ -1065,7 +1015,6 @@ my_isprint(
 		return 1;
 	else /* KOI8-* and UTF-8 */
 		return (isprint(c) || (c >= 0x80 && c <= 0xff));
-#	endif /* LOCAL_CHARSET */
 #endif /* !NO_LOCALE */
 }
 
@@ -1124,8 +1073,10 @@ toggle_inverse_video(
 	if (!(tinrc.inverse_okay = bool_not(tinrc.inverse_okay)))
 		tinrc.draw_arrow = TRUE;
 #ifndef USE_INVERSE_HACK
+#	if 0
 	else
 		tinrc.draw_arrow = FALSE;
+#	endif /* 0 */
 #endif /* !USE_INVERSE_HACK */
 }
 
@@ -1166,9 +1117,8 @@ show_color_status(
 
 
 /*
- * Check for lock file to stop multiple copies of tind (obsolete) or
- * tin -U running and if it does not exist create it so this is the
- * only copy running
+ * Check for lock file to stop multiple copies of tin -u running and if it
+ * does not exist create it so this is the only copy running
  */
 void
 create_index_lock_file(
@@ -1302,7 +1252,7 @@ strfquote(
 
 				case 'F':	/* Articles Address+Name */
 					if (arts[respnum].name)
-						snprintf(tbuf, sizeof(tbuf) - 1 , "%s <%s>", arts[respnum].name, arts[respnum].from);
+						snprintf(tbuf, sizeof(tbuf), "%s <%s>", arts[respnum].name, arts[respnum].from);
 					else {
 						STRCPY(tbuf, arts[respnum].from);
 					}
@@ -1671,7 +1621,7 @@ _strfpath(
 					if (strfpath(group->attribute->savedir, buf, sizeof(buf), group)) {
 						char tmp[PATH_LEN];
 #ifdef HAVE_LONG_FILE_NAMES
-						my_strncpy(tmp, group->name, sizeof(tmp));
+						my_strncpy(tmp, group->name, sizeof(tmp) - 1);
 #else
 						my_strncpy(tmp, group->name, 14);
 #endif /* HAVE_LONG_FILE_NAMES */
@@ -1818,13 +1768,13 @@ escape_shell_meta(
  */
 int
 strfmailer(
-	char *the_mailer,
-	char *subject,
-	char *to,
+	const char *mail_prog,
+	char *subject,	/* FIXME: should be const char */
+	char *to, /* FIXME: should be const char */
 	const char *filename,
 	char *dest,
 	size_t maxsize,
-	char *format)
+	const char *format)
 {
 	char *endp = dest + maxsize;
 	char *start = dest;
@@ -1900,13 +1850,14 @@ strfmailer(
 		 * - '%' followed by NULL gets '%' and leaves loop
 		 * - '%%' gets '%'
 		 * - '%F' expands to filename
-		 * - '%M' expands to mailer
+		 * - '%M' expands to mailer program
 		 * - '%S' expands to subject of message
 		 * - '%T' expands to recipient(s) of message
 		 * - '%U' expands to userid
 		 * - '%' followed by any other character is copied literally
 		 */
 		if (*format == '%') {
+			char *p;
 			t_bool ismail = TRUE;
 			t_bool escaped = FALSE;
 			switch (*++format) {
@@ -1923,22 +1874,20 @@ strfmailer(
 					break;
 
 				case 'M':	/* Mailer */
-					STRCPY(tbuf, the_mailer);
+					STRCPY(tbuf, mail_prog);
 					break;
 
 				case 'S':	/* Subject */
 					/* don't MIME encode Subject if using external mail client */
 					if (tinrc.use_mailreader_i)
-						strncpy(tbuf, escape_shell_meta(subject, quote_area), sizeof(tbuf));
+						strncpy(tbuf, escape_shell_meta(subject, quote_area), sizeof(tbuf) - 1);
 					else {
-						char *p;
-
 #ifdef CHARSET_CONVERSION
 						p = rfc1522_encode(subject, txt_mime_charsets[tinrc.mm_network_charset], ismail);
 #else
 						p = rfc1522_encode(subject, tinrc.mm_charset, ismail);
 #endif /* CHARSET_CONVERSION */
-						strncpy(tbuf, escape_shell_meta(p, quote_area), sizeof(tbuf));
+						strncpy(tbuf, escape_shell_meta(p, quote_area), sizeof(tbuf) - 1);
 						free(p);
 					}
 					tbuf[sizeof(tbuf) - 1] = '\0';	/* just in case */
@@ -1948,16 +1897,14 @@ strfmailer(
 				case 'T':	/* To */
 					/* don't MIME encode To if using external mail client */
 					if (tinrc.use_mailreader_i)
-						strncpy(tbuf, escape_shell_meta(to, quote_area), sizeof(tbuf));
+						strncpy(tbuf, escape_shell_meta(to, quote_area), sizeof(tbuf) - 1);
 					else {
-						char *p;
-
 #ifdef CHARSET_CONVERSION
 						p = rfc1522_encode(to, txt_mime_charsets[tinrc.mm_network_charset], ismail);
 #else
 						p = rfc1522_encode(to, tinrc.mm_charset, ismail);
 #endif /* CHARSET_CONVERSION */
-						strncpy(tbuf, escape_shell_meta(p, quote_area), sizeof(tbuf));
+						strncpy(tbuf, escape_shell_meta(p, quote_area), sizeof(tbuf) - 1);
 						free(p);
 					}
 					tbuf[sizeof(tbuf) - 1] = '\0';	/* just in case */
@@ -1967,16 +1914,14 @@ strfmailer(
 				case 'U':	/* User */
 					/* don't MIME encode User if using external mail client */
 					if (tinrc.use_mailreader_i)
-						strncpy(tbuf, userid, sizeof(tbuf));
+						strncpy(tbuf, userid, sizeof(tbuf) - 1);
 					else {
-						char *p;
-
 #ifdef CHARSET_CONVERSION
 						p = rfc1522_encode(userid, txt_mime_charsets[tinrc.mm_network_charset], ismail);
 #else
 						p = rfc1522_encode(userid, tinrc.mm_charset, ismail);
 #endif /* CHARSET_CONVERSION */
-						strncpy(tbuf, p, sizeof(tbuf));
+						strncpy(tbuf, p, sizeof(tbuf) - 1);
 						free(p);
 					}
 					tbuf[sizeof(tbuf) - 1] = '\0';	/* just in case */
@@ -2052,7 +1997,7 @@ get_cwd(
 #	ifdef HAVE_GETWD
 	getwd(buf);
 #	else
-#		error "No getcwd() or getwd() function found"
+	*buf = '\0';
 #	endif /* HAVE_GETWD */
 #endif /* HAVE_GETCWD */
 }
@@ -2064,7 +2009,7 @@ get_cwd(
  */
 void
 make_group_path(
-	char *name,
+	const char *name,
 	char *path)
 {
 #ifdef VMS
@@ -2075,11 +2020,31 @@ make_group_path(
 		name++;
 		path++;
 	}
+	*path++ = '/';
 	*path = '\0';
 #endif /* VMS */
 }
 
 
+/*
+ * Given a base pathname & a newsgroup name build an absolute pathname.
+ * base_dir = /usr/spool/news
+ * group_name = alt.sources
+ * group_path = /usr/spool/news/alt/sources
+ */
+void
+make_base_group_path(
+	const char *base_dir,
+	const char *group_name,
+	char *group_path)
+{
+	char buf[LEN];
+
+	make_group_path(group_name, buf);
+	joinpath(group_path, base_dir, buf);
+}
+
+
 /*
  * Delete tmp index & local newsgroups file
  */
@@ -2087,19 +2052,18 @@ void
 cleanup_tmp_files(
 	void)
 {
+#if 0
 	char acNovFile[PATH_LEN];
 
-	if (read_news_via_nntp && xover_supported && !tinrc.cache_overview_files) {
+	if (xover_cmd && !tinrc.cache_overview_files) {
 		sprintf(acNovFile, "%s%d.idx", TMPDIR, (int) process_id);
 		unlink(acNovFile);
 	}
+#endif /* 0 */
 
 	if (!tinrc.cache_overview_files)
 		unlink(local_newsgroups_file);
 
-	/*
-	 * Even though batch_mode is turned off with -U, the child still has it set
-	 */
 	if (batch_mode)
 		unlink(lock_file);
 }
@@ -2155,6 +2119,7 @@ file_mtime(
 /*
  * TODO: this seems to be unix specific, avoid !M_UNIX calls or
  *       add code for other OS
+ *       this feature also isn't documented anywhere
  */
 char *
 random_organization(
@@ -2315,12 +2280,13 @@ quote_wild(
 	for (target = buff; *str != '\0'; str++) {
 		if (tinrc.wildcard) { /* regex */
 			/*
-			 * quote meta characters ()[]{}\^$*+?.
+			 * quote meta characters ()[]{}\^$*+?.#
 			 * replace whitespace with '\s' (pcre)
 			 */
 			if (*str == '(' || *str == ')' || *str == '[' || *str == ']' || *str == '{' || *str == '}'
 			    || *str == '\\' || *str == '^' || *str == '$'
 			    || *str == '*' || *str == '+' || *str == '?' || *str == '.'
+			    || *str == '#'
 			    || *str == ' ' || *str == '\t') {
 				*target++ = '\\';
 				*target++ = ((*str == ' ' || *str == '\t')? 's' : *str);
@@ -2365,142 +2331,20 @@ quote_wild_whitespace(
 }
 
 
-#if 0 /* unused */
-/*
- * strip_address() removes the address part from a given e-mail address
- */
-void
-strip_address(
-	char *the_address,
-	char *stripped_address)
-{
-	char *end_pos;
-	char *start_pos;
-
-	if (strchr(the_address, '@') != NULL) {
-		if ((end_pos = strchr(the_address, '<')) == NULL) {
-			if ((start_pos = strchr(the_address, ' ')) == NULL)
-				strcpy(stripped_address, the_address);
-			else {
-				strcpy(stripped_address, start_pos + 2);
-				if (stripped_address[strlen(stripped_address) - 1] == ')')
-					stripped_address[strlen(stripped_address) - 1] = '\0';
-			}
-		} else
-			if (end_pos > the_address)
-				strncpy(stripped_address, the_address, end_pos - the_address - 1);
-			else
-				strcpy(stripped_address, the_address);
-	} else {
-		if (the_address[0] == '(')
-			strcpy(stripped_address, the_address + 1);
-		else
-			strcpy(stripped_address, the_address);
-		if (stripped_address[strlen(stripped_address) - 1] == ')')
-			stripped_address[strlen(stripped_address) - 1] = '\0';
-	}
-}
-#endif /* 0 */
-
-
 /*
  * strip_name() removes the realname part from a given e-mail address
  */
 void
 strip_name(
-	char *the_address,
-	char *stripped_address)
+	const char *from,
+	char *address)
 {
-	char *end_pos;
-	char *start_pos;
+	char name[HEADER_LEN];
 
-	/* skip realname in address */
-	if ((start_pos = strchr(the_address, '<')) == NULL) {
-		/* address in user@domain(realname) syntax or realname is missing */
-		strcpy(stripped_address, the_address);
-		start_pos = stripped_address;
-		if ((end_pos = strchr(start_pos, ' ')) == NULL)
-			end_pos = start_pos + strlen(start_pos);
-	} else {
-		start_pos++; /* skip '<' */
-		strcpy(stripped_address, start_pos);
-		start_pos = stripped_address;
-		if ((end_pos = strchr(start_pos, '>')) == NULL)
-			end_pos = start_pos + strlen(start_pos); /* skip '>' */
-	}
-	*(end_pos) = '\0';
+	gnksa_do_check_from(from, address, name);
 }
 
 
-#ifdef LOCAL_CHARSET
-/*
- * convert between local and network charset (e.g. NeXT and latin1)
- */
-#	define CHARNUM 256
-#	define BAD (-1)
-/* use the appropriate conversion tables */
-#	if LOCAL_CHARSET == 437
-#		include "l1_ibm437.tab"
-#		include "ibm437_l1.tab"
-#	else
-#		if LOCAL_CHARSET == 850
-#			include "l1_ibm850.tab"
-#			include "ibm850_l1.tab"
-#		else
-#			include "l1_next.tab"
-#			include "next_l1.tab"
-#		endif /* 850 */
-#	endif /* 437 */
-static int
-to_local(
-	int c)
-{
-	if (use_local_charset) {
-		c = c_network_local[(unsigned char)c];
-		if (c == BAD)
-			return '?';
-		else
-			return c;
-	} else
-		return c;
-}
-
-
-static void
-buffer_to_local(
-	char *b)
-{
-	for (; *b; b++)
-		*b = to_local(*b);
-}
-
-
-static int
-to_network(
-	int c)
-{
-	if (use_local_charset) {
-		c = c_local_network[(unsigned char) c];
-		if (c == BAD)
-			return '?';
-		else
-			return c;
-	} else
-		return c;
-}
-
-
-void
-buffer_to_network(
-	char *b,
-	int mmnwcharset)
-{
-	for (; *b; b++)
-		*b = to_network(*b);
-}
-#endif /* LOCAL_CHARSET */
-
-
 #ifdef CHARSET_CONVERSION
 static t_bool
 buffer_to_local(
@@ -2543,10 +2387,13 @@ buffer_to_local(
 				char unknown_buf[4];
 				char *obuf, *outbuf;
 				char *tmpbuf, *tbuf;
+				ICONV_CONST char *cur_inbuf;
+				int used;
 				size_t inbytesleft;
 				size_t unknown_bytesleft;
 				size_t tmpbytesleft, tsize;
 				size_t outbytesleft, osize;
+				size_t cur_obl, cur_ibl;
 				size_t result;
 
 				inbytesleft = 1;
@@ -2598,6 +2445,15 @@ buffer_to_local(
 				outbuf = (char *) obuf;
 
 				do {
+					/*
+					 * save the parameters we need to redo the call of iconv
+					 * if we get into the E2BIG case
+					 */
+					cur_inbuf = inbuf;
+					cur_ibl = inbytesleft;
+					used = outbuf - obuf;
+					cur_obl = outbytesleft;
+
 					errno = 0;
 					result = iconv(cd2, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
 					if (result == (size_t) (-1)) {
@@ -2611,10 +2467,20 @@ buffer_to_local(
 								break;
 
 							case E2BIG:
-								obuf = my_realloc(obuf, osize * 2);
-								outbuf = (char *) (obuf + osize - outbytesleft);
-								outbytesleft += osize;
+								/*
+								 * outbuf was too small
+								 * As some input could be converted successfully
+								 * and we don`t know where the last complete char
+								 * ends, redo the last conversation completely.
+								 */
+								/* resize the output buffer */
+								obuf = my_realloc(obuf, osize * 2 + 1);
+								outbuf = obuf + used;
+								outbytesleft = cur_obl + osize;
 								osize <<= 1; /* double size */
+								/* reset the other params */
+								inbuf = cur_inbuf;
+								inbytesleft = cur_ibl;
 								break;
 
 							default:
@@ -2648,7 +2514,7 @@ buffer_to_local(
 
 
 /* convert from local_charset to txt_mime_charsets[mmnwcharset] */
-void
+t_bool
 buffer_to_network(
 	char *line,
 	int mmnwcharset)
@@ -2659,6 +2525,7 @@ buffer_to_network(
 	iconv_t cd;
 	size_t result, osize;
 	size_t inbytesleft, outbytesleft;
+	t_bool conv_success = TRUE;
 
 	if ((cd = iconv_open(txt_mime_charsets[mmnwcharset], tinrc.mm_local_charset)) != (iconv_t) (-1)) {
 		inbytesleft = strlen(line);
@@ -2674,10 +2541,12 @@ buffer_to_network(
 			if (result == (size_t) (-1)) {
 				switch (errno) {
 					case EILSEQ:
+						/* TODO: only one '?' for each multibyte sequence ? */
 						**&outbuf = '?';
 						outbuf++;
 						inbuf++;
 						inbytesleft--;
+						conv_success = FALSE;
 						break;
 
 					case E2BIG:
@@ -2689,6 +2558,7 @@ buffer_to_network(
 
 					default:	/* EINVAL */
 						inbytesleft = 0;
+						conv_success = FALSE;
 				}
 			}
 		} while (inbytesleft > 0);
@@ -2698,6 +2568,7 @@ buffer_to_network(
 		free(obuf);
 		iconv_close(cd);
 	}
+	return conv_success;
 }
 #endif /* CHARSET_CONVERSION */
 
@@ -2732,16 +2603,17 @@ process_charsets(
 	const char *local_charset,
 	t_bool conv_tex2iso)
 {
+	char *p;
+
 #ifdef CHARSET_CONVERSION
 	if (strcasecmp(network_charset, "US-ASCII")) {	/* network_charset is NOT US-ASCII */
-		char *clocal_charset;
 		if (iso2asc_supported >= 0)
-			clocal_charset = my_strdup("ISO-8859-1");
+			p = my_strdup("ISO-8859-1");
 		else
-			clocal_charset = my_strdup(local_charset);
-		if (!buffer_to_local(line, max_line_len, network_charset, clocal_charset))
+			p = my_strdup(local_charset);
+		if (!buffer_to_local(line, max_line_len, network_charset, p))
 			buffer_to_ascii(*line);
-		free(clocal_charset);
+		free(p);
 	} else /* set non-ASCII characters to '?' */
 		buffer_to_ascii(*line);
 #else
@@ -2751,9 +2623,6 @@ process_charsets(
 		buffer_to_ascii(*line);
 #	endif /* MIME_STRICT_CHARSET && !NO_LOCALE */
 	/* charset conversion (codepage version) */
-#	ifdef LOCAL_CHARSET
-	buffer_to_local(*line);
-#	endif /* LOCAL_CHARSET */
 #endif /* CHARSET_CONVERSION */
 
 	/*
@@ -2761,10 +2630,9 @@ process_charsets(
 	 * to allow TEX2ISO && ISO2ASC, i.e. "a -> auml -> ae
 	 */
 	if (conv_tex2iso) {
-		char *texbuf;
-		texbuf = my_strdup(*line);
-		convert_tex2iso(texbuf, *line);
-		free(texbuf);
+		p = my_strdup(*line);
+		convert_tex2iso(p, *line);
+		free(p);
 	}
 
 	/* iso2asc support */
@@ -2774,11 +2642,9 @@ process_charsets(
 	if (iso2asc_supported >= 0 && !strcasecmp(network_charset, "ISO-8859-1"))
 #endif /* CHARSET_CONVERSION */
 	{
-		char *isobuf;
-
-		isobuf = my_strdup(*line);
-		convert_iso2asc(isobuf, line, max_line_len, iso2asc_supported);
-		free(isobuf);
+		p = my_strdup(*line);
+		convert_iso2asc(p, line, max_line_len, iso2asc_supported);
+		free(p);
 	}
 }
 
@@ -2879,10 +2745,12 @@ static char gnksa_legal_localpart_chars[256] = {
 
 /*
  * legal realname characters according to son of RFC 1036
+ *
+ * we also allow CR & LF for folding
  */
 static char gnksa_legal_realname_chars[256] = {
 /*         0 1 2 3  4 5 6 7  8 9 a b  c d e f */
-/* 0x00 */ 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
+/* 0x00 */ 0,0,0,0, 0,0,0,0, 0,0,1,0, 0,1,0,0,
 /* 0x10 */ 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
 /* 0x20 */ 1,1,1,1, 1,1,1,1, 1,1,1,1, 1,1,1,1,
 /* 0x30 */ 1,1,1,1, 1,1,1,1, 1,1,1,1, 1,1,1,1,
@@ -3622,7 +3490,7 @@ gnksa_do_check_from(
 			code = result;
 
 		/* restore separator character */
-		*--addr_begin= '@';
+		*--addr_begin = '@';
 	}
 
 #ifdef DEBUG
@@ -3769,7 +3637,7 @@ utf8_valid(
 				if (d < 0xf0 || d > 0xf7 || (d == 0xf0 && e < 0x90))
 					illegal = TRUE;
 				/* largest current used sequence */
-				if (d == 0xf4 && e > 0x8f)
+				if ((d == 0xf4 && e > 0x8f) || d > 0xf4)
 					illegal = TRUE;
 				/* Unicode 3.1 noncharacters */
 				/* U+1FFFE, U+1FFFF, U+2FFFE, U+2FFFF, U+3FFFE, U+3FFFF; (Unicode 3.1) */
