@@ -3,7 +3,7 @@
  *  Module    : open.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2002-07-10
+ *  Updated   : 2003-07-22
  *  Notes     : Routines to make reading news locally (ie. /var/spool/news)
  *              or via NNTP transparent
  *
@@ -52,7 +52,8 @@
  */
 static int base_comp(t_comptype p1, t_comptype p2);
 #if 0 /* currently unused */
-	static FILE * open_xhdr_fp(char *header, long min, long max);
+	static FILE *open_xhdr_fp(char *header, long min, long max);
+	static t_bool stat_article(long art, const char *group_path);
 #endif /* 0 */
 
 
@@ -64,13 +65,12 @@ long head_next;
 	t_bool can_post = TRUE;
 #endif /* NO_POSTING */
 
-char *nntp_server = (char *) 0;
+char *nntp_server = NULL;
+char *xover_cmd = NULL;
 #ifdef NNTP_ABLE
-	static char txt_xover_string[] = "XOVER";
-	static char *txt_xover = txt_xover_string;
+	static char txt_xover_buff[] = "XOVER";
 #endif /* NNTP_ABLE */
 
-
 /*
  * Open a connection to the NNTP server. Authenticate if necessary or
  * desired, and test if the server supports XOVER.
@@ -85,8 +85,9 @@ nntp_open(
 #ifdef NNTP_ABLE
 	char *linep;
 	char line[NNTP_STRLEN];
-	int ret;
+	int i, ret;
 	t_bool sec = FALSE;
+	/* It appears that is_reconnect guards code that should be run only once */
 	static t_bool is_reconnect = FALSE;
 
 	if (!read_news_via_nntp)
@@ -96,10 +97,6 @@ nntp_open(
 	debug_nntp("nntp_open", "BEGIN");
 #	endif /* DEBUG */
 
-	/* do this only once at start-up */
-	if (!is_reconnect)
-		nntp_server = getserverbyfile(NNTP_SERVER_FILE);
-
 	if (nntp_server == NULL) {
 		error_message(_(txt_cannot_get_nntp_server_name));
 		error_message(_(txt_server_name_in_file_env_var), NNTP_SERVER_FILE);
@@ -145,9 +142,7 @@ nntp_open(
 
 		case OK_CANPOST:
 /*		case OK_NOIHAVE: */
-#	ifndef NO_POSTING
-			can_post = TRUE;
-#	endif /* !NO_POSTING */
+			can_post = TRUE && !force_no_post;
 			break;
 
 		case OK_NOPOST:
@@ -157,9 +152,7 @@ nntp_open(
 
 		default:
 			if (ret >= 200 && ret <= 299) {
-#	ifndef NO_POSTING
-				can_post = TRUE;
-#	endif /* !NO_POSTING */
+				can_post = TRUE && !force_no_post;
 				break;
 			}
 			if (ret < 0)
@@ -205,13 +198,10 @@ nntp_open(
 	 * which do return ERR_COMMAND if they are feed only servers.
 	 */
 
-	ret = get_respcode(line, sizeof(line));
-	switch (ret) {
+	switch ((ret = get_respcode(line, sizeof(line)))) {
 		case OK_CANPOST:
 /*		case OK_NOIHAVE: */
-#	ifndef NO_POSTING
-			can_post = TRUE;
-#	endif /* !NO_POSTING */
+			can_post = TRUE && !force_no_post;
 			sec = TRUE;
 			break;
 
@@ -256,13 +246,10 @@ nntp_open(
 #	endif /* DEBUG */
 		authenticate(nntp_server, userid, TRUE);
 		put_server("MODE READER");
-		ret = get_respcode(line, sizeof(line));
-		switch (ret) {
+		switch ((ret = get_respcode(line, sizeof(line)))) {
 			case OK_CANPOST:
 /*			case OK_NOIHAVE: */
-#	ifndef NO_POSTING
-				can_post = TRUE;
-#	endif /* !NO_POSTING */
+				can_post = TRUE && !force_no_post;
 				sec = TRUE;
 				break;
 
@@ -286,7 +273,7 @@ nntp_open(
 
 	if (!is_reconnect) {
 		/* Inform user if he cannot post */
-		if (!can_post)
+		if (!can_post && !batch_mode)
 			wait_message(0, "%s\n", _(txt_cannot_post));
 
 		/* Remove leading white space and save server's second response */
@@ -303,7 +290,7 @@ nntp_open(
 		 * TODO: This only breaks the line once, but the response could be
 		 * longer than two lines ...
 		 */
-		{
+		if (!batch_mode || verbose) {
 			char *chr1, *chr2;
 			int j;
 
@@ -326,28 +313,34 @@ nntp_open(
 	}
 
 	/*
-	 * Check if NNTP supports XOVER or OVER (successor of XOVER as of latest
-	 * NNTP Draft (Jan 2002) command; ie, we _don't_ get an ERR_COMMAND
+	 * Check if NNTP supports XOVER or OVER command (successor of XOVER as of
+	 * latest NNTP Draft (Jan 2002); ie, we _don't_ get an ERR_COMMAND
+	 * Could use (i=1;i>=0;i--) to give OVER higher priority than XOVER
 	 *
 	 * TODO: Don't try (X)OVER if listed in LIST EXTENSIONS.
 	 */
-
-	if (!nntp_command(txt_xover_string, ERR_COMMAND, NULL, 0)) {
-		xover_supported = TRUE;
-		txt_xover = txt_xover_string;
-		/* TODO issue warning if old index files found? */
-	} else {
-		if (!nntp_command(&txt_xover_string[1], ERR_COMMAND, NULL, 0)) {
-			xover_supported = TRUE;
-			txt_xover = &txt_xover_string[1];
-			/* TODO issue warning if old index files found? */
-		} else {
-			if (!is_reconnect)
-				wait_message(2, _(txt_no_xover_support));
-		}
+	for (i = 0; i < 2; i++) {
+		xover_cmd = &txt_xover_buff[i];
+		if (!nntp_command(xover_cmd, ERR_COMMAND, NULL, 0))
+			break;
 	}
 
-#	if 0 /* TODO */
+	if (i == 2)	{	/* ie XOVER and OVER gave ERR_COMMAND */
+		xover_cmd = NULL;
+		if (!is_reconnect && !batch_mode) {
+			wait_message(2, _(txt_no_xover_support));
+
+			if (tinrc.cache_overview_files)
+				wait_message(2, _(txt_caching_on));
+			else
+				wait_message(2, _(txt_caching_off));
+		}
+	} else {
+		/* TODO: issue warning if old index files found? */
+		/*		 in index_newsdir ? */
+	}
+
+#	if 0 /* TODO: */
 	/* if we're using -n, check for LIST NEWSGROUPS <wildmat> */
 	if (newsrc_active && !list_active) { /* -n */
 		/* code goes here */
@@ -399,9 +392,16 @@ get_only_respcode(
 
 	ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
-	if (tin_errno || ptr == NULL)
+	if (tin_errno || ptr == NULL) {
+#	ifdef DEBUG
+		debug_nntp("<<<", "Error: tin_error<>0 or ptr==NULL in get_only_respcode()");
+#	endif /* DEBUG */
 		return -1;
+	}
 
+#	ifdef DEBUG
+	debug_nntp("<<<", ptr);
+#	endif /* DEBUG */
 	respcode = (int) strtol(ptr, &end, 10);
 	DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
 
@@ -417,13 +417,20 @@ get_only_respcode(
 		put_server(last_put);
 		ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
-		if (tin_errno)
+		if (tin_errno) {
+#	ifdef DEBUG
+			debug_nntp("<<<", "Error: tin_errno <> 0");
+#	endif /* DEBUG */
 			return -1;
+		}
 
+#	ifdef DEBUG
+		debug_nntp("<<<", ptr);
+#	endif /* DEBUG */
 		respcode = (int) strtol(ptr, &end, 10);
 		DEBUG_IO((stderr, "get_only_respcode(%d)\n", respcode));
 	}
-	if (message != NULL)		/* Pass out the rest of the text */
+	if (message != NULL && mlen > 1)		/* Pass out the rest of the text */
 		my_strncpy(message, end, mlen - 1);
 
 #endif /* NNTP_ABLE */
@@ -459,7 +466,7 @@ get_respcode(
 #	ifdef DEBUG
 		debug_nntp("get_respcode", "authentication");
 #	endif /* DEBUG */
-		strncpy(savebuf, last_put, NNTP_STRLEN);		/* Take copy, as authenticate() will clobber this */
+		strncpy(savebuf, last_put, sizeof(savebuf) - 1);		/* Take copy, as authenticate() will clobber this */
 
 		if (authenticate(nntp_server, userid, FALSE)) {
 			strcpy(last_put, savebuf);
@@ -467,12 +474,19 @@ get_respcode(
 			put_server(last_put);
 			ptr = tin_fgets(FAKE_NNTP_FP, FALSE);
 
-			if (tin_errno)
+			if (tin_errno) {
+#	ifdef DEBUG
+				debug_nntp("<<<", "Error: tin_errno <> 0");
+#	endif /* DEBUG */
 				return -1;
+			}
 
+#	ifdef DEBUG
+			debug_nntp("<<<", ptr);
+#	endif /* DEBUG */
 			respcode = (int) strtol(ptr, &end, 10);
-			if (message != NULL)				/* Pass out the rest of the text */
-				strcpy(message, end);
+			if (message != NULL && mlen > 1)				/* Pass out the rest of the text */
+				strncpy(message, end, mlen - 1);
 
 		} else {
 			error_message(_(txt_auth_failed), ERR_ACCESS);
@@ -550,7 +564,7 @@ open_overview_fmt_fp(
 
 #ifdef NNTP_ABLE
 	if (read_news_via_nntp && !read_saved_news) {
-		if (!xover_supported)
+		if (!xover_cmd)
 			return (FILE *) 0;
 
 		sprintf(line, "LIST %s", OVERVIEW_FMT);
@@ -589,7 +603,7 @@ open_newgroups_fp(
 	 * compilant... we should switch over to ngtm->tm_year + 1900
 	 * after most of the server could handle the new format
 	 */
-		sprintf(line, "NEWGROUPS %02d%02d%02d %02d%02d%02d",
+		snprintf(line, sizeof(line), "NEWGROUPS %02d%02d%02d %02d%02d%02d",
 			ngtm->tm_year % 100, ngtm->tm_mon + 1, ngtm->tm_mday,
 			ngtm->tm_hour, ngtm->tm_min, ngtm->tm_sec);
 
@@ -605,8 +619,10 @@ open_newgroups_fp(
  */
 /* TODO: fixme/checkme
  *      - logic seems to be wrong, NNTP_ABLE && read_saved_news
- *        looks for a local subscriptions_file, but read_saved_news
- *        doesn't require a local server...
+ *        looks for a local subscriptions_file, but read_saved_news doesn't
+ *        require a local server... (a missing subscriptions_file doesn't
+ *        cause any trouble, we just have to bother with the read_saved_news
+ *        and a existing local subscriptions_file file case).
  *        open_newgroups_fp() uses the same logic.
  */
 FILE *
@@ -635,7 +651,7 @@ open_mail_active_fp(
 
 
 /*
- *  Open mail groups description file locally
+ * Open mail groups description file locally
  */
 FILE *
 open_mailgroups_fp(
@@ -668,7 +684,7 @@ open_newsgroups_fp(
 			}
 			read_local_newsgroups_file = FALSE;
 		}
-#	if 0 /* TODO */
+#	if 0 /* TODO: */
 		if (list_newsgroups_wildmat_supported && newsrc_active
 		    && !list_active
 		    && num_active < some_useful_limit) {
@@ -685,70 +701,6 @@ open_newsgroups_fp(
 }
 
 
-/*
- * Open a group NOV/XOVER file
- */
-FILE *
-open_xover_fp(
-	struct t_group *psGrp,
-	const char *mode,
-	long lMin,
-	long lMax)
-{
-#ifdef NNTP_ABLE
-	if (read_news_via_nntp && xover_supported && *mode == 'r' && psGrp->type == GROUP_TYPE_NEWS) {
-		char line[NNTP_STRLEN];
-
-		sprintf(line, "%s %ld-%ld", txt_xover, lMin, lMax);
-		return (nntp_command(line, OK_XOVER, NULL, 0));
-	} else
-#endif /* NNTP_ABLE */
-	{
-		char *pcNovFile;
-
-		pcNovFile = find_nov_file(psGrp, (*mode == 'r' ? R_OK : W_OK));
-#ifdef DEBUG
-		if (debug)
-			error_message("READ file=[%s]", pcNovFile);
-#endif /* DEBUG */
-		if (pcNovFile != NULL)
-			return fopen(pcNovFile, mode);
-
-		return (FILE *) 0;
-	}
-}
-
-
-/*
- * Stat a mail/news article to see if it still exists
- */
-t_bool
-stat_article(
-	long art,
-	const char *group_path)
-{
-	char buf[NNTP_STRLEN];
-	struct t_group currgrp;
-
-	currgrp = CURR_GROUP;
-
-#ifdef NNTP_ABLE
-	if (read_news_via_nntp && currgrp.type == GROUP_TYPE_NEWS) {
-		sprintf(buf, "STAT %ld", art);
-		return (nntp_command(buf, OK_NOTEXT, NULL, 0) != NULL);
-	} else
-#endif /* NNTP_ABLE */
-	{
-		struct stat sb;
-
-		joinpath(buf, currgrp.spooldir, group_path);
-		sprintf(&buf[strlen(buf)], "/%ld", art);
-
-		return (stat(buf, &sb) != -1);
-	}
-}
-
-
 /*
  * Open an article for reading just the header
  */
@@ -799,22 +751,28 @@ open_art_header(
  */
 FILE *
 open_art_fp(
-	const char *group_path,
+	struct t_group *group,
 	long art)
 {
-	char buf[NNTP_STRLEN];
 	FILE *art_fp = (FILE *) 0;
+	char buf[NNTP_STRLEN];
 
 #ifdef NNTP_ABLE
-	if (read_news_via_nntp && CURR_GROUP.type == GROUP_TYPE_NEWS) {
-		snprintf(buf, sizeof(buf) - 1, "ARTICLE %ld", art);
+	if (read_news_via_nntp && group->type == GROUP_TYPE_NEWS) {
+		snprintf(buf, sizeof(buf), "ARTICLE %ld", art);
 		art_fp = nntp_command(buf, OK_ARTICLE, NULL, 0);
 	} else {
 #endif /* NNTP_ABLE */
-		joinpath(buf, CURR_GROUP.spooldir, group_path);
-		sprintf(&buf[strlen(buf)], "/%ld", art);
+		char pbuf[PATH_LEN];
+		char fbuf[NAME_LEN + 1];
+		char group_path[PATH_LEN];
 
-		art_fp = fopen(buf, "r");
+		make_group_path(group->name, group_path);
+		joinpath(buf, group->spooldir, group_path);
+		snprintf(fbuf, sizeof(fbuf), "%ld", art);
+		joinpath(pbuf, buf, fbuf);
+
+		art_fp = fopen(pbuf, "r");
 #ifdef NNTP_ABLE
 	}
 #endif /* NNTP_ABLE */
@@ -863,8 +821,7 @@ base_comp(
  */
 long
 setup_hard_base(
-	struct t_group *group,
-	const char *group_path)
+	struct t_group *group)
 {
 	char buf[NNTP_STRLEN];
 	long art;
@@ -897,7 +854,7 @@ setup_hard_base(
 			char *ptr;
 
 #	ifdef DEBUG
-			debug_nntp("setup_base", buf);
+			debug_nntp("setup_hard_base", buf);
 #	endif /* DEBUG */
 
 			while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
@@ -950,7 +907,7 @@ setup_hard_base(
 		DIR *d;
 		DIR_BUF *e;
 
-		joinpath(buf, group->spooldir, group_path);
+		make_base_group_path(group->spooldir, group->name, buf);
 
 		if (access(buf, R_OK) != 0) {
 			error_message(_(txt_not_exist));
@@ -1059,7 +1016,7 @@ group_get_art_info(
 				return -ERR_NOGROUP;
 
 			case ERR_ACCESS:
-				error_message(cCRLF "%s", line);
+				error_message("%s%s", cCRLF, line);
 				tin_done(NNTP_ERROR_EXIT);
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -1072,7 +1029,7 @@ group_get_art_info(
 				return -1;
 		}
 #else
-		my_fprintf(stderr, _("Unreachable ?\n"));
+		my_fprintf(stderr, _("Unreachable?\n"));
 		return 0;
 #endif /* NNTP_ABLE */
 	} else {
@@ -1111,6 +1068,38 @@ group_get_art_info(
 }
 
 
+#if 0
+/*
+ * Stat a mail/news article to see if it still exists
+ */
+static t_bool
+stat_article(
+	long art,
+	const char *group_path)
+{
+	char buf[NNTP_STRLEN];
+	struct t_group currgrp;
+
+	currgrp = CURR_GROUP;
+
+#	ifdef NNTP_ABLE
+	if (read_news_via_nntp && currgrp.type == GROUP_TYPE_NEWS) {
+		sprintf(buf, "STAT %ld", art);
+		return (nntp_command(buf, OK_NOTEXT, NULL, 0) != NULL);
+	} else
+#	endif /* NNTP_ABLE */
+	{
+		struct stat sb;
+
+		joinpath(buf, currgrp.spooldir, group_path);
+		sprintf(&buf[strlen(buf)], "/%ld", art);
+
+		return (stat(buf, &sb) != -1);
+	}
+}
+#endif /* 0 */
+
+
 /* This will come in useful for filtering on non-overview hdr fields */
 #if 0
 static FILE *
