@@ -3,7 +3,7 @@
  *  Module    : prompt.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-05
+ *  Updated   : 2003-02-08
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -78,7 +78,7 @@ prompt_num(
  *  prompt_string
  *  get a string from the user
  *  Return TRUE if a valid string was typed, FALSE otherwise
- *  TODO no bounds checking on buf size, tin_getline() defaults to 1024
+ *  TODO: no bounds checking on buf size, tin_getline() defaults to 1024
  */
 t_bool
 prompt_string(
@@ -155,7 +155,7 @@ prompt_menu_string(
  * prompt_yn
  * prompt user for 'y'es or 'n'o decision. "prompt" will be displayed in line
  * "line" giving the default answer "default_answer".
- * TODO 'line' is constant - can we remove it ?
+ * TODO: 'line' is constant - can we remove it ?
  * The function returns 1 if the user decided "yes", -1 if the user wanted
  * to escape, or 0 for any other key or decision.
  */
@@ -319,7 +319,7 @@ prompt_on_off(
 {
 	t_bool ret;
 
-	ret = prompt_list(row, col, (int) *var, help_text, _(prompt_text), txt_onoff, 2);
+	ret = prompt_list(row, col, (int) *var, help_text, _(prompt_text), txt_onoff, 2) ? TRUE : FALSE;
 	*var = (ret != 0);
 }
 
@@ -409,7 +409,7 @@ prompt_option_char(
 			return FALSE;
 		}
 		if (!*p)
-			info_message(_("Please enter a valid character")); /* TODO: Use move to lang.c */
+			info_message(_(txt_info_enter_valid_character));
 	} while (!*p);
 
 	*variable = p[0];
@@ -509,7 +509,8 @@ prompt_msgid(
 /*
  * Format a message such that it'll fit within the screen width
  * Useful for fitting long Subjects and newsgroup names into prompts
- * TODO - maybe add a '...' to the string to show it was truncated. See center_line()
+ * TODO: maybe add a '...' to the string to show it was truncated.
+ *       See center_line()
  */
 char *
 sized_message(
@@ -545,7 +546,7 @@ prompt_slk_response(
 	char buf[LEN];
 
 	va_start(ap, fmt);
-	vsnprintf(buf, sizeof(buf) - 1, fmt, ap);	/* We need to do this, else wait_message() will clobber us */
+	vsnprintf(buf, sizeof(buf), fmt, ap);	/* We need to do this, else wait_message() will clobber us */
 	va_end(ap);
 
 	ch_default = map_to_local(ch_default, responses);
@@ -581,9 +582,8 @@ prompt_continue(
 	cmd_line = TRUE;
 #endif /* USE_CURSES */
 	info_message(_(txt_return_key));
-	ch = ReadCh();
 
-	switch (ch) {
+	switch ((ch = ReadCh())) {
 		case ESC:
 #	ifdef HAVE_KEY_PREFIX
 		case KEY_PREFIX:
