@@ -1,12 +1,12 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : rfc1524.c
- *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@radar.tele2.co.uk>
+ *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-05-15
  *  Updated   : 2002-06-18
  *  Notes     : mailcap parsing as defined in RFC 1524
  *
- * Copyright (c) 2000-2003 Urs Janssen <urs@tin.org>, Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2003 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -157,7 +157,6 @@ parse_mailcap_line(
 {
 	char *ptr, *optr, *buf;
 	int i = MAILCAPFIELDS - 2; /* max MAILCAPFIELDS - required fileds */
-	size_t blen;
 	t_mailcap *tmailcap;
 
 	/* malloc and init */
@@ -180,8 +179,7 @@ parse_mailcap_line(
 
 	/* get required entrys */
 	ptr = get_mailcap_field(ptr);
-	blen = strlen(content_types[part->type]) + strlen(part->subtype) + 2;
-	buf = my_calloc(1, blen);
+	buf = my_calloc(1, strlen(content_types[part->type]) + strlen(part->subtype) + 2);
 	sprintf(buf, "%s/%s", content_types[part->type], part->subtype);
 	tmailcap->type = buf;
 	ptr += strlen(ptr) + 1;
@@ -267,8 +265,10 @@ parse_mailcap_line(
 	free(optr);
 
 	if (tmailcap->test != NULL) { /* test field given */
-		/* TODO: EndWin()/InitWin() around system needed? */
-		/* TODO: use invoke_cmd()? */
+		/*
+		 * TODO: EndWin()/InitWin() around system needed?
+		 *       use invoke_cmd()?
+		 */
 		if (system(tmailcap->test)) { /* test failed? */
 			free_mailcap(tmailcap);
 			return ((t_mailcap *) 0);
@@ -326,7 +326,7 @@ get_mailcap_field(
 
 
 #define CHECK_SPACE(minlen) { \
-	while (space <= (minlen)) { /* need more space ? */ \
+	while (space <= (minlen)) { /* need more space? */ \
 		olen = strlen(line); \
 		space += linelen; \
 		linelen <<= 1; \
@@ -419,7 +419,7 @@ expand_mailcap_meta(
 				}
 				/* FALLTHROUGH */
 
-#if 0 /* TODO */
+#if 0 /* TODO: */
 			case 'F':
 				if (percent) {
 					percent = FALSE;
