@@ -1,12 +1,12 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : rfc2046.c
- *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
+ *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-02-18
- *  Updated   : 2002-06-09
+ *  Updated   : 2003-06-09
  *  Notes     : RFC 2046 MIME article parsing
  *
- * Copyright (c) 2000-2003 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2003 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,8 +51,7 @@ static int count_lines(char *line);
 static int parse_multipart_article(FILE *infile, t_openartinfo *artinfo, t_part *part, int depth, t_bool show_progress_meter);
 static int parse_normal_article(FILE *in, t_openartinfo *artinfo, t_bool show_progress_meter);
 static int parse_rfc2045_article(FILE *infile, int line_count, t_openartinfo *artinfo, t_bool show_progress_meter);
-static unsigned parse_content_encoding(const char *encoding);
-static void add_persist(struct t_header *hdr, const char *p_header, char *p_content);
+static unsigned int parse_content_encoding(const char *encoding);
 static void free_list(t_param *list);
 static void parse_content_type(char *type, t_part *content);
 static void parse_content_disposition(char *disp, t_part *part);
@@ -68,7 +67,7 @@ static void progress(int line_count);
  */
 static int art_lines = 0;		/* lines in art on spool */
 
-#define PARAM_SEP		"; \n"
+#define PARAM_SEP	"; \n"
 /* default parameters for Content-Type */
 #define CT_DEFPARMS	"charset=US-ASCII"
 
@@ -114,8 +113,9 @@ boundary_cmp(
 	const char *line,
 	const char *boundary)
 {
-	int blen = strlen(boundary);
-	int len, nl;
+	size_t blen = strlen(boundary);
+	size_t len;
+	int nl;
 
 	if ((len = strlen(line)) == 0)
 		return BOUND_NONE;
@@ -148,10 +148,35 @@ boundary_cmp(
 }
 
 
+/*
+ * RFC2046 5.1.2 says that we are required to check for all possible
+ * boundaries, not only the one that is expected. Iterate through all
+ * the parts.
+ */
+static int
+boundary_check(
+	const char *line,
+	t_part *part)
+{
+	const char *boundary;
+	int bnd = BOUND_NONE;
+
+	for (; part != NULL; part = part->next) {
+		/* We may not have even parsed a boundary for this part yet */
+		if ((boundary = get_param(part->params, "boundary")) == NULL)
+			continue;
+		if ((bnd = boundary_cmp(line, boundary)) != BOUND_NONE)
+			break;
+	}
+
+	return bnd;
+}
+
+
 /*
  * Parse a Content-* parameter list into a linked list
  * Ensure the ->params element is correctly initialised before calling
- * TODO may still not catch everything permitted in the RFC
+ * TODO: may still not catch everything permitted in the RFC
  */
 static void
 parse_params(
@@ -176,7 +201,7 @@ parse_params(
 			eql++;
 			*param = '\0';
 		}
-		ptr->value= my_strdup(rfc1522_decode(eql));
+		ptr->value = my_strdup(rfc1522_decode(eql));
 		ptr->next = content->params;		/* Push onto start of list */
 		content->params = ptr;
 	}
@@ -219,7 +244,7 @@ get_param(
 
 
 /*
- * Split a Content-Type header into a t_content structure
+ * Split a Content-Type header into a t_part structure
  */
 static void
 parse_content_type(
@@ -271,11 +296,11 @@ parse_content_type(
 #ifndef CHARSET_CONVERSION
 			parse_params(defparms, content);
 #else
-			if (CURR_GROUP.attribute->undeclared_charset) {
+			if (curr_group->attribute->undeclared_charset) {
 				char *charsetheader;
 
-				charsetheader = my_malloc(strlen(CURR_GROUP.attribute->undeclared_charset) + 9); /* 9=len('charset=\0') */
-				sprintf(charsetheader, "charset=%s", CURR_GROUP.attribute->undeclared_charset);
+				charsetheader = my_malloc(strlen(curr_group->attribute->undeclared_charset) + 9); /* 9=len('charset=\0') */
+				sprintf(charsetheader, "charset=%s", curr_group->attribute->undeclared_charset);
 				parse_params(charsetheader, content);
 				free(charsetheader);
 			} else {
@@ -289,11 +314,11 @@ parse_content_type(
 }
 
 
-static unsigned
+static unsigned int
 parse_content_encoding(
 	const char *encoding)
 {
-	int i;
+	unsigned int i;
 
 	for (i = 0; i < NUM_ENCODINGS; ++i) {
 		if (strcasecmp(encoding, content_encodings[i]) == 0)
@@ -333,24 +358,6 @@ parse_content_disposition(
 }
 
 
-static void
-add_persist(
-	struct t_header *hdr,
-	const char *p_header,
-	char *p_content)
-{
-	t_param *ptr;
-
-	ptr = my_malloc(sizeof(t_param));
-	ptr->name = my_strdup(p_header);
-	ptr->value = my_strdup(rfc1522_decode(str_trim(p_content)));
-	ptr->next = hdr->persist;
-	hdr->persist = ptr;
-
-	return;
-}
-
-
 /*
  * Return a freshly allocated and initialised part structure attached to the
  * end of the list of article parts given
@@ -367,17 +374,18 @@ new_part(
 
 	ptr->type = TYPE_TEXT;					/* Defaults per RFC */
 	ptr->subtype = my_strdup("plain");
+	ptr->description = NULL;
 	ptr->encoding = ENCODING_7BIT;
 	ptr->params = NULL;
 
 #ifndef CHARSET_CONVERSION
 	parse_params(defparms, ptr);
 #else
-	if (CURR_GROUP.attribute->undeclared_charset) {
+	if (curr_group && curr_group->attribute->undeclared_charset) {
 		char *charsetheader;
 
-		charsetheader = my_malloc(strlen(CURR_GROUP.attribute->undeclared_charset) + 9); /* 9=len('charset=\0') */
-		sprintf(charsetheader, "charset=%s", CURR_GROUP.attribute->undeclared_charset);
+		charsetheader = my_malloc(strlen(curr_group->attribute->undeclared_charset) + 9); /* 9=len('charset=\0') */
+		sprintf(charsetheader, "charset=%s", curr_group->attribute->undeclared_charset);
 		parse_params(charsetheader, ptr);
 		free(charsetheader);
 	} else {
@@ -406,6 +414,8 @@ new_part(
 
 /*
  * Free a linked list of t_part
+ *
+ * TODO: plug mem leak: ptr->uue is not entirely freed
  */
 void
 free_parts(
@@ -417,6 +427,7 @@ free_parts(
 	}
 
 	free(ptr->subtype);
+	FreeAndNull(ptr->description);
 	if (ptr->params)
 		free_list(ptr->params);
 	if (ptr->uue)
@@ -446,13 +457,11 @@ free_and_init_header(
 	FreeAndNull(hdr->summary);
 	FreeAndNull(hdr->followup);
 	FreeAndNull(hdr->ftnto);
-	FreeAndNull(hdr->authorids);
+#ifdef XFACE_ABLE
+	FreeAndNull(hdr->xface);
+#endif /* XFACE_ABLE */
 	hdr->mime = FALSE;
 
-	if (hdr->persist)
-		free_list(hdr->persist);
-	hdr->persist = NULL;
-
 	if (hdr->ext)
 		free_parts(hdr->ext);
 	hdr->ext = NULL;
@@ -460,8 +469,11 @@ free_and_init_header(
 
 
 /*
- * buf:  Article header
- * pat:  Text to match in header
+ * buf:         Article header
+ * pat:         Text to match in header
+ * decode:      RFC2047-decode the header
+ * structured:  extract address-part before decoding the header
+ *
  * Returns:
  *	(decoded) body of header if matched or NULL
  */
@@ -469,7 +481,8 @@ char *
 parse_header(
 	char *buf,
 	const char *pat,
-	t_bool decode)
+	t_bool decode,
+	t_bool structured)
 {
 	size_t plen = strlen(pat);
 	char *ptr = buf + plen;
@@ -492,8 +505,25 @@ parse_header(
 	if (!*ptr)
 		return ptr;
 
-	if (decode)
-		return (rfc1522_decode(ptr));
+	if (decode) {
+		if (structured) {
+			char addr[HEADER_LEN];
+			char name[HEADER_LEN];
+			int type;
+
+			if (gnksa_split_from(ptr, addr, name, &type) == GNKSA_OK) {
+				if (*name) {
+					if (type == GNKSA_ADDRTYPE_OLDSTYLE)
+						sprintf(ptr, "%s (%s)", addr, rfc1522_decode(name));
+					else
+						sprintf(ptr, "%s <%s>", rfc1522_decode(name), addr);
+				} else
+					sprintf(ptr, "%s", addr);
+			} else
+				return ptr;
+		} else
+			return (rfc1522_decode(ptr));
+	}
 
 	return ptr;
 }
@@ -532,103 +562,117 @@ parse_rfc822_headers(
 		}
 
 		/*
-		 * FIXME: multiple headers of the same name could lead to memory leak
-		 * and loss of information (multiple Cc: lines are allowed, for example)
+		 * FIXME: multiple headers of the same name could lead to information
+		 *        loss (multiple Cc: lines are allowed, for example)
 		 */
 		unfold_header(line);
-		if ((ptr = parse_header(line, "From", TRUE))) {
+		if ((ptr = parse_header(line, "From", TRUE, TRUE))) {
+			FreeIfNeeded(hdr->from);
 			hdr->from = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "To", TRUE))) {
+		if ((ptr = parse_header(line, "To", TRUE, TRUE))) {
+			FreeIfNeeded(hdr->to);
 			hdr->to = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Cc", TRUE))) {
+		if ((ptr = parse_header(line, "Cc", TRUE, TRUE))) {
+			FreeIfNeeded(hdr->cc);
 			hdr->cc = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Bcc", TRUE))) {
+		if ((ptr = parse_header(line, "Bcc", TRUE, TRUE))) {
+			FreeIfNeeded(hdr->bcc);
 			hdr->bcc = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Date", FALSE))) {
+		if ((ptr = parse_header(line, "Date", FALSE, FALSE))) {
+			FreeIfNeeded(hdr->date);
 			hdr->date = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Subject", TRUE))) {
+		if ((ptr = parse_header(line, "Subject", TRUE, FALSE))) {
+			FreeIfNeeded(hdr->subj);
 			hdr->subj = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Organization", TRUE))) {
+		if ((ptr = parse_header(line, "Organization", TRUE, FALSE))) {
+			FreeIfNeeded(hdr->org);
 			hdr->org = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Reply-To", TRUE))) {
+		if ((ptr = parse_header(line, "Reply-To", TRUE, TRUE))) {
+			FreeIfNeeded(hdr->replyto);
 			hdr->replyto = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Newsgroups", FALSE))) {
+		if ((ptr = parse_header(line, "Newsgroups", FALSE, FALSE))) {
+			FreeIfNeeded(hdr->newsgroups);
 			hdr->newsgroups = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Message-ID", FALSE))) {
+		if ((ptr = parse_header(line, "Message-ID", FALSE, FALSE))) {
+			FreeIfNeeded(hdr->messageid);
 			hdr->messageid = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "References", FALSE))) {
+		if ((ptr = parse_header(line, "References", FALSE, FALSE))) {
+			FreeIfNeeded(hdr->references);
 			hdr->references = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Distribution", TRUE))) {
+		if ((ptr = parse_header(line, "Distribution", TRUE, FALSE))) {
+			FreeIfNeeded(hdr->distrib);
 			hdr->distrib = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Keywords", TRUE))) {
+		if ((ptr = parse_header(line, "Keywords", TRUE, FALSE))) {
+			FreeIfNeeded(hdr->keywords);
 			hdr->keywords = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Summary", TRUE))) {
+		if ((ptr = parse_header(line, "Summary", TRUE, FALSE))) {
+			FreeIfNeeded(hdr->summary);
 			hdr->summary = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Followup-To", FALSE))) {
+		if ((ptr = parse_header(line, "Followup-To", FALSE, FALSE))) {
+			FreeIfNeeded(hdr->followup);
 			hdr->followup = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "X-Comment-To", TRUE))) {
+		if ((ptr = parse_header(line, "X-Comment-To", TRUE, TRUE))) {
+			FreeIfNeeded(hdr->ftnto);
 			hdr->ftnto = my_strdup(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Author-IDs", TRUE)) ||
-			(ptr = parse_header(line, "P-Author-IDs", TRUE)) ||
-			(ptr = parse_header(line, "X-P-Author-IDs", TRUE))) {
-			hdr->authorids = my_strdup(ptr);
+#ifdef XFACE_ABLE
+		if ((ptr = parse_header(line, "X-Face", FALSE, FALSE))) {
+			FreeIfNeeded(hdr->xface);
+			hdr->xface = my_strdup(ptr);
 			continue;
 		}
+#endif /* XFACE_ABLE */
 		/* TODO: check version */
-		if (parse_header(line, "MIME-Version", FALSE)) {
+		if (parse_header(line, "MIME-Version", FALSE, FALSE)) {
 			hdr->mime = TRUE;
 			continue;
 		}
-		if ((ptr = parse_header(line, "Content-Type", FALSE))) {
+		if ((ptr = parse_header(line, "Content-Type", FALSE, FALSE))) {
 			parse_content_type(ptr, hdr->ext);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Content-Transfer-Encoding", FALSE))) {
+		if ((ptr = parse_header(line, "Content-Transfer-Encoding", FALSE, FALSE))) {
 			hdr->ext->encoding = parse_content_encoding(ptr);
 			continue;
 		}
-		if ((ptr = parse_header(line, "Content-Disposition", FALSE))) {
-			parse_content_disposition(ptr, hdr->ext);
+		if ((ptr = parse_header(line, "Content-Description", TRUE, FALSE))) {
+			FreeIfNeeded(hdr->ext->description);
+			hdr->ext->description = my_strdup(ptr);
 			continue;
 		}
-		/*
-		 * Persistent headers
-		 */
-		if ((strncmp(line, "P-", 2) == 0 || strncmp(line, "X-P-", 4) == 0) && (ptr = strstr(line, ": "))) {
-			*ptr = '\0';
-			add_persist(hdr, line, ptr + 2);
+		if ((ptr = parse_header(line, "Content-Disposition", FALSE, FALSE))) {
+			parse_content_disposition(ptr, hdr->ext);
 			continue;
 		}
 	}
@@ -678,12 +722,15 @@ unfold_header(
 #define M_HDR		2	/* In MIME headers */
 #define M_BODY		3	/* In MIME body */
 
+#define TIN_EOF		0xf00	/* Used internally for error recovery */
+
 /*
  * Handles multipart/ article types, write data to a raw stream
  * artinfo is used for generic article pointers
  * part contains content info about the attachment we're parsing
  * depth is the number of levels by which the current part is embedded
- * Returns a tin_errno value
+ * Returns a tin_errno value which is '&'ed with TIN_EOF if the end of the
+ * article is reached (to prevent broken articles from hanging the NNTP socket)
  */
 static int
 parse_multipart_article(
@@ -695,21 +742,17 @@ parse_multipart_article(
 {
 	char *line;
 	char *ptr;
-	const char *boundary;
 	int bnd;
 	int state = M_SEARCHING;
 	t_part *curr_part = 0;
 
-	/*
-	 * Get the boundary marker
-	 */
-	if ((boundary = get_param(part->params, "boundary")) == NULL)
-		return -1;
-
 	while ((line = tin_fgets(infile, (state == M_HDR))) != NULL) {
-		bnd = boundary_cmp(line, boundary);
+/* fprintf(stderr, "%d---:%s\n", depth, line); */
 
-/* fprintf(stderr, "---:%s\n", line); */
+		/*
+		 * Check current line for boundary markers
+		 */
+		bnd = boundary_check(line, artinfo->hdr.ext);
 
 		fprintf(artinfo->raw, "%s\n", line);
 
@@ -718,14 +761,13 @@ parse_multipart_article(
 			progress(artinfo->hdr.ext->line_count);		/* Overall line count */
 
 		if (bnd == BOUND_END) {							/* End of this part detected */
-#ifdef NNTP_ABLE
 			/*
-			 * We have reached the end boundary of the outermost envelope.
-			 * Syphon off any trailing data.
+			 * When we have reached the end boundary of the outermost envelope
+			 * just log any trailing data for the raw article format.
 			 */
 			if (depth == 0)
-				drain_buffer(infile);
-#endif /* NNTP_ABLE */
+				while ((line = tin_fgets(infile, FALSE)) != NULL)
+					fprintf(artinfo->raw, "%s\n", line);
 			return tin_errno;
 		}
 
@@ -746,7 +788,7 @@ parse_multipart_article(
 			case M_HDR:
 				switch (bnd) {
 					case BOUND_START:
-						fprintf(stderr, "MIME parse error: Start boundary whilst reading headers\n");
+						error_message(_(txt_error_mime_start));
 						continue;
 
 					case BOUND_NONE:
@@ -763,26 +805,32 @@ parse_multipart_article(
 				 * Keep headers that interest us
 				 */
 /*fprintf(stderr, "HDR:%s\n", line);*/
-				if ((ptr = parse_header(line, "Content-Type", FALSE))) {
+				unfold_header(line);
+				if ((ptr = parse_header(line, "Content-Type", FALSE, FALSE))) {
 					parse_content_type(ptr, curr_part);
 
-					if (curr_part->type == TYPE_MULTIPART) {	/* Complex mutlipart article */
+					if (curr_part->type == TYPE_MULTIPART) {	/* Complex multipart article */
 						int ret;
 
 						if ((ret = parse_multipart_article(infile, artinfo, curr_part, depth + 1, show_progress_meter)) != 0)
-							return ret;
+							return ret;							/* User abort or EOF reached */
 						else
 							break;
 					}
 				}
-				if ((ptr = parse_header(line, "Content-Transfer-Encoding", FALSE))) {
+				if ((ptr = parse_header(line, "Content-Transfer-Encoding", FALSE, FALSE))) {
 					curr_part->encoding = parse_content_encoding(ptr);
 					break;
 				}
-				if ((ptr = parse_header(line, "Content-Disposition", FALSE))) {
+				if ((ptr = parse_header(line, "Content-Disposition", FALSE, FALSE))) {
 					parse_content_disposition(ptr, curr_part);
 					break;
 				}
+				if ((ptr = parse_header(line, "Content-Description", TRUE, FALSE))) {
+					FreeIfNeeded(curr_part->description);
+					curr_part->description = my_strdup(ptr);
+					break;
+				}
 				break;
 
 			case M_BODY:
@@ -800,12 +848,13 @@ parse_multipart_article(
 				}
 				break;
 		} /* switch (state) */
-	}
+	} /* while() */
 
 	/*
-	 * We only reach this point when we reach the end of the article
+	 * We only reach this point when we (unexpectedly) reach the end of the
+	 * article
 	 */
-	return tin_errno;
+	return tin_errno | TIN_EOF;		/* Flag EOF */
 }
 
 
@@ -832,6 +881,26 @@ parse_normal_article(
 
 #ifdef DEBUG_ART
 /* DEBUG dump of what we got */
+static void
+dump_uue(
+	t_part *ptr,
+	t_openartinfo *art)
+{
+	if (ptr->uue != NULL) {
+		t_part *uu;
+		for (uu = ptr->uue; uu != NULL; uu = uu->next) {
+			fprintf(stderr, "UU: %s\n", get_param(uu->params, "name"));
+			fprintf(stderr, "    Content-Type: %s/%s\n    Content-Transfer-Encoding: %s\n",
+				content_types[uu->type], uu->subtype,
+				content_encodings[uu->encoding]);
+			fprintf(stderr, "    Offset: %ld  Lines: %d\n", uu->offset, uu->line_count);
+			fprintf(stderr, "    Depth: %d\n", uu->depth);
+			fseek(art->raw, uu->offset, SEEK_SET);
+			fprintf(stderr, "[%s]\n\n", tin_fgets(art->raw, FALSE));
+		}
+	}
+}
+
 static void
 dump_art(
 	t_openartinfo *art)
@@ -844,31 +913,28 @@ dump_art(
 	fprintf(stderr, "Content-Type: %s/%s\nContent-Transfer-Encoding: %s\n",
 		content_types[note_h.ext->type], note_h.ext->subtype,
 		content_encodings[note_h.ext->encoding]);
+	if (note_h.ext->description)
+		fprintf(stderr, "Content-Description: %s\n", note_h.ext->description);
 	fprintf(stderr, "Offset: %ld\nLines: %d\n", note_h.ext->offset, note_h.ext->line_count);
 	for (pptr = note_h.ext->params; pptr != NULL; pptr = pptr->next)
 		fprintf(stderr, "P: %s = %s\n", pptr->name, pptr->value);
-	if (note_h.ext->uue != NULL) {
-		t_part *uu;
-		for (uu = note_h.ext->uue; uu != NULL; uu = uu->next) {
-			fprintf(stderr, "UU: %s\n", get_param(uu->params, "name"));
-			fprintf(stderr, "    Offset: %ld  Lines: %d\n", uu->offset, uu->line_count);
-			fseek(art->raw, uu->offset, SEEK_SET);
-			fprintf(stderr, "[%s]\n\n", tin_fgets(art->raw, FALSE));
-		}
-	}
+	dump_uue(note_h.ext, art);
 	fseek(art->raw, note_h.ext->offset, SEEK_SET);
 	fprintf(stderr, "[%s]\n\n", tin_fgets(art->raw, FALSE));
 	fprintf(stderr, "\n");
 
 	for (ptr = note_h.ext->next; ptr != NULL; ptr = ptr->next) {
 		fprintf(stderr, "Attachment:\n");
-		fprintf(stderr, "	Content-Type: %s/%s\n	Content-Transfer-Encoding: %s\n",
+		fprintf(stderr, "\tContent-Type: %s/%s\n\tContent-Transfer-Encoding: %s\n",
 			content_types[ptr->type], ptr->subtype,
 			content_encodings[ptr->encoding]);
-		fprintf(stderr, "	Offset: %ld\n	Lines: %d\n", ptr->offset, ptr->line_count);
-		fprintf(stderr, "	Depth: %d\n", ptr->depth);
+		if (ptr->description)
+			fprintf(stderr, "\tContent-Description: %s\n", ptr->description);
+		fprintf(stderr, "\tOffset: %ld\n\tLines: %d\n", ptr->offset, ptr->line_count);
+		fprintf(stderr, "\tDepth: %d\n", ptr->depth);
 		for (pptr = ptr->params; pptr != NULL; pptr = pptr->next)
-			fprintf(stderr, "	P: %s = %s\n", pptr->name, pptr->value);
+			fprintf(stderr, "\tP: %s = %s\n", pptr->name, pptr->value);
+		dump_uue(ptr, art);
 		fseek(art->raw, ptr->offset, SEEK_SET);
 		fprintf(stderr, "[%s]\n\n", tin_fgets(art->raw, FALSE));
 	}
@@ -902,8 +968,16 @@ parse_rfc2045_article(
 	 * We don't bother to parse all plain text articles
 	 */
 	if (artinfo->hdr.mime && artinfo->hdr.ext->type == TYPE_MULTIPART) {
-		if ((ret = parse_multipart_article(infile, artinfo, artinfo->hdr.ext, 0, show_progress_meter)) != 0)
-			goto error;
+		if ((ret = parse_multipart_article(infile, artinfo, artinfo->hdr.ext, 0, show_progress_meter)) != 0) {
+			/* Strip off EOF condition if present */
+			if (ret & TIN_EOF) {
+				ret ^= TIN_EOF;
+				error_message(_(txt_error_mime_end), content_types[artinfo->hdr.ext->type], artinfo->hdr.ext->subtype);
+				if (ret != 0)
+					goto error;
+			} else
+				goto error;
+		}
 	} else {
 		if ((ret = parse_normal_article(infile, artinfo, show_progress_meter)) != 0)
 			goto error;
@@ -936,14 +1010,15 @@ int
 art_open(
 	t_bool wrap_lines,
 	struct t_article *art,
-	const char *group_path,
+	struct t_group *group,
 	t_openartinfo *artinfo,
 	t_bool show_progress_meter)
 {
-	char *ptr;
 	FILE *fp;
 
-	if ((fp = open_art_fp(group_path, art->artnum)) == NULL)
+	memset(artinfo, 0, sizeof(t_openartinfo));
+
+	if ((fp = open_art_fp(group, art->artnum)) == NULL)
 		return ((tin_errno == 0) ? ART_UNAVAILABLE : ART_ABORT);
 
 #ifdef DEBUG_ART
@@ -953,7 +1028,12 @@ art_open(
 	if (parse_rfc2045_article(fp, art->line_count, artinfo, show_progress_meter) != 0)
 		return ART_ABORT;
 
-	if ((pgart.tex2iso = ((CURR_GROUP.attribute->tex2iso_conv) ? is_art_tex_encoded(artinfo->raw) : FALSE)))
+	/*
+	 * TODO: compare art->msgid and artinfo->hdr.messageid and issue a
+	 *       warinng (once) about broken overviews if they differ
+	 */
+
+	if ((artinfo->tex2iso = ((group->attribute->tex2iso_conv) ? is_art_tex_encoded(artinfo->raw) : FALSE)))
 		wait_message(0, _(txt_is_tex_encoded));
 
 	/* Maybe fix it so if this fails, we default to raw? */
@@ -966,12 +1046,10 @@ art_open(
 
 	/*
 	 * If Newsgroups is empty its a good bet the article is a mail article
+	 * TODO: Why do this ?
 	 */
-	if (!artinfo->hdr.newsgroups) {
-		artinfo->hdr.newsgroups = my_strdup(group_path);
-		while ((ptr = strchr(artinfo->hdr.newsgroups, '/')))
-			*ptr = '.';		/* TODO - combine with code to fixup Archive-name? */
-	}
+	if (!artinfo->hdr.newsgroups)
+		artinfo->hdr.newsgroups = my_strdup(group->name);
 
 	return 0;
 }
@@ -988,6 +1066,9 @@ art_close(
 	fprintf(stderr, "art_close(%p)\n", (void *) artinfo);
 #endif /* DEBUG_ART */
 
+	if (artinfo == NULL)
+		return;
+
 	free_and_init_header(&artinfo->hdr);
 
 	artinfo->tex2iso = FALSE;
